@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,15 +10,9 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Classes to manage tool instance execution in various forms.
-
-:author: P. Savolainen (VTT)
-:date:   1.2.2018
-"""
-
+"""Classes to manage tool instance execution in various forms."""
 import logging
-from PySide6.QtCore import QObject, QProcess, Slot, Signal
+from PySide6.QtCore import QObject, QProcess, Signal, Slot
 
 
 class ExecutionManager(QObject):
@@ -52,8 +47,7 @@ class QProcessExecutionManager(ExecutionManager):
     """Class to manage tool instance execution using a PySide6 QProcess."""
 
     def __init__(self, logger, program="", args=None, silent=False, semisilent=False):
-        """Class constructor.
-
+        """
         Args:
             logger (LoggerInterface): a logger instance
             program (str): Path to program to run in the subprocess (e.g. julia.exe)
@@ -142,9 +136,9 @@ class QProcessExecutionManager(ExecutionManager):
             new_state (int): Process state number (``QProcess::ProcessState``)
         """
         if new_state == QProcess.Starting:
-            self._logger.msg.emit("\tStarting program <b>{0}</b>".format(self._program))
+            self._logger.msg.emit(f"\tStarting program <b>{self._program}</b>")
             arg_str = " ".join(self._args)
-            self._logger.msg.emit("\tArguments: <b>{0}</b>".format(arg_str))
+            self._logger.msg.emit(f"\tArguments: <b>{arg_str}</b>")
         elif new_state == QProcess.Running:
             self._logger.msg_warning.emit("\tExecution in progress...")
         elif new_state == QProcess.NotRunning:
@@ -179,15 +173,13 @@ class QProcessExecutionManager(ExecutionManager):
         elif process_error == QProcess.UnknownError:
             self._logger.msg_error.emit("Unknown error in process")
         else:
-            self._logger.msg_error.emit("Unspecified error in process: {0}".format(process_error))
+            self._logger.msg_error.emit(f"Unspecified error in process: {process_error}")
         self.teardown_process()
 
     def teardown_process(self):
         """Tears down the QProcess in case a QProcess.ProcessError occurred.
         Emits execution_finished signal."""
-        if not self._process:
-            pass
-        else:
+        if self._process:
             out = str(self._process.readAllStandardOutput().data(), "utf-8", errors="replace")
             errout = str(self._process.readAllStandardError().data(), "utf-8", errors="replace")
             if out is not None:
@@ -210,7 +202,7 @@ class QProcessExecutionManager(ExecutionManager):
                 self._process.finished.emit(-1, -1)
                 self._process.deleteLater()
         except Exception as ex:  # pylint: disable=broad-except
-            self._logger.msg_error.emit("[{0}] exception when terminating process".format(ex))
+            self._logger.msg_error.emit(f"[{ex}] exception when terminating process")
             logging.exception("Exception in closing QProcess: %s", ex)
         finally:
             self._process = None
@@ -233,7 +225,7 @@ class QProcessExecutionManager(ExecutionManager):
             pass
         else:
             if not self._silent:
-                self._logger.msg_error.emit("Unknown QProcess exit status [{0}]".format(exit_status))
+                self._logger.msg_error.emit(f"Unknown QProcess exit status [{exit_status}]")
             exit_code = -1
         if exit_code != 0:
             self.process_failed = True
