@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,29 +10,24 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Classes for drawing graphics items on QGraphicsScene.
-
-:authors: M. Marin (KTH), P. Savolainen (VTT)
-:date:    4.4.2018
-"""
-
+"""Classes for drawing graphics items on QGraphicsScene."""
 import functools
-from math import sin, cos, pi, radians
-from PySide6.QtCore import Qt, Slot, QPointF, QLineF, QRectF, QVariantAnimation
+from math import cos, pi, radians, sin
+from PySide6.QtCore import QLineF, QPointF, QRectF, Qt, QVariantAnimation, Slot
+from PySide6.QtGui import QBrush, QColor, QCursor, QFont, QLinearGradient, QPainterPath, QPainterPathStroker, QPen
+from PySide6.QtSvg import QSvgRenderer
+from PySide6.QtSvgWidgets import QGraphicsSvgItem
 from PySide6.QtWidgets import (
+    QGraphicsColorizeEffect,
+    QGraphicsEllipseItem,
     QGraphicsItem,
     QGraphicsPathItem,
     QGraphicsTextItem,
-    QGraphicsEllipseItem,
     QStyle,
     QToolTip,
-    QGraphicsColorizeEffect,
 )
-from PySide6.QtGui import QColor, QPen, QBrush, QPainterPath, QLinearGradient, QFont, QCursor, QPainterPathStroker
-from PySide6.QtSvgWidgets import QGraphicsSvgItem
-from PySide6.QtSvg import QSvgRenderer
 from spinetoolbox.helpers import color_from_index
+from .font import TOOLBOX_FONT
 from .project_item_icon import ConnectorButton
 
 LINK_COLOR = color_from_index(0, 2, base_hue=60)
@@ -280,7 +276,7 @@ class _IconBase(QGraphicsEllipseItem):
         if tooltip:
             self.setToolTip(tooltip)
         self.setAcceptHoverEvents(True)
-        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=False)
+        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, enabled=False)
         self.setBrush(palette.window())
 
     def hoverEnterEvent(self, event):
@@ -291,7 +287,7 @@ class _IconBase(QGraphicsEllipseItem):
 
 
 class _SvgIcon(_IconBase):
-    """A svg icon to show over a Link."""
+    """An SVG icon to show over a Link."""
 
     def __init__(self, parent, extent, path, tooltip=None, active=False):
         super().__init__(0, 0, extent, extent, parent, tooltip=tooltip, active=active)
@@ -305,7 +301,7 @@ class _SvgIcon(_IconBase):
         scale = 0.8 * self.rect().width() / self._renderer.defaultSize().width()
         self._svg_item.setScale(scale)
         self._svg_item.setPos(self.sceneBoundingRect().center() - self._svg_item.sceneBoundingRect().center())
-        self.setPen(Qt.NoPen)
+        self.setPen(Qt.PenStyle.NoPen)
 
     def wipe_out(self):
         """Cleans up icon's resources."""
@@ -320,12 +316,12 @@ class _TextIcon(_IconBase):
     def __init__(self, parent, extent, char, tooltip=None, active=False):
         super().__init__(0, 0, extent, extent, parent, tooltip=tooltip, active=active)
         self._text_item = QGraphicsTextItem(self)
-        font = QFont('Font Awesome 5 Free Solid', weight=QFont.Bold)
+        font = QFont(TOOLBOX_FONT.family, weight=QFont.Weight.Bold)
         self._text_item.setFont(font)
         self._text_item.setDefaultTextColor(self._fg_color)
         self._text_item.setPlainText(char)
         self._text_item.setPos(self.sceneBoundingRect().center() - self._text_item.sceneBoundingRect().center())
-        self.setPen(Qt.NoPen)
+        self.setPen(Qt.PenStyle.NoPen)
 
     def wipe_out(self):
         """Cleans up icon's resources."""
@@ -347,8 +343,8 @@ class JumpOrLink(LinkBase):
 
     def __init__(self, toolbox, src_connector, dst_connector):
         super().__init__(toolbox, src_connector, dst_connector)
-        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=True)
-        self.setFlag(QGraphicsItem.ItemIsFocusable, enabled=True)
+        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, enabled=True)
+        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsFocusable, enabled=True)
         self._icon_extent = 3 * self.magic_number
         self._icons = []
         self._anim = self._make_execution_animation()
@@ -373,7 +369,7 @@ class JumpOrLink(LinkBase):
         if icon_count == 1:
             self._icons[0].setPos(center - offset)
             return
-        points = list(_regular_poligon_points(icon_count, icon_extent, self._guide_path.angleAtPercent(0.5)))
+        points = list(_regular_polygon_points(icon_count, icon_extent, self._guide_path.angleAtPercent(0.5)))
         points_center = functools.reduce(lambda a, b: a + b, points) / icon_count
         offset += points_center - center
         scale = icon_extent / self._icon_extent
@@ -631,12 +627,14 @@ class LinkDrawerBase(LinkBase):
         view = self._toolbox.ui.graphicsView
         self.tip = view.mapToScene(view.mapFromGlobal(QCursor.pos()))
         self.src_connector = src_connector
-        self.src_connector.scene().addItem(self)
+        scene = self.src_connector.scene()
+        scene.addItem(self)
         self._stroker.setWidth(self.magic_number)
         self._pen.setWidthF(self.magic_number)
         self.setPen(self._pen)
         self.update_geometry()
         self.show()
+        scene.link_about_to_be_drawn.emit()
 
     def sleep(self):
         """Removes this drawer from the scene, clears its source and destination connectors, and hides it.
@@ -646,6 +644,7 @@ class LinkDrawerBase(LinkBase):
         scene.removeItem(self)
         scene.link_drawer = self.src_connector = self.dst_connector = self.tip = None
         self.hide()
+        scene.link_drawing_finished.emit()
 
 
 class ConnectionLinkDrawer(LinkDrawerBase):
@@ -692,7 +691,7 @@ class JumpLinkDrawer(LinkDrawerBase):
         self.sleep()
 
 
-def _regular_poligon_points(n, side, initial_angle=0):
+def _regular_polygon_points(n, side, initial_angle=0):
     internal_angle = 180 * (n - 2) / n
     angle_inc = 180 - internal_angle
     current_angle = initial_angle
