@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,14 +10,10 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Provides the main() function.
-
-:author: A. Soininen (VTT)
-:date:   4.10.2019
-"""
-import os
+"""Provides the main() function."""
+import asyncio
 import multiprocessing
+import os
 import PySide6
 
 dirname = os.path.dirname(PySide6.__file__)
@@ -25,48 +22,46 @@ os.environ["QT_QPA_PLATFORM_PLUGIN_PATH"] = plugin_path
 
 # pylint: disable=wrong-import-position, wrong-import-order
 from argparse import ArgumentParser
-import sys
 import logging
-from PySide6.QtGui import QFontDatabase
+import sys
+from PySide6.QtCore import QTimer
 from PySide6.QtWidgets import QApplication
-
-# Importing resources_icons_rc initializes resources and Font Awesome gets added to the application
-from . import resources_icons_rc  # pylint: disable=unused-import
-from spine_items import resources_icons_rc  # pylint: disable=unused-import
-
+from .font import TOOLBOX_FONT
+from .headless import Status, headless_main
+from .helpers import pyside6_version_check
 from .ui_main import ToolboxUI
 from .version import __version__
-from .headless import headless_main, Status
-from .helpers import pyside6_version_check
+
+# MacOS complains about missing item icons without the following line.
+from spine_items import resources_icons_rc  # pylint: disable=unused-import  # isort: skip
 
 
 def main():
     """Creates main window GUI and starts main event loop."""
     multiprocessing.freeze_support()
+    if sys.platform == "win32":
+        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
     logging.basicConfig(
         stream=sys.stderr,
         level=logging.DEBUG,
-        format='%(asctime)s %(levelname)s: %(message)s',
-        datefmt='%Y-%m-%d %H:%M:%S',
+        format="%(asctime)s %(levelname)s: %(message)s",
+        datefmt="%Y-%m-%d %H:%M:%S",
     )
     if not pyside6_version_check():
         return 1
-    _add_pywin32_system32_to_path()
     parser = _make_argument_parser()
     args = parser.parse_args()
-    if args.execute_only or args.list_items:
+    if args.execute_only or args.list_items or args.execute_remotely:
         return_code = headless_main(args)
         if return_code == Status.ARGUMENT_ERROR:
             parser.print_usage()
         return return_code
     app = QApplication(sys.argv)
     app.setApplicationName("Spine Toolbox")
-    status = QFontDatabase.addApplicationFont(":/fonts/fontawesome5-solid-webfont.ttf")
-    if status < 0:
-        logging.warning("Could not load fonts from resources file. Some icons may not render properly.")
+    TOOLBOX_FONT.get_family_from_font_database()
     window = ToolboxUI()
     window.show()
-    window.init_project(args.project)
+    QTimer.singleShot(0, lambda: window.init_tasks(args.project))
     # Enter main event loop and wait until exit() is called
     return_code = app.exec()
     return return_code
@@ -96,19 +91,9 @@ def _make_argument_parser():
         "-d",
         "--deselect",
         action="append",
-        help="deselect project item ITEM for execution (takes precendence over --select)",
+        help="deselect project item ITEM for execution (takes precedence over --select)",
         nargs="*",
         metavar="ITEM",
     )
+    parser.add_argument("--execute-remotely", help="execute remotely", action="append", metavar="SERVER CONFIG FILE")
     return parser
-
-
-def _add_pywin32_system32_to_path():
-    """Adds a directory to PATH on Windows that is required to make pywin32 work
-    on (Conda) Python 3.8. See https://github.com/spine-tools/Spine-Toolbox/issues/1230."""
-    if not sys.platform == "win32":
-        return
-    if sys.version_info[0:2] == (3, 8):
-        p = os.path.join(sys.exec_prefix, "Lib", "site-packages", "pywin32_system32")
-        if os.path.exists(p):
-            os.environ["PATH"] = p + ";" + os.environ["PATH"]
