@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -13,7 +14,4 @@
 Modules in this package contain classes that represent Spine Toolbox's models (internal data structures) in the
 Model-View-Controller design pattern. The model classes define an interface that is used by views and delegates
 to access data in the application.
-
-:author: P. Savolainen (VTT)
-:date:   24.9.2019
 """
