@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,18 +10,12 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains FilterExecutionModel.
-
-:author: M. Marin (KTH)
-:date:   26.11.2020
-"""
-
-from PySide6.QtCore import Qt, QModelIndex, QAbstractListModel
+"""Contains FilterExecutionModel."""
+from PySide6.QtCore import QAbstractListModel, QModelIndex, Qt
 
 
 class FilterExecutionModel(QAbstractListModel):
-    _filter_consoles = dict()
+    _filter_consoles = {}
 
     def reset_model(self, filter_consoles):
         self.beginResetModel()
