@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,33 +10,27 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-A model for maps, used by the parameter_value editors.
-
-:authors: A. Soininen (VTT)
-:date:    11.2.2020
-"""
+"""A model for maps, used by the parameter_value editors."""
 from copy import deepcopy
-from numbers import Number
 from itertools import takewhile
+from numbers import Number
 import numpy
 from PySide6.QtCore import QAbstractTableModel, QModelIndex, Qt
 from PySide6.QtGui import QColor, QFont
 from spinedb_api import (
     Array,
-    convert_leaf_maps_to_specialized_containers,
-    convert_map_to_table,
     DateTime,
     Duration,
     Map,
     ParameterValueFormatError,
     TimePattern,
     TimeSeries,
+    convert_leaf_maps_to_specialized_containers,
+    convert_map_to_table,
 )
 from .indexed_value_table_model import EXPANSE_COLOR
 from .shared import PARSED_ROLE
 
-
 empty = object()
 """Sentinel for empty cells."""
 
@@ -187,7 +182,7 @@ class MapModel(QAbstractTableModel):
     def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
         """Returns row numbers for vertical headers and column titles for horizontal ones."""
         if role != Qt.ItemDataRole.DisplayRole:
-            return None
+            return super().headerData(section, orientation, role)
         if orientation == Qt.Orientation.Vertical:
             if section < len(self._rows):
                 return section + 1
@@ -234,7 +229,7 @@ class MapModel(QAbstractTableModel):
                 row_before = len(self._rows[0]) * [empty]
         else:
             row_before = (len(self._index_names) + 1) * [empty]
-        inserted = list()
+        inserted = []
         for _ in range(count):
             inserted.append([deepcopy(x) if x is not empty else x for x in row_before])
         self._rows = self._rows[:row] + inserted + self._rows[row:]
@@ -472,7 +467,7 @@ def _rows_to_dict(rows):
     Returns:
         dict: a nested dictionary
     """
-    tree = dict()
+    tree = {}
     for row in rows:
         current = tree
         for i, column in enumerate(row):
@@ -481,7 +476,7 @@ def _rows_to_dict(rows):
             if i < len(row) - 2 and row[i + 2] is not empty:
                 if not isinstance(column, (str, int, float, DateTime, Duration)):
                     raise ParameterValueFormatError(f"Index on row {rows.index(row) + 1} column {i + 1} is not scalar.")
-                current = current.setdefault(column, dict())
+                current = current.setdefault(column, {})
                 if not isinstance(current, dict):
                     raise ParameterValueFormatError(f"Indexing broken on row {rows.index(row) + 1} column {i + 1}.")
             else:
@@ -503,8 +498,8 @@ def _reconstruct_map(tree):
     Returns:
         Map: reconstructed Map
     """
-    indexes = list()
-    values = list()
+    indexes = []
+    values = []
     for key, value in tree.items():
         if isinstance(value, dict):
             value = _reconstruct_map(value)
@@ -512,8 +507,10 @@ def _reconstruct_map(tree):
         values.append(value)
     if len(indexes) > 1:
         first_type = type(indexes[0])
+        if first_type == numpy.float64:
+            first_type = float
         if any(not isinstance(i, first_type) for i in indexes[1:]):
-            raise ParameterValueFormatError(f"Index type mismatch.")
+            raise ParameterValueFormatError("Index type mismatch.")
     map_ = Map(indexes, values)
     return map_
 
@@ -542,7 +539,7 @@ def _gather_index_names(map_value):
     Returns:
         list of str: index names
     """
-    nested_names = list()
+    nested_names = []
     for v in map_value.values:
         if isinstance(v, Map):
             new_nested_names = _gather_index_names(v)
@@ -565,7 +562,7 @@ def _apply_index_names(map_value, index_names):
 
 
 def _numpy_string_to_python_strings(rows):
-    """Converts instances of numpy.str_ to regular Python strings.
+    """Converts instances of ``numpy.str_`` to regular Python strings.
 
     Args:
         rows (list of list): table rows
