@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,15 +10,9 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains a class for storing Tool specifications.
-
-:authors: P. Savolainen (VTT)
-:date:   23.1.2018
-"""
-
+"""Contains a class for storing Tool specifications."""
 import bisect
-from PySide6.QtCore import Qt, QModelIndex, QAbstractListModel, QSortFilterProxyModel, Slot, Signal
+from PySide6.QtCore import QAbstractListModel, QModelIndex, QSortFilterProxyModel, Qt, Signal, Slot
 
 
 class ProjectItemSpecificationModel(QAbstractListModel):
@@ -27,7 +22,7 @@ class ProjectItemSpecificationModel(QAbstractListModel):
 
     def __init__(self, icons):
         super().__init__()
-        self._spec_names = list()
+        self._spec_names = []
         self._icons = icons
         self._project = None
 
@@ -81,7 +76,7 @@ class ProjectItemSpecificationModel(QAbstractListModel):
 
     def clear(self):
         self.beginResetModel()
-        self._spec_names = list()
+        self._spec_names = []
         self.endResetModel()
 
     def rowCount(self, parent=None):
@@ -110,13 +105,6 @@ class ProjectItemSpecificationModel(QAbstractListModel):
         row = index.row()
         if role == Qt.ItemDataRole.DisplayRole:
             return self._spec_names[row]
-        if role == Qt.ItemDataRole.ToolTipRole:
-            if row >= self.rowCount():
-                return ""
-            return (
-                "<p>Drag-and-drop this onto the Design View "
-                f"to create a new <b>{self._spec_names[row]}</b> item.</p>"
-            )
         if role == Qt.ItemDataRole.DecorationRole:
             spec = self.specification(row)
             return self._icons[spec.item_type]
