@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,14 +10,13 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains stuff that is used by more than one model
-
-:author: M. Marin (KTH)
-:date:   23.3.2020
-"""
-
+"""Contains stuff that is used by more than one model."""
 from PySide6.QtCore import Qt
 
 PARSED_ROLE = Qt.ItemDataRole.UserRole
 DB_MAP_ROLE = Qt.ItemDataRole.UserRole + 1
+PARAMETER_TYPE_VALIDATION_ROLE = Qt.ItemDataRole.UserRole + 2
+
+INVALID_TYPE = 0
+TYPE_NOT_VALIDATED = 1
+VALID_TYPE = 2
