@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,15 +10,8 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-A model for variable resolution time series, used by the parameter_value editors.
-
-:authors: A. Soininen (VTT)
-:date:   5.7.2019
-"""
-
+"""A model for variable resolution time series, used by the parameter_value editors."""
 import numpy as np
-
 from PySide6.QtCore import QModelIndex, Qt, Slot
 from spinedb_api import TimeSeriesVariableResolution
 from .indexed_value_table_model import IndexedValueTableModel
@@ -26,6 +20,14 @@ from .indexed_value_table_model import IndexedValueTableModel
 class TimeSeriesModelVariableResolution(IndexedValueTableModel):
     """A model for variable resolution time series type parameter values."""
 
+    def column_type(self, column):
+        """Returns column's type."""
+        if column == 0:
+            return np.datetime64
+        if column == 1:
+            return float
+        raise RuntimeError("Logic error: column out of bounds")
+
     def flags(self, index):
         """Returns the flags for given model index."""
         if not index.isValid():
@@ -66,7 +68,7 @@ class TimeSeriesModelVariableResolution(IndexedValueTableModel):
             if len(old_indexes) > 1:
                 last_time_step = last_time_stamp - old_indexes[-2]
             else:
-                last_time_step = np.timedelta64(1, 'h')
+                last_time_step = np.timedelta64(1, "h")
 
             new_indexes[: len(old_indexes)] = old_indexes
             for i in range(count):
@@ -81,7 +83,7 @@ class TimeSeriesModelVariableResolution(IndexedValueTableModel):
                 if len(old_indexes) > 1:
                     time_step = old_indexes[1] - first_time_stamp
                 else:
-                    time_step = np.timedelta64(1, 'h')
+                    time_step = np.timedelta64(1, "h")
                 for i in range(count):
                     new_indexes[i] = first_time_stamp - (count - i) * time_step
                 new_indexes[count:] = old_indexes
@@ -153,7 +155,7 @@ class TimeSeriesModelVariableResolution(IndexedValueTableModel):
             try:
                 self._value.indexes[row] = value
             except ValueError:
-                self._value.indexes[row] = np.datetime64()
+                self._value.indexes[row] = np.datetime64()  # pylint: disable=no-value-for-parameter
         else:
             try:
                 self._value.values[row] = value
@@ -170,20 +172,20 @@ class TimeSeriesModelVariableResolution(IndexedValueTableModel):
             indexes (Sequence): a sequence of model indexes
             values (Sequence): a sequence of datetimes/floats corresponding to the indexes
         """
-        modified_rows = list()
-        modified_columns = list()
+        modified_rows = set()
+        modified_columns = set()
         for index, value in zip(indexes, values):
             row = index.row()
-            modified_rows.append(row)
+            modified_rows.add(row)
             column = index.column()
-            modified_columns.append(column)
+            modified_columns.add(column)
             if column == 0:
                 self._value.indexes[row] = value
             else:
                 self._value.values[row] = value
         left_top = self.index(min(modified_rows), min(modified_columns))
         right_bottom = self.index(max(modified_rows), max(modified_columns))
-        self.dataChanged.emit(left_top, right_bottom, [Qt.ItemDataRole.EditRole])
+        self.dataChanged.emit(left_top, right_bottom, [Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole])
 
     @Slot(bool, name="set_ignore_year")
     def set_ignore_year(self, ignore_year):
