@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,34 +10,25 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Functions for plotting on PlotWidget.
-
-:author: A. Soininen (VTT)
-:date:   9.7.2019
-"""
-import datetime
-from enum import auto, Enum, unique
-import math
+"""Functions for plotting on PlotWidget."""
 from contextlib import contextmanager
 from dataclasses import dataclass, field, replace
+import datetime
+from enum import Enum, auto, unique
 import functools
-from operator import methodcaller, itemgetter
+import math
+from operator import itemgetter, methodcaller
 from typing import Dict, List, Optional, Union
-
-from matplotlib.container import BarContainer
 from matplotlib.patches import Patch
 from matplotlib.ticker import MaxNLocator
 import numpy as np
-from PySide6.QtCore import Qt, QModelIndex
-
+from PySide6.QtCore import Qt
+from spinedb_api import DateTime, IndexedValue
 from spinedb_api.parameter_value import NUMPY_DATETIME64_UNIT, from_database
-from spinedb_api import Array, IndexedValue, TimeSeries, DateTime
 from .mvcmodels.shared import PARSED_ROLE
 from .widgets.plot_canvas import LegendPosition
 from .widgets.plot_widget import PlotWidget
 
-
 LEGEND_PLACEMENT_THRESHOLD = 8
 
 
@@ -54,30 +46,35 @@ _BASE_SETTINGS = {"alpha": 0.7}
 _SCATTER_PLOT_SETTINGS = {"linestyle": "", "marker": "o"}
 _LINE_PLOT_SETTINGS = {"linestyle": "solid"}
 _SCATTER_LINE_PLOT_SETTINGS = dict(_SCATTER_PLOT_SETTINGS, **_LINE_PLOT_SETTINGS)
-_TIME_SERIES_PLOT_SETTINGS = dict(_LINE_PLOT_SETTINGS, where="post")
 
 
 class PlottingError(Exception):
     """An exception signalling failure in plotting."""
 
 
+@dataclass(frozen=True)
+class IndexName:
+    label: str
+    id: int
+
+
 @dataclass(frozen=True)
 class XYData:
     """Two-dimensional data for plotting."""
 
     x: List[Union[float, int, str, np.datetime64]]
     y: List[Union[float, int]]
-    x_label: str
+    x_label: IndexName
     y_label: str
     data_index: List[str]
-    index_names: List[str]
+    index_names: List[IndexName]
 
 
 @dataclass
 class TreeNode:
     """A labeled node in tree structure."""
 
-    label: str
+    label: Union[str, IndexName]
     content: Dict = field(default_factory=dict)
 
 
@@ -108,8 +105,8 @@ def convert_indexed_value_to_tree(value):
         else:
             try:
                 x = float(x)
-            except TypeError:
-                raise ValueError("cannot plot null values")
+            except TypeError as error:
+                raise ValueError("cannot plot null values") from error
         d.content[index] = x
     return d
 
@@ -120,7 +117,7 @@ def turn_node_to_xy_data(root_node, y_label_position, index_names=None, indexes=
     Args:
         root_node (TreeNode): root node
         y_label_position (int, optional): position of y label in indexes
-        index_names (list of str, optional): list of current index names
+        index_names (list of IndexName, optional): list of current index names
         indexes (list): list of current indexes
 
     Yields:
@@ -130,7 +127,10 @@ def turn_node_to_xy_data(root_node, y_label_position, index_names=None, indexes=
         index_names = []
     if indexes is None:
         indexes = []
-    current_index_names = index_names + [root_node.label]
+    index_name = (
+        root_node.label if isinstance(root_node.label, IndexName) else IndexName(root_node.label, len(index_names))
+    )
+    current_index_names = index_names + [index_name]
     x = []
     y = []
     for index, sub_node in root_node.content.items():
@@ -158,7 +158,7 @@ def raise_if_not_common_x_labels(data_list):
     if len(data_list) < 2:
         return
     first_label = data_list[0].x_label
-    if any(data.x_label != first_label for data in data_list[1:]):
+    if any(data.x_label.label != first_label.label for data in data_list[1:]):
         raise PlottingError("X axis labels don't match.")
 
 
@@ -177,7 +177,7 @@ def raise_if_incompatible_x(data_list):
     if not data.x:
         return
     first_type = type(data.x[0])
-    if any(type(x) is not first_type for data in data_list for x in data.x):
+    if any(not isinstance(x, first_type) for data in data_list for x in data.x):
         raise PlottingError("Incompatible x axes.")
 
 
@@ -226,7 +226,7 @@ def combine_data_with_same_indexes(data_list):
             continue
         combined_xy = []
         for i in list_is:
-            combined_xy += [(x, y) for x, y in zip(data_list[i].x, data_list[i].y)]
+            combined_xy += list(zip(data_list[i].x, data_list[i].y))
         combined_xy.sort(key=itemgetter(0))
         x, y = zip(*combined_xy)
         model_data = data_list[list_is[0]]
@@ -260,9 +260,9 @@ def plot_data(data_list, plot_widget=None, plot_type=None):
     """
     if plot_widget is None:
         plot_widget = PlotWidget(
-            legend_axes_position=LegendPosition.BOTTOM
-            if len(data_list) < LEGEND_PLACEMENT_THRESHOLD
-            else LegendPosition.RIGHT
+            legend_axes_position=(
+                LegendPosition.BOTTOM if len(data_list) < LEGEND_PLACEMENT_THRESHOLD else LegendPosition.RIGHT
+            )
         )
         needs_redraw = False
     else:
@@ -285,14 +285,17 @@ def plot_data(data_list, plot_widget=None, plot_type=None):
     _limit_string_x_tick_labels(squeezed_data, plot_widget)
     y_labels = sorted({xy_data.y_label for xy_data in data_list})
     if len(y_labels) == 1 or _always_single_y_axis(plot_type):
-        legend_handles = _plot_single_y_axis(squeezed_data, y_labels[0], plot_widget, plot_type)
+        legend_handles = _plot_single_y_axis(squeezed_data, y_labels[0], plot_widget.canvas.axes, plot_type)
     elif len(y_labels) == 2:
         legend_handles = _plot_double_y_axis(squeezed_data, y_labels, plot_widget, plot_type)
     else:
-        legend_handles = _plot_single_y_axis(squeezed_data, "", plot_widget, plot_type)
-    plot_widget.canvas.axes.set_xlabel(squeezed_data[0].x_label)
+        legend_handles = _plot_single_y_axis(squeezed_data, "", plot_widget.canvas.axes, plot_type)
+    plot_widget.canvas.axes.set_xlabel(squeezed_data[0].x_label.label)
     plot_title = " | ".join(map(str, common_indexes))
     plot_widget.canvas.axes.set_title(plot_title)
+    for data in data_list:
+        if type(data.x[0]) not in (float, np.float64, int):
+            plot_widget.canvas.axes.tick_params(axis="x", labelrotation=30)
     if len(squeezed_data) > 1:
         plot_widget.add_legend(legend_handles)
     if needs_redraw:
@@ -301,40 +304,40 @@ def plot_data(data_list, plot_widget=None, plot_type=None):
     return plot_widget
 
 
-def _plot_single_y_axis(data_list, y_label, plot_widget, plot_type):
+def _plot_single_y_axis(data_list, y_label, axes, plot_type):
     """Plots all data on single y-axis.
 
     Args:
         data_list (list of XYData): data to plot
         y_label (str): y-axis label
-        plot_widget (PlotWidget): plot widget
+        axes (Axes): plot axes
         plot_type (PlotType): plot type
 
     Returns:
         list: legend handles
     """
     if plot_type == PlotType.STACKED_LINE:
-        return _plot_stacked_line(data_list, y_label, plot_widget)
+        return _plot_stacked_line(data_list, y_label, axes)
+    if plot_type == PlotType.BAR:
+        return _plot_bar(data_list, y_label, axes)
     legend_handles = []
-    plot = _make_plot_function(plot_type, type(data_list[0].x[0]), plot_widget.canvas.axes)
+    plot = _make_plot_function(plot_type, type(data_list[0].x[0]), axes)
     for data in data_list:
         plot_label = " | ".join(map(str, data.data_index))
         x = _make_x_plottable(data.x)
         handles = plot(x, data.y, label=plot_label)
-        if isinstance(handles, BarContainer):
-            handles = [Patch(color=handles.patches[0].get_facecolor(), label=plot_label)]
         legend_handles += handles
-    plot_widget.canvas.axes.set_ylabel(y_label)
+    axes.set_ylabel(y_label)
     return legend_handles
 
 
-def _plot_stacked_line(data_list, y_label, plot_widget):
+def _plot_stacked_line(data_list, y_label, axes):
     """Plots all data as stacked lines.
 
     Args:
         data_list (list of XYData): data to plot
         y_label (str): y-axis label
-        plot_widget (PlotWidget): plot widget
+        axes (Axes): plot axes
 
     Returns:
         list: legend handles
@@ -344,11 +347,40 @@ def _plot_stacked_line(data_list, y_label, plot_widget):
     x = _make_x_plottable(data_list[0].x)
     y = [data.y for data in data_list]
     labels = [" | ".join(map(str, data.data_index)) for data in data_list]
-    handles = plot_widget.canvas.axes.stackplot(x, y, labels=labels, **_LINE_PLOT_SETTINGS, **_BASE_SETTINGS)
-    plot_widget.canvas.axes.set_ylabel(y_label)
+    handles = axes.stackplot(x, y, labels=labels, **_LINE_PLOT_SETTINGS, **_BASE_SETTINGS)
+    axes.set_ylabel(y_label)
     return handles
 
 
+def _plot_bar(data_list, y_label, axes):
+    """Plots all data as bars.
+
+    Args:
+        data_list (list of XYData): data to plot
+        y_label (str): y-axis label
+        axes (Axes): plot axes
+
+    Returns:
+        list: legend handles
+    """
+    legend_handles = []
+    plot_kwargs = {"axes": axes, **_BASE_SETTINGS}
+    data_list, bar_width, x_ticks = _group_bars(data_list)
+    if bar_width is not None:
+        plot_kwargs["width"] = bar_width
+    for data in data_list:
+        plot_kwargs["label"] = " | ".join(map(str, data.data_index))
+        x = _make_x_plottable(data.x)
+        handles = _bar(x, data.y, **plot_kwargs)
+        legend_handles += handles
+    if x_ticks is not None:
+        axes.set_xticks(*x_ticks)
+    if axes.get_ylim()[0] < 0:
+        axes.axhline(linewidth=1, color="black")
+    axes.set_ylabel(y_label)
+    return legend_handles
+
+
 def _plot_double_y_axis(data_list, y_labels, plot_widget, plot_type):
     """Plots all data on two y-axes.
 
@@ -371,8 +403,15 @@ def _plot_double_y_axis(data_list, y_labels, plot_widget, plot_type):
     for data in data_list:
         plot_label = " | ".join(map(str, data.data_index))
         x = _make_x_plottable(data.x)
-        plot = plot_left if data.y_label == left_label else plot_right
-        handles = plot(x, data.y, label=plot_label)
+        if data.y_label == left_label:
+            plot = plot_left
+            color = "crimson"
+            marker = "s"
+        else:
+            plot = plot_right
+            color = None
+            marker = "o"
+        handles = plot(x, data.y, label=plot_label, color=color, marker=marker)
         legend_handles += handles
     plot_widget.canvas.axes.set_ylabel(left_label)
     right_axes.set_ylabel(right_label)
@@ -403,7 +442,21 @@ class _PlotStackedBars:
         for key, h in zip(x, height):
             cumulative = self._cumulative_height.get(key, 0.0)
             self._cumulative_height[key] = cumulative + h
-        return self._axes.bar(x, height, bottom=bottom, **_BASE_SETTINGS, **kwargs)
+        return _bar(x, height, self._axes, bottom=bottom, **_BASE_SETTINGS, **kwargs)
+
+
+def _make_time_series_settings(plot_settings):
+    """Creates plot settings suitable for time series step plots.
+
+    Args:
+        plot_settings (dict): base plot settings
+
+    Returns:
+        dict: time series step plot settings
+    """
+    settings = dict(plot_settings)
+    settings.update(where="post")
+    return settings
 
 
 def _make_plot_function(plot_type, x_data_type, axes):
@@ -417,29 +470,70 @@ def _make_plot_function(plot_type, x_data_type, axes):
     Returns:
         Callable: plot method
     """
-    if plot_type == PlotType.SCATTER:
-        return functools.partial(_plot_or_step(x_data_type, axes), **_SCATTER_PLOT_SETTINGS, **_BASE_SETTINGS)
-    if plot_type == PlotType.SCATTER_LINE:
-        return functools.partial(_plot_or_step(x_data_type, axes), **_SCATTER_LINE_PLOT_SETTINGS, **_BASE_SETTINGS)
-    if plot_type == PlotType.LINE:
-        return functools.partial(_plot_or_step(x_data_type, axes), **_LINE_PLOT_SETTINGS, **_BASE_SETTINGS)
-    if plot_type == PlotType.BAR:
-        return functools.partial(axes.bar, **_BASE_SETTINGS)
     if plot_type == PlotType.STACKED_BAR:
         return _PlotStackedBars(axes)
-    raise RuntimeError(f"Unknown plot type '{plot_type}'")
+    is_time_series = _is_time_stamp_type(x_data_type)
+    plot_method = axes.step if is_time_series else axes.plot
+    if plot_type == PlotType.SCATTER:
+        plot_settings = _SCATTER_PLOT_SETTINGS
+    elif plot_type == PlotType.SCATTER_LINE:
+        plot_settings = _SCATTER_LINE_PLOT_SETTINGS
+    elif plot_type == PlotType.LINE:
+        plot_settings = _LINE_PLOT_SETTINGS
+    else:
+        raise RuntimeError(f"Unknown plot type '{plot_type}'")
+    if is_time_series:
+        plot_settings = _make_time_series_settings(plot_settings)
+    return functools.partial(plot_method, **plot_settings, **_BASE_SETTINGS)
 
 
-def _plot_or_step(x_data_type, axes):
-    """Makes choice between Axes.plot() and Axes.step().
+def _is_time_stamp_type(data_type):
+    """Tests if a type looks like time stamp.
 
     Args:
-        x_data_type (Type): data type of x-axis
-        axes (Axes): plot axes
+        data_type (Type): data type to test
+
+    Returns:
+        bool: True if type is a time stamp type, False otherwise
     """
-    if x_data_type in (np.datetime64, datetime.datetime, datetime.date, datetime.time):
-        return axes.step
-    return axes.plot
+    return data_type in (np.datetime64, datetime.datetime, datetime.date, datetime.time)
+
+
+def _bar(x, y, axes, **kwargs):
+    """Plots bar chart on axes but returns patches instead of bar container.
+
+    Args:
+        x (Any): x data
+        y (Any): y data
+        axes (Axes): plot axes
+        **kwargs: keyword arguments passed to bar()
+
+    Returns:
+        list of Patch: patches
+    """
+    bar_container = axes.bar(x, y, **kwargs)
+    return [Patch(color=bar_container.patches[0].get_facecolor(), label=kwargs["label"])]
+
+
+def _group_bars(data_list):
+    """Gives data with same x small offsets to prevent bar stacking.
+
+    Args:
+        data_list (List of XYData): squeezed data
+
+    Returns:
+        tuple: grouped data, bar width and x ticks
+    """
+    if len(data_list) < 2:
+        return data_list, None, None
+    ticks = np.arange(len(data_list[0].x))
+    bar_width = 1 / (len(data_list) + 1)
+    offset = bar_width * (len(data_list) - 1) / 2
+    shifted_data = []
+    for step, xy_data in enumerate(data_list):
+        x = list(ticks + (step * bar_width - offset))
+        shifted_data.append(replace(xy_data, x=x))
+    return shifted_data, bar_width, (ticks, data_list[0].x)
 
 
 def _clear_plot(plot_widget):
@@ -598,12 +692,13 @@ def plot_pivot_table_selection(model, model_indexes, plot_widget=None):
     return plot_data(data_list, plot_widget)
 
 
-def plot_db_mngr_items(items, db_maps, plot_widget=None):
+def plot_db_mngr_items(items, db_maps, db_name_registry, plot_widget=None):
     """Returns a plot widget with plots of database manager parameter value items.
 
     Args:
         items (list of dict): parameter value items
-        db_maps (list of DatabaseMappingBase): database mappings corresponding to items
+        db_maps (list of DatabaseMapping): database mappings corresponding to items
+        db_name_registry (NameRegistry): database display name registry
         plot_widget (PlotWidget, optional): widget to add plots to
     """
     if not items:
@@ -612,22 +707,20 @@ def plot_db_mngr_items(items, db_maps, plot_widget=None):
         raise PlottingError("Database maps don't match parameter values.")
     root_node = TreeNode("database")
     for item, db_map in zip(items, db_maps):
-        value = from_database(item["value"], item["type"])
+        value = item["parsed_value"]
+        db_name = db_name_registry.display_name(db_map.sa_url)
         if value is None:
             continue
         try:
             leaf_content = _convert_to_leaf(value)
         except PlottingError as error:
-            raise PlottingError(f"Failed to plot value in {db_map.codename}: {error}")
-        db_name = db_map.codename
-        parameter_name = item["parameter_name"]
-        object_name_list = item["object_name_list"]
-        if object_name_list is not None:
-            object_names = tuple(object_name_list.split(","))
-        else:
-            object_names = (item["object_name"],)
+            raise PlottingError(f"Failed to plot value in {db_name}: {error}") from error
+        parameter_name = item["parameter_definition_name"]
+        entity_byname = item["entity_byname"]
+        if not isinstance(entity_byname, tuple):
+            entity_byname = (entity_byname,)
         alternative_name = item["alternative_name"]
-        indexes = (db_name, parameter_name) + object_names + (alternative_name,)
+        indexes = (db_name, parameter_name) + entity_byname + (alternative_name,)
         index_names = _pivot_index_names(indexes)
         node = root_node
         for i, index in enumerate(indexes[:-1]):
@@ -734,15 +827,13 @@ def _convert_to_leaf(y):
     try:
         if isinstance(y, IndexedValue):
             return convert_indexed_value_to_tree(y)
-        else:
-            return float(y)
+        return float(y)
     except ValueError as error:
-        raise PlottingError(str(error))
-    except TypeError:
+        raise PlottingError(str(error)) from error
+    except TypeError as error:
         if isinstance(y, DateTime):
             return y.value
-        else:
-            raise PlottingError(f"couldn't convert {type(y).__name__} to float.")
+        raise PlottingError(f"couldn't convert {type(y).__name__} to float.") from error
 
 
 @contextmanager
@@ -779,7 +870,9 @@ def add_time_series_plot(plot_widget, value):
         plot_widget (PlotWidget): a plot widget to modify
         value (TimeSeries): the time series to plot
     """
-    plot_widget.canvas.axes.step(value.indexes, value.values, **_TIME_SERIES_PLOT_SETTINGS, **_BASE_SETTINGS)
+    plot_widget.canvas.axes.step(
+        value.indexes, value.values, **_make_time_series_settings(_LINE_PLOT_SETTINGS), **_BASE_SETTINGS
+    )
     plot_widget.canvas.axes.set_xlabel(value.index_name)
     # matplotlib cannot have time stamps before 0001-01-01T00:00 on the x axis
     left, _ = plot_widget.canvas.axes.get_xlim()
