@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,31 +10,26 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains PluginManager class.
-
-:author: M. Marin (KTH)
-:date:   21.2.2021
-"""
+"""Contains PluginManager class."""
 import itertools
-import os
 import json
-import urllib.request
+import os
+import shutil
 import urllib.error
 from urllib.parse import urljoin
-import shutil
-from PySide6.QtCore import Qt, Signal, Slot, QObject, QThread
-from spine_engine.utils.serialization import serialize_path, deserialize_path, deserialize_remote_path
-from .config import PLUGINS_PATH, PLUGIN_REGISTRY_URL
+import urllib.request
+from PySide6.QtCore import QObject, QThread, Signal, Slot
+from spine_engine.utils.serialization import deserialize_path, deserialize_remote_path, serialize_path
+from .config import PLUGIN_REGISTRY_URL, PLUGINS_PATH
 from .helpers import (
     load_plugin_dict,
     load_plugin_specifications,
-    plugins_dirs,
-    load_specification_local_data,
     load_specification_from_file,
+    load_specification_local_data,
+    plugins_dirs,
 )
-from .widgets.toolbars import PluginToolBar
 from .widgets.plugin_manager_widgets import InstallPluginDialog, ManagePluginsDialog
+from .widgets.toolbars import PluginToolBar
 
 
 def _download_file(remote, local):
@@ -41,13 +37,13 @@ def _download_file(remote, local):
     try:
         urllib.request.urlretrieve(remote, local)
     except urllib.error.HTTPError as e:
-        raise PluginWorkFailed(f"Failed to download {remote}: {str(e)}")
+        raise PluginWorkFailed(f"Failed to download {remote}: {str(e)}") from e
 
 
 def _download_plugin(plugin, plugin_local_dir):
     # 1. Create paths
     plugin_remote_file = plugin["url"]
-    plugin_remote_dir = urljoin(plugin_remote_file, '.')
+    plugin_remote_dir = urljoin(plugin_remote_file, ".")
     plugin_local_file = os.path.join(plugin_local_dir, "plugin.json")
     # 2. Download and parse plugin.json file
     _download_file(plugin_remote_file, plugin_local_file)
@@ -108,7 +104,6 @@ class PluginManager:
         local_data = load_specification_local_data(project.config_dir) if project else {}
         for plugin_dir in plugins_dirs(self._toolbox.qsettings()):
             self.load_individual_plugin(plugin_dir, local_data)
-        self._toolbox.refresh_toolbars()
 
     def reload_plugins_with_local_data(self):
         """Reloads plugins that have project specific local data."""
@@ -156,12 +151,11 @@ class PluginManager:
         self._plugin_specs.update(plugin_specs)
         toolbar = self._plugin_toolbars[name] = PluginToolBar(name, parent=self._toolbox)
         toolbar.setup(plugin_specs, disabled_plugins)
-        self._toolbox.addToolBar(Qt.TopToolBarArea, toolbar)
 
     def _create_worker(self):
         worker = _PluginWorker()
         self._workers.append(worker)
-        worker.finished.connect(lambda worker=worker: self._clean_up_worker(worker))
+        worker.finished.connect(lambda wrkr=worker: self._clean_up_worker(wrkr))
         return worker
 
     def _clean_up_worker(self, worker):
@@ -169,8 +163,11 @@ class PluginManager:
         worker.clean_up()
 
     def _load_registry(self):
-        with urllib.request.urlopen(PLUGIN_REGISTRY_URL) as url:
-            registry = json.loads(url.read().decode())
+        try:
+            with urllib.request.urlopen(PLUGIN_REGISTRY_URL) as url:
+                registry = json.loads(url.read().decode())
+        except urllib.error.URLError as error:
+            raise PluginWorkFailed("Failed to load plugin registry. Are you connected to a network?") from error
         self._registry_plugins = {plugin_dict["name"]: plugin_dict for plugin_dict in registry["plugins"]}
 
     @Slot(bool)
@@ -179,6 +176,7 @@ class PluginManager:
         worker = self._create_worker()
         worker.succeeded.connect(self._do_show_install_plugin_dialog)
         worker.failed.connect(self._toolbox.msg_error)
+        worker.failed.connect(lambda: self._toolbox.ui.menuPlugins.setEnabled(True))
         worker.start(self._load_registry)
 
     @Slot()
@@ -200,9 +198,7 @@ class PluginManager:
         plugin = self._registry_plugins[plugin_name]
         plugin_local_dir = os.path.join(PLUGINS_PATH, plugin_name)
         worker = self._create_worker()
-        worker.succeeded.connect(
-            lambda plugin_local_dir=plugin_local_dir: self._load_installed_plugin(plugin_local_dir)
-        )
+        worker.succeeded.connect(lambda plugin_local_d=plugin_local_dir: self._load_installed_plugin(plugin_local_d))
         worker.failed.connect(self._toolbox.msg_error)
         worker.start(_download_plugin, plugin, plugin_local_dir)
 
@@ -218,6 +214,7 @@ class PluginManager:
         worker = self._create_worker()
         worker.succeeded.connect(self._do_show_manage_plugins_dialog)
         worker.failed.connect(self._toolbox.msg_error)
+        worker.failed.connect(lambda: self._toolbox.ui.menuPlugins.setEnabled(True))
         worker.start(self._load_registry)
 
     @Slot()
@@ -270,7 +267,6 @@ class _PluginWorker(QObject):
     def __init__(self):
         super().__init__()
         self._thread = QThread()
-        self.moveToThread(self._thread)
         self._function = None
         self._args = None
         self._kwargs = None
