@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -8,16 +9,18 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-"""Contains logging connection and jump classes."""
 
-from spinedb_api.filters.scenario_filter import SCENARIO_FILTER_TYPE
-from spinedb_api.filters.tool_filter import TOOL_FILTER_TYPE
+"""Contains logging connection and jump classes."""
+from spine_engine.project_item.connection import ConnectionBase, FilterSettings, Jump, ResourceConvertingConnection
 from spinedb_api import DatabaseMapping, SpineDBAPIError, SpineDBVersionError
-from spine_engine.project_item.connection import ResourceConvertingConnection, Jump, ConnectionBase, FilterSettings
+from spinedb_api.filters.alternative_filter import ALTERNATIVE_FILTER_TYPE
+from spinedb_api.filters.scenario_filter import SCENARIO_FILTER_TYPE
+from ..fetch_parent import FlexibleFetchParent
+from ..helpers import busy_effect
 from ..log_mixin import LogMixin
 from ..mvcmodels.resource_filter_model import ResourceFilterModel
-from ..helpers import busy_effect
-from ..fetch_parent import FlexibleFetchParent
+
+_DATABASE_ITEM_TYPE = {ALTERNATIVE_FILTER_TYPE: "alternative", SCENARIO_FILTER_TYPE: "scenario"}
 
 
 class HeadlessConnection(ResourceConvertingConnection):
@@ -55,6 +58,15 @@ class HeadlessConnection(ResourceConvertingConnection):
         )
         specific_filter_settings[filter_name] = enabled
 
+    def set_filter_type_enabled(self, filter_type, enabled):
+        """Enables or disables a filter type.
+
+        Args:
+            filter_type (str): filter type
+            enabled (bool): True to enable the filter type, False to disable it
+        """
+        self._filter_settings.enabled_filter_types[filter_type] = enabled
+
     def _convert_legacy_resource_filter_ids_to_filter_settings(self):
         """Converts legacy resource filter ids to filter settings.
 
@@ -69,26 +81,16 @@ class HeadlessConnection(ResourceConvertingConnection):
             if not url:
                 continue
             try:
-                db_map = DatabaseMapping(url)
+                with DatabaseMapping(url) as db_map:
+                    scenario_filter_ids = resource_filter_ids.get(SCENARIO_FILTER_TYPE)
+                    if scenario_filter_ids is not None:
+                        specific_filter_settings = self._filter_settings.known_filters.setdefault(
+                            resource.label, {}
+                        ).setdefault(SCENARIO_FILTER_TYPE, {})
+                        for row in db_map.query(db_map.scenario_sq):
+                            specific_filter_settings[row.name]: row.id = row.id in scenario_filter_ids
             except (SpineDBAPIError, SpineDBVersionError):
                 continue
-            try:
-                scenario_filter_ids = resource_filter_ids.get(SCENARIO_FILTER_TYPE)
-                if scenario_filter_ids is not None:
-                    specific_filter_settings = self._filter_settings.known_filters.setdefault(
-                        resource.label, {}
-                    ).setdefault(SCENARIO_FILTER_TYPE, {})
-                    for row in db_map.query(db_map.scenario_sq):
-                        specific_filter_settings[row.name]: row.id = row.id in scenario_filter_ids
-                tool_filter_ids = resource_filter_ids.get(TOOL_FILTER_TYPE)
-                if tool_filter_ids is not None:
-                    specific_filter_settings = self._filter_settings.known_filters.setdefault(
-                        resource.label, {}
-                    ).setdefault(TOOL_FILTER_TYPE, {})
-                    for row in db_map.query(db_map.tool_sq):
-                        specific_filter_settings[row.name] = row.id in tool_filter_ids
-            finally:
-                db_map.connection.close()
         self._legacy_resource_filter_ids = None
 
     @staticmethod
@@ -157,7 +159,7 @@ class LoggingConnection(LogMixin, HeadlessConnection):
         return self.link
 
     def has_filters(self):
-        """Returns True if connection has scenario or tool filters.
+        """Returns True if connection has any filters.
 
         Returns:
             bool: True if connection has filters, False otherwise
@@ -169,20 +171,14 @@ class LoggingConnection(LogMixin, HeadlessConnection):
             db_map = self._get_db_map(url)
             if db_map is None:
                 continue
-            available_scenarios = {
-                x["name"] for x in self._toolbox.db_mngr.get_items(db_map, "scenario", only_visible=True)
-            }
-            scenario_filters = self._filter_settings.known_filters.get(resource.label, {}).get(SCENARIO_FILTER_TYPE, {})
-            if any(enabled for s, enabled in scenario_filters.items() if s in available_scenarios):
-                return True
-            if self._filter_settings.auto_online and any(name not in scenario_filters for name in available_scenarios):
-                return True
-            available_tools = {x["name"] for x in self._toolbox.db_mngr.get_items(db_map, "tool", only_visible=True)}
-            tool_filters = self._filter_settings.known_filters.get(resource.label, {}).get(TOOL_FILTER_TYPE, {})
-            if any(enabled for t, enabled in tool_filters.items() if t in available_tools):
-                return True
-            if self._filter_settings.auto_online and any(name not in tool_filters for name in available_tools):
-                return True
+            known_filters = self._filter_settings.known_filters.get(resource.label, {})
+            for filter_type, item_type in _DATABASE_ITEM_TYPE.items():
+                available = {x["name"] for x in self._toolbox.db_mngr.get_items(db_map, item_type)}
+                filters = known_filters.get(filter_type, {})
+                if any(enabled for s, enabled in filters.items() if s in available):
+                    return True
+                if self._filter_settings.auto_online and any(name not in filters for name in available):
+                    return True
         return False
 
     def _get_db_map(self, url):
@@ -222,7 +218,7 @@ class LoggingConnection(LogMixin, HeadlessConnection):
 
     def _fetch_more_if_possible(self):
         for db_map in self._db_maps.values():
-            for item_type in ("scenario", "tool"):
+            for item_type in ("scenario",):
                 fetch_parent = self._make_fetch_parent(db_map, item_type)
                 if self._toolbox.db_mngr.can_fetch_more(db_map, fetch_parent):
                     self._toolbox.db_mngr.fetch_more(db_map, fetch_parent)
@@ -241,17 +237,23 @@ class LoggingConnection(LogMixin, HeadlessConnection):
     def receive_error_msg(self, _db_map_error_log):
         pass
 
-    def get_scenario_names(self, url):
+    def get_filter_item_names(self, filter_type, url):
         db_map = self._get_db_map(url)
         if db_map is None:
             return []
-        return sorted(x["name"] for x in self._toolbox.db_mngr.get_items(db_map, "scenario", only_visible=True))
+        item_type = _DATABASE_ITEM_TYPE[filter_type]
+        return sorted(x["name"] for x in self._toolbox.db_mngr.get_items(db_map, item_type))
 
-    def get_tool_names(self, url):
-        db_map = self._get_db_map(url)
-        if db_map is None:
-            return []
-        return sorted(x["name"] for x in self._toolbox.db_mngr.get_items(db_map, "tool", only_visible=True))
+    def _do_purge_before_writing(self, resources):
+        purged_urls = super()._do_purge_before_writing(resources)
+        committed_db_maps = set()
+        for url in purged_urls:
+            db_map = self._toolbox.db_mngr.db_map(url)
+            if db_map:
+                committed_db_maps.add(db_map)
+        if committed_db_maps:
+            self._toolbox.db_mngr.notify_session_committed(self, *committed_db_maps)
+        return purged_urls
 
     def may_have_filters(self):
         """Returns whether this connection may have filters.
@@ -317,8 +319,8 @@ class LoggingConnection(LogMixin, HeadlessConnection):
 
         Args:
             resource (str): Resource label
-            filter_type (str): Either SCENARIO_FILTER_TYPE or TOOL_FILTER_TYPE, for now.
-            online (dict): mapping from scenario/tool name to online flag
+            filter_type (str): filter type
+            online (dict): mapping from scenario name to online flag
         """
         self._filter_settings.known_filters.setdefault(resource, {}).setdefault(filter_type, {}).update(online)
 
@@ -336,6 +338,13 @@ class LoggingConnection(LogMixin, HeadlessConnection):
         """Makes resource filter mode fetch filter data from database."""
         self.resource_filter_model.build_tree()
 
+    def set_filter_type_enabled(self, filter_type, enabled):
+        """See base class."""
+        super().set_filter_type_enabled(filter_type, enabled)
+        self.resource_filter_model.set_filter_type_enabled(filter_type, enabled)
+        if self is self._toolbox.active_link_item:
+            self._toolbox.link_properties_widgets[LoggingConnection].set_filter_type_enabled(filter_type, enabled)
+
     def receive_resources_from_source(self, resources):
         """See base class."""
         super().receive_resources_from_source(resources)
@@ -362,8 +371,8 @@ class LoggingConnection(LogMixin, HeadlessConnection):
         sibling_conns = project.incoming_connections(self.destination)
         for conn in sibling_conns:
             conn.link.update_icons()
-        item = project.get_item(self.source)
-        project.notify_resource_changes_to_successors(item)
+        project.notify_resource_changes_to_successors(project.get_item(self.source))
+        project.notify_resource_changes_to_predecessors(project.get_item(self.destination))
         if self is self._toolbox.active_link_item:
             self._toolbox.link_properties_widgets[LoggingConnection].load_connection_options()
 
@@ -373,9 +382,12 @@ class LoggingConnection(LogMixin, HeadlessConnection):
         Returns:
             FilterSettings: filter settings containing only filters that exist in source databases
         """
-        filter_settings = FilterSettings(auto_online=self._filter_settings.auto_online)
+        filter_settings = FilterSettings(
+            auto_online=self._filter_settings.auto_online,
+            enabled_filter_types=self._filter_settings.enabled_filter_types,
+        )
         for resource in self._resources:
-            for filter_type in (SCENARIO_FILTER_TYPE, TOOL_FILTER_TYPE):
+            for filter_type in (SCENARIO_FILTER_TYPE, ALTERNATIVE_FILTER_TYPE):
                 online_filters = self._resource_filters_online(resource, filter_type)
                 if online_filters is not None:
                     filter_settings.known_filters.setdefault(resource.label, {})[filter_type] = online_filters
@@ -388,10 +400,8 @@ class LoggingConnection(LogMixin, HeadlessConnection):
         db_map = self._get_db_map(url)
         if db_map is None:
             return None
-        db_item_type = {SCENARIO_FILTER_TYPE: "scenario", TOOL_FILTER_TYPE: "tool"}[filter_type]
-        available_filters = (
-            x["name"] for x in self._toolbox.db_mngr.get_items(db_map, db_item_type, only_visible=True)
-        )
+        db_item_type = _DATABASE_ITEM_TYPE[filter_type]
+        available_filters = (x["name"] for x in self._toolbox.db_mngr.get_items(db_map, db_item_type))
         specific_filter_settings = self._filter_settings.known_filters.get(resource.label, {}).get(filter_type, {})
         checked_specific_filter_settings = {}
         for name in sorted(available_filters):
