@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,42 +10,60 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains SpecificationEditorWindowBase and ChangeSpecPropertyCommand
-
-:author: M. Marin (KTH), P. Savolainen (VTT)
-:date:   12.4.2018
-"""
-
-from PySide6.QtGui import QKeySequence, QIcon, QUndoStack, QAction, QUndoCommand
-from PySide6.QtCore import Signal, Slot, Qt
+"""Contains SpecificationEditorWindowBase and ChangeSpecPropertyCommand"""
+from enum import IntEnum, unique
+from PySide6.QtCore import Qt, Signal, Slot
+from PySide6.QtGui import QAction, QIcon, QKeySequence, QUndoCommand, QUndoStack
 from PySide6.QtWidgets import (
-    QMainWindow,
-    QWidget,
-    QToolBar,
-    QLabel,
-    QHBoxLayout,
-    QMessageBox,
-    QMenu,
-    QLineEdit,
     QCheckBox,
     QErrorMessage,
+    QHBoxLayout,
+    QLabel,
+    QLineEdit,
+    QMainWindow,
+    QMenu,
+    QMessageBox,
+    QToolBar,
     QToolButton,
+    QWidget,
 )
+from spinetoolbox.helpers import CharIconEngine, SealCommand, restore_ui, save_ui
 from spinetoolbox.widgets.notification import ChangeNotifier, Notification
-from spinetoolbox.helpers import CharIconEngine, restore_ui, save_ui
+
+
+class UniqueCommandId:
+    _NEXT_ID = 1
+
+    @classmethod
+    def unique_id(cls):
+        """Returns a new unique command id.
+
+        Returns:
+            int: unique id
+        """
+        new_id = cls._NEXT_ID
+        cls._NEXT_ID += 1
+        return new_id
+
+
+@unique
+class CommandId(IntEnum):
+    NONE = -1
+    NAME_UPDATE = UniqueCommandId.unique_id()
+    DESCRIPTION_UPDATE = UniqueCommandId.unique_id()
 
 
 class ChangeSpecPropertyCommand(QUndoCommand):
     """Command to set specification properties."""
 
-    def __init__(self, callback, new_value, old_value, cmd_name):
+    def __init__(self, callback, new_value, old_value, cmd_name, command_id=CommandId.NONE):
         """
         Args:
-            callback (function): Function to call to set the spec property.
-            new_value (any): new value
-            old_value (any): old value
+            callback (Callable): Function to call to set the spec property.
+            new_value (Any): new value
+            old_value (Any): old value
             cmd_name (str): command name
+            command_id (IntEnum): command id
         """
         super().__init__()
         self._callback = callback
@@ -52,6 +71,8 @@ class ChangeSpecPropertyCommand(QUndoCommand):
         self._old_value = old_value
         self.setText(cmd_name)
         self.setObsolete(new_value == old_value)
+        self._id = command_id
+        self._sealed = False
 
     def redo(self):
         self._callback(self._new_value)
@@ -59,6 +80,21 @@ class ChangeSpecPropertyCommand(QUndoCommand):
     def undo(self):
         self._callback(self._old_value)
 
+    def id(self):
+        return self._id.value
+
+    def mergeWith(self, other):
+        if not isinstance(other, ChangeSpecPropertyCommand):
+            self._sealed = True
+            return False
+        if self._sealed or self.id() != other.id():
+            return False
+        if self._old_value == other._new_value:
+            self.setObsolete(True)
+        else:
+            self._new_value = other._new_value
+        return True
+
 
 class SpecificationEditorWindowBase(QMainWindow):
     """Base class for spec editors."""
@@ -94,7 +130,7 @@ class SpecificationEditorWindowBase(QMainWindow):
         self._spec_toolbar = _SpecNameDescriptionToolbar(self, specification, self._undo_stack)
         self._spec_toolbar.show_toolbox_action.triggered.connect(self._toolbox.restore_and_activate)
         self._spec_toolbar.name_changed.connect(self._set_window_title)
-        self.addToolBar(Qt.TopToolBarArea, self._spec_toolbar)
+        self.addToolBar(Qt.ToolBarArea.TopToolBarArea, self._spec_toolbar)
         self._populate_main_menu()
         self._spec_toolbar.save_action.triggered.connect(self._save)
         self._spec_toolbar.duplicate_action.triggered.connect(self._duplicate)
@@ -173,9 +209,12 @@ class SpecificationEditorWindowBase(QMainWindow):
         self.setWindowTitle(title)
         self.windowTitleChanged.emit(self.windowTitle())
 
-    def _save(self):
+    def _save(self, exiting=None):
         """Saves spec.
 
+        Args:
+            exiting (bool, optional): Set as True if called when trying to exit the editor window
+
         Returns:
             bool: True if operation was successful, False otherwise
         """
@@ -184,11 +223,13 @@ class SpecificationEditorWindowBase(QMainWindow):
             return False
         name = self._spec_toolbar.name()
         if not name:
+            if exiting:
+                return self.prompt_exit_without_saving()
             self.show_error("Please enter a name for the specification.")
             return False
         spec = self._make_new_specification(name)
         if spec is None:
-            return False
+            return self.prompt_exit_without_saving() if exiting else False
         if not self._original_spec_name:
             if self._toolbox.project().is_specification_name_reserved(name):
                 self.show_error("Specification name already in use. Please enter a new name.")
@@ -213,6 +254,24 @@ class SpecificationEditorWindowBase(QMainWindow):
         self.setWindowTitle(self.specification.name)
         return True
 
+    def prompt_exit_without_saving(self):
+        """Prompts whether the user wants to exit without saving or cancel the exit.
+
+        Returns:
+            bool: False if the user chooses to cancel, in which case we don't close the form.
+        """
+
+        msg = QMessageBox(self)
+        msg.setIcon(QMessageBox.Icon.Question)
+        msg.setWindowTitle(self.windowTitle())
+        msg.setText("Can't save unfinished specification.\nDo you want to exit without saving?")
+        msg.setStandardButtons(QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.Cancel)
+        answer = msg.exec()
+        if answer == QMessageBox.StandardButton.Cancel:
+            return False
+        if answer == QMessageBox.StandardButton.Yes:
+            return True
+
     @property
     def _duplicate_kwargs(self):
         return {}
@@ -227,7 +286,9 @@ class SpecificationEditorWindowBase(QMainWindow):
     def tear_down(self):
         if self.focusWidget():
             self.focusWidget().clearFocus()
-        if not self._undo_stack.isClean() and not prompt_to_save_changes(self, self._toolbox.qsettings(), self._save):
+        if not self._undo_stack.isClean() and not prompt_to_save_changes(
+            self, self._toolbox.qsettings(), self._save, True
+        ):
             return False
         self._change_notifier.tear_down()
         self._undo_stack.cleanChanged.disconnect(self._update_window_modified)
@@ -267,9 +328,9 @@ class _SpecNameDescriptionToolbar(QToolBar):
         self.setMovable(False)
         widget = QWidget()
         layout = QHBoxLayout(widget)
-        layout.addWidget(QLabel("Name"))
+        layout.addWidget(QLabel("Name:"))
         layout.addWidget(self._line_edit_name)
-        layout.addWidget(QLabel("Description"))
+        layout.addWidget(QLabel("Description:"))
         layout.addWidget(self._line_edit_description)
         layout.setContentsMargins(3, 3, 3, 3)
         layout.setStretchFactor(self._line_edit_name, 1)
@@ -277,7 +338,7 @@ class _SpecNameDescriptionToolbar(QToolBar):
         self.addWidget(widget)
         toolbox_icon = QIcon(":/symbols/Spine_symbol.png")
         self.show_toolbox_action = self.addAction(toolbox_icon, "Show Spine Toolbox (Ctrl+ESC)")
-        self.show_toolbox_action.setShortcut(Qt.CTRL | Qt.Key_Escape)
+        self.show_toolbox_action.setShortcut(QKeySequence(Qt.Modifier.CTRL.value | Qt.Key.Key_Escape.value))
         self.menu = self._make_main_menu()
         self.save_action = self.menu.addAction("Save")
         self.duplicate_action = self.menu.addAction("Duplicate")
@@ -286,7 +347,7 @@ class _SpecNameDescriptionToolbar(QToolBar):
         self.duplicate_action.setEnabled(self._parent.specification is not None)
         self.save_action.setShortcut(QKeySequence.Save)
         self.save_action.setIcon(QIcon(":/icons/menu_icons/save_solid.svg"))
-        self.duplicate_action.setShortcut(QKeySequence(Qt.CTRL | Qt.Key_D))
+        self.duplicate_action.setShortcut(QKeySequence(Qt.Modifier.CTRL.value | Qt.Key.Key_D.value))
         self.duplicate_action.setIcon(QIcon(":/icons/menu_icons/copy.svg"))
         self.close_action.setShortcut(QKeySequence.Close)
         self.close_action.setIcon(QIcon(":/icons/menu_icons/window-close.svg"))
@@ -294,8 +355,10 @@ class _SpecNameDescriptionToolbar(QToolBar):
         if spec:
             self.do_set_name(spec.name)
             self.do_set_description(spec.description)
-        self._line_edit_name.editingFinished.connect(self._set_name)
-        self._line_edit_description.editingFinished.connect(self._set_description)
+        self._line_edit_name.textEdited.connect(self._update_name)
+        self._line_edit_name.editingFinished.connect(self._finish_name_editing)
+        self._line_edit_description.textEdited.connect(self._update_description)
+        self._line_edit_description.editingFinished.connect(self._finish_description_editing)
 
     def _make_main_menu(self):
         menu = QMenu(self)
@@ -305,40 +368,66 @@ class _SpecNameDescriptionToolbar(QToolBar):
         menu_button.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
         action = QAction(self)
         action.triggered.connect(menu_button.showMenu)
-        keys = [QKeySequence(Qt.ALT | Qt.Key_F), QKeySequence(Qt.ALT | Qt.Key_E)]
+        keys = [
+            QKeySequence(Qt.Modifier.ALT.value | Qt.Key.Key_F.value),
+            QKeySequence(Qt.Modifier.ALT.value | Qt.Key.Key_E.value),
+        ]
         action.setShortcuts(keys)
         self._parent.addAction(action)
         keys_str = ", ".join([key.toString() for key in keys])
         menu_button.setToolTip(f"<p>Main menu ({keys_str})</p>")
         return menu
 
-    @Slot()
-    def _set_name(self):
-        if self.name() == self._current_name:
-            return
+    @Slot(str)
+    def _update_name(self, name):
+        """Pushes a command to undo stack that updates the specification name.
+
+        Args:
+            name (str): updated name
+        """
         self._undo_stack.push(
-            ChangeSpecPropertyCommand(self.do_set_name, self.name(), self._current_name, "change specification name")
+            ChangeSpecPropertyCommand(
+                self.do_set_name, name, self._current_name, "change specification name", CommandId.NAME_UPDATE
+            )
         )
 
     @Slot()
-    def _set_description(self):
-        if self.description() == self._current_description:
-            return
+    def _finish_name_editing(self):
+        """Seals the last undo command."""
+        self._undo_stack.push(SealCommand(CommandId.NAME_UPDATE.value))
+
+    @Slot(str)
+    def _update_description(self, description):
+        """Pushes a command to undo stack that updates the specification description.
+
+        Args:
+            description (str): updated description
+        """
         self._undo_stack.push(
             ChangeSpecPropertyCommand(
                 self.do_set_description,
                 self.description(),
                 self._current_description,
                 "change specification description",
+                CommandId.DESCRIPTION_UPDATE,
             )
         )
 
+    @Slot()
+    def _finish_description_editing(self):
+        """Seals the last undo command."""
+        self._undo_stack.push(SealCommand(CommandId.DESCRIPTION_UPDATE.value))
+
     def do_set_name(self, name):
+        self.name_changed.emit(name)
+        if self._line_edit_name.text() == name:
+            return
         self._current_name = name
         self._line_edit_name.setText(name)
-        self.name_changed.emit(name)
 
     def do_set_description(self, description):
+        if self._line_edit_description.text() == description:
+            return
         self._current_description = description
         self._line_edit_description.setText(description)
 
@@ -349,7 +438,7 @@ class _SpecNameDescriptionToolbar(QToolBar):
         return self._line_edit_description.text()
 
 
-def prompt_to_save_changes(parent, settings, save_callback):
+def prompt_to_save_changes(parent, settings, save_callback, exiting=None):
     """Prompts to save changes.
 
     Args:
@@ -357,6 +446,7 @@ def prompt_to_save_changes(parent, settings, save_callback):
         settings (QSettings): Toolbox settings
         save_callback (Callable): A function to call if the user chooses Save.
             It must return True or False depending on the outcome of the 'saving'.
+        exiting (bool, optional): Set as True if called when trying to exit the editor window
 
     Returns:
         bool: False if the user chooses to cancel, in which case we don't close the form.
@@ -365,7 +455,7 @@ def prompt_to_save_changes(parent, settings, save_callback):
     if save_spec == 0:
         return True
     if save_spec == 2:
-        return save_callback()
+        return save_callback(exiting)
     msg = QMessageBox(parent)
     msg.setIcon(QMessageBox.Icon.Question)
     msg.setWindowTitle(parent.windowTitle())
@@ -384,5 +474,5 @@ def prompt_to_save_changes(parent, settings, save_callback):
         preference = "2" if answer == QMessageBox.StandardButton.Yes else "0"
         settings.setValue("appSettings/saveSpecBeforeClosing", preference)
     if answer == QMessageBox.StandardButton.Yes:
-        return save_callback()
+        return save_callback(exiting)
     return True
