@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,43 +10,27 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Classes for drawing graphics items on QGraphicsScene.
-
-:authors: M. Marin (KTH), P. Savolainen (VTT)
-:date:    4.4.2018
-"""
-
+"""Classes for drawing graphics items on QGraphicsScene."""
 import math
-from PySide6.QtCore import Qt, QPointF, QRectF, QLineF
-from PySide6.QtWidgets import (
-    QGraphicsItem,
-    QGraphicsTextItem,
-    QGraphicsSimpleTextItem,
-    QGraphicsPathItem,
-    QGraphicsEllipseItem,
-    QGraphicsColorizeEffect,
-    QGraphicsDropShadowEffect,
-    QApplication,
-    QToolTip,
-    QStyle,
-)
-from PySide6.QtGui import (
-    QColor,
-    QPen,
-    QBrush,
-    QTextCursor,
-    QPalette,
-    QTextBlockFormat,
-    QFont,
-    QPainterPath,
-    QRadialGradient,
-)
+from PySide6.QtCore import QLineF, QPointF, QRectF, Qt
+from PySide6.QtGui import QBrush, QColor, QFont, QPainterPath, QPen, QRadialGradient, QTextBlockFormat, QTextCursor
 from PySide6.QtSvg import QSvgRenderer
 from PySide6.QtSvgWidgets import QGraphicsSvgItem
+from PySide6.QtWidgets import (
+    QApplication,
+    QGraphicsColorizeEffect,
+    QGraphicsDropShadowEffect,
+    QGraphicsEllipseItem,
+    QGraphicsItem,
+    QGraphicsPathItem,
+    QGraphicsTextItem,
+    QStyle,
+    QToolTip,
+)
 from spine_engine.spine_engine import ItemExecutionFinishState
-from .project_commands import MoveIconCommand
+from .font import TOOLBOX_FONT
 from .helpers import LinkType, fix_lightness_color
+from .project_commands import MoveIconCommand
 
 
 class ProjectItemIcon(QGraphicsPathItem):
@@ -84,15 +69,17 @@ class ProjectItemIcon(QGraphicsPathItem):
         self.rank_icon = RankIcon(self)
         # Make item name graphics item.
         self._name = ""
-        self.name_item = QGraphicsSimpleTextItem(self._name)
+        self.name_item = QGraphicsTextItem(self._name)
         self.name_item.setZValue(100)
         self.set_name_attributes()  # Set font, size, position, etc.
+        self.spec_item = None  # For displaying Tool Spec icon
+        self.spec_item_renderer = None
         # Make connector buttons
-        self.connectors = dict(
-            bottom=ConnectorButton(toolbox, self, position="bottom"),
-            left=ConnectorButton(toolbox, self, position="left"),
-            right=ConnectorButton(toolbox, self, position="right"),
-        )
+        self.connectors = {
+            "bottom": ConnectorButton(toolbox, self, position="bottom"),
+            "left": ConnectorButton(toolbox, self, position="left"),
+            "right": ConnectorButton(toolbox, self, position="right"),
+        }
         self._setup()
         shadow_effect = QGraphicsDropShadowEffect()
         shadow_effect.setOffset(1)
@@ -100,6 +87,31 @@ class ProjectItemIcon(QGraphicsPathItem):
         self.setGraphicsEffect(shadow_effect)
         self._update_path()
 
+    def add_specification_icon(self, spec_icon_path):
+        """Adds an SVG icon to bottom left corner of the item icon based on Tool Specification type.
+
+        Args:
+            spec_icon_path (str): Path to icon resource file.
+        """
+        self.spec_item = QGraphicsSvgItem(self)
+        self.spec_item_renderer = QSvgRenderer()
+        loading_ok = self.spec_item_renderer.load(spec_icon_path)
+        if not loading_ok:
+            self._toolbox.msg_error.emit(f"Loading SVG icon from resource {spec_icon_path} failed")
+            return
+        size = self.spec_item_renderer.defaultSize()
+        self.spec_item.setSharedRenderer(self.spec_item_renderer)
+        self.spec_item.setElementId("")
+        dim_max = max(size.width(), size.height())
+        rect_w = 0.3 * self.rect().width()  # Parent rect width
+        self.spec_item.setScale(rect_w / dim_max)
+        self.spec_item.setPos(self.sceneBoundingRect().bottomLeft() - self.spec_item.sceneBoundingRect().center())
+
+    def remove_specification_icon(self):
+        """Removes the specification icon SVG from the scene."""
+        self.spec_item.setParentItem(None)
+        self.spec_item = None
+
     def rect(self):
         return self._rect
 
@@ -128,20 +140,19 @@ class ProjectItemIcon(QGraphicsPathItem):
         selection_pen.setWidthF(pen_width)
         self._selection_halo.setPen(selection_pen)
 
-    def finalize(self, name, x, y):
-        """
-        Names the icon and moves it by given amount.
+    def finalize(self, name: str, x: float, y: float):
+        """Names the icon and moves it by a given amount.
 
         Args:
-            name (str): icon's name
-            x (int): horizontal offset
-            y (int): vertical offset
+            name: icon's name
+            x: horizontal offset
+            y: vertical offset
         """
         self.moveBy(x, y)
         self.update_name_item(name)
 
     def _setup(self):
-        """Setup item's attributes."""
+        """Sets up item attributes."""
         self.colorizer.setColor(self._icon_color)
         background_color = fix_lightness_color(self._icon_color)
         gradient = QRadialGradient(self._rect.center(), 1 * self._rect.width())
@@ -157,7 +168,7 @@ class ProjectItemIcon(QGraphicsPathItem):
         # Load SVG
         loading_ok = self.renderer.load(self.icon_file)
         if not loading_ok:
-            self._toolbox.msg_error.emit("Loading SVG icon from resource:{0} failed".format(self.icon_file))
+            self._toolbox.msg_error.emit(f"Loading SVG icon from resource:{self.icon_file} failed")
             return
         size = self.renderer.defaultSize()
         self.svg_item.setSharedRenderer(self.renderer)
@@ -190,25 +201,28 @@ class ProjectItemIcon(QGraphicsPathItem):
         return self._name
 
     def update_name_item(self, new_name):
-        """Set a new text to name item.
+        """Sets a new text to name item.
 
         Args:
             new_name (str): icon's name
         """
         self._name = new_name
-        self.name_item.setText(new_name)
+        self.name_item.setPlainText(new_name)
         self._reposition_name_item()
+        self.name_item.setTextWidth(100)
 
     def set_name_attributes(self):
-        """Set name QGraphicsSimpleTextItem attributes (font, size, position, etc.)"""
-        # Set font size and style
+        """Sets name item attributes (font, size, style, alignment)."""
         font = self.name_item.font()
         font.setPixelSize(self.FONT_SIZE_PIXELS)
         font.setBold(True)
         self.name_item.setFont(font)
+        option = self.name_item.document().defaultTextOption()
+        option.setAlignment(Qt.AlignmentFlag.AlignCenter)
+        self.name_item.document().setDefaultTextOption(option)
 
     def _reposition_name_item(self):
-        """Set name item position (centered on top of the master icon)."""
+        """Sets name item position (centered on top of the master icon)."""
         main_rect = self.sceneBoundingRect()
         name_rect = self.name_item.sceneBoundingRect()
         self.name_item.setPos(main_rect.center().x() - name_rect.width() / 2, main_rect.y() - name_rect.height() - 4)
@@ -302,9 +316,13 @@ class ProjectItemIcon(QGraphicsPathItem):
         if not scene:
             return
         icon_group = scene.icon_group | {self}
-        scene.dirty_links |= set(
-            link for icon in icon_group for conn in icon.connectors.values() for link in conn.links
-        )
+        dirty_links = set(link for icon in icon_group for conn in icon.connectors.values() for link in conn.links)
+        if not dirty_links:
+            return
+        qsettings = self._toolbox.qsettings()
+        curved_links = qsettings.value("appSettings/curvedLinks", defaultValue="false") == "true"
+        for link in dirty_links:
+            link.update_geometry(curved_links)
 
     def mouseReleaseEvent(self, event):
         """Clears pre-bump rects, and pushes a move icon command if necessary."""
@@ -331,14 +349,14 @@ class ProjectItemIcon(QGraphicsPathItem):
         event.accept()
         self.scene().clearSelection()
         self.setSelected(True)
-        ind = self._toolbox.project_item_model.find_item(self.name())
-        self._toolbox.show_project_or_item_context_menu(event.screenPos(), ind)
+        item = self._toolbox.project().get_item(self.name())
+        self._toolbox.show_project_or_item_context_menu(event.screenPos(), item)
 
     def itemChange(self, change, value):
         """
         Reacts to item removal and position changes.
 
-        In particular, destroys the drop shadow effect when the items is removed from a scene
+        In particular, destroys the drop shadow effect when the item is removed from a scene
         and keeps track of item's movements on the scene.
 
         Args:
@@ -417,11 +435,6 @@ class ProjectItemIcon(QGraphicsPathItem):
             pass
         return restablished
 
-    def select_item(self):
-        """Update GUI to show the details of the selected item."""
-        ind = self._toolbox.project_item_model.find_item(self.name())
-        self._toolbox.ui.treeView_project.setCurrentIndex(ind)
-
     def paint(self, painter, option, widget=None):
         """Sets a dashed pen if selected."""
         selected = bool(option.state & QStyle.StateFlag.State_Selected)
@@ -448,7 +461,7 @@ class ConnectorButton(QGraphicsPathItem):
         self._parent = parent
         self._toolbox = toolbox
         self.position = position
-        self.links = list()
+        self.links = []
         self.setBrush(self.brush)
         parent_rect = parent.rect()
         extent = 0.2 * parent_rect.width()
@@ -504,7 +517,6 @@ class ConnectorButton(QGraphicsPathItem):
         if event.button() != Qt.LeftButton:
             event.accept()
             return
-        self._parent.select_item()
         self._start_link(event)
 
     def _start_link(self, event):
@@ -574,17 +586,17 @@ class ExecutionIcon(QGraphicsEllipseItem):
         self._parent = parent
         self._execution_state = "not started"
         self._text_item = QGraphicsTextItem(self)
-        font = QFont('Font Awesome 5 Free Solid')
+        font = QFont(TOOLBOX_FONT.family)
         self._text_item.setFont(font)
         parent_rect = parent.rect()
         self.setRect(0, 0, 0.5 * parent_rect.width(), 0.5 * parent_rect.height())
-        self.setPen(Qt.NoPen)
+        self.setPen(Qt.PenStyle.NoPen)
         # pylint: disable=undefined-variable
-        self.normal_brush = qApp.palette().window()
-        self.selected_brush = qApp.palette().highlight()
+        self.normal_brush = QApplication.palette().window()
+        self.selected_brush = QApplication.palette().highlight()
         self.setBrush(self.normal_brush)
         self.setAcceptHoverEvents(True)
-        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=False)
+        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, enabled=False)
         self.hide()
 
     def item_name(self):
@@ -606,6 +618,10 @@ class ExecutionIcon(QGraphicsEllipseItem):
         self._execution_state = "waiting for dependencies"
         self._repaint(self._CLOCK, QColor("orange"))
 
+    def mark_execution_ignored(self):
+        self._execution_state = "not started"
+        self.hide()
+
     def mark_execution_started(self):
         self._execution_state = "in progress"
         self._repaint(self._CHECK, QColor("orange"))
@@ -644,8 +660,8 @@ class ExclamationIcon(QGraphicsTextItem):
         """
         super().__init__(parent)
         self._parent = parent
-        self._notifications = list()
-        font = QFont('Font Awesome 5 Free Solid')
+        self._notifications = []
+        font = QFont(TOOLBOX_FONT.family)
         font.setPixelSize(self.FONT_SIZE_PIXELS)
         self.setFont(font)
         self.setDefaultTextColor(QColor("red"))
@@ -653,7 +669,7 @@ class ExclamationIcon(QGraphicsTextItem):
         doc = self.document()
         doc.setDocumentMargin(0)
         self.setAcceptHoverEvents(True)
-        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=False)
+        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, enabled=False)
         self.hide()
 
     def clear_notifications(self):
@@ -661,6 +677,14 @@ class ExclamationIcon(QGraphicsTextItem):
         self._notifications.clear()
         self.hide()
 
+    def clear_other_notifications(self, subtext):
+        """Remove notifications that don't include the given subtext."""
+        k = next((i for i, text in enumerate(self._notifications) if subtext not in text), None)
+        if k is not None:
+            self._notifications.pop(k)
+            if not self._notifications:
+                self.hide()
+
     def add_notification(self, text):
         """Add a notification."""
         self._notifications.append(text)
@@ -706,7 +730,7 @@ class RankIcon(QGraphicsTextItem):
         self._parent = parent
         self._rect = parent.component_rect
         self.bg = QGraphicsPathItem(self)
-        bg_brush = QApplication.palette().brush(QPalette.ToolTipBase)
+        bg_brush = QBrush(QColor(Qt.GlobalColor.white))
         self.bg.setBrush(bg_brush)
         self.bg.setFlag(QGraphicsItem.ItemStacksBehindParent)
         self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=False)
