@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,22 +10,17 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains ProjectUpgrader class used in upgrading and converting projects
-and project dicts from earlier versions to the latest version.
-
-:authors: P. Savolainen (VTT)
-:date:   8.11.2019
-"""
-
-import shutil
-import os
-import json
+"""Contains ProjectUpgrader class used in upgrading and converting projects
+and project dicts from earlier versions to the latest version."""
 import copy
+import json
+import os
+import shutil
 from PySide6.QtWidgets import QFileDialog, QMessageBox
-from spine_engine.utils.serialization import serialize_path, deserialize_path
+from spine_engine.utils.serialization import deserialize_path, serialize_path
 from .config import LATEST_PROJECT_VERSION, PROJECT_FILENAME
 from .helpers import home_dir
+from .project_settings import ProjectSettings
 
 
 class ProjectUpgrader:
@@ -55,22 +51,36 @@ class ProjectUpgrader:
                 f"Opening project <b>{project_dir}</b> failed. The project's version is {v}, while "
                 f"this version of Spine Toolbox supports project versions up to and "
                 f"including {LATEST_PROJECT_VERSION}. To open this project, you should "
-                f"upgrade Spine Toolbox"
+                f"upgrade Spine Toolbox."
             )
             return False
         if v < LATEST_PROJECT_VERSION:
+            if not self.confirm_upgrade(project_dir):
+                return False
             # Back up project.json file before upgrading
             if not self.backup_project_file(project_dir, v):
-                self._toolbox.msg_error.emit("Upgrading project failed")
+                self._toolbox.msg_error.emit(f"Upgrading project <b>{project_dir}</b> failed")
                 return False
             upgraded_dict = self.upgrade_to_latest(v, project_dict, project_dir)
             # Force save project dict to project.json
             if not self.force_save(upgraded_dict, project_dir):
-                self._toolbox.msg_error.emit("Upgrading project failed")
+                self._toolbox.msg_error.emit(f"Upgrading project <b>{project_dir}</b> failed")
                 return False
             return upgraded_dict
         return project_dict
 
+    def confirm_upgrade(self, project_dir):
+        """Asks user whether to upgrade the project to a new version."""
+        button = QMessageBox.question(
+            self._toolbox,
+            "Upgrade project?",
+            f"Project <b>{project_dir}</b> needs an upgrade to work "
+            f"with this version of Spine Toolbox. <br><br>Upgrade project?",
+        )
+        if button == QMessageBox.StandardButton.Yes:
+            return True
+        return False
+
     def upgrade_to_latest(self, v, project_dict, project_dir):
         """Upgrades the given project dictionary to the latest version.
 
@@ -82,8 +92,8 @@ class ProjectUpgrader:
         Returns:
             dict: Upgraded project dictionary
         """
-        # TODO: Fix upgrade_vx_to_vx() methods so they do not depend on self._toolbox.item_factories because these are
-        # TODO: going to change
+        # Note: upgrade_vx_to_vx() methods should not depend on self._toolbox.item_factories
+        # because these are likely to change
         while v < LATEST_PROJECT_VERSION:
             if v == 1:
                 project_dict = self.upgrade_v1_to_v2(project_dict, self._toolbox.item_factories)
@@ -103,6 +113,12 @@ class ProjectUpgrader:
                 project_dict = self.upgrade_v8_to_v9(project_dict)
             elif v == 9:
                 project_dict = self.upgrade_v9_to_v10(project_dict)
+            elif v == 10:
+                project_dict = self.upgrade_v10_to_v11(project_dict)
+            elif v == 11:
+                project_dict = self.upgrade_v11_to_v12(project_dict)
+            elif v == 12:
+                project_dict = self.upgrade_v12_to_v13(project_dict)
             v += 1
             self._toolbox.msg_success.emit(f"Project upgraded to version {v}")
         return project_dict
@@ -125,16 +141,16 @@ class ProjectUpgrader:
         Returns:
             dict: Version 2 project dictionary
         """
-        new = dict()
+        new = {}
         new["version"] = 2
         new["name"] = old["project"]["name"]
         new["description"] = old["project"]["description"]
-        new["specifications"] = dict()
+        new["specifications"] = {}
         new["specifications"]["Tool"] = old["project"]["tool_specifications"]
         new["connections"] = old["project"]["connections"]
         # Change 'objects' to 'items' and remove all 'short name' entries
         # Also stores item_dict under their name and not under category
-        items = dict()
+        items = {}
         for category in old["objects"].keys():
             for item_name in old["objects"][category].keys():
                 old["objects"][category][item_name].pop("short name", "")  # Remove 'short name'
@@ -149,7 +165,7 @@ class ProjectUpgrader:
                     item_type = "GdxExporter"
                 v2_item_dict = factories[item_type].item_class().upgrade_v1_to_v2(item_name, v1_item_dict)
                 items[item_name] = v2_item_dict  # Store items using their name as key
-        return dict(project=new, items=items)
+        return {"project": new, "items": items}
 
     def upgrade_v2_to_v3(self, old, project_dir, factories):
         """Upgrades version 2 project dictionary to version 3.
@@ -201,9 +217,9 @@ class ProjectUpgrader:
                     new["items"][item_name] = old_item_dict
             if item_type == "Importer":
                 mappings = old_item_dict.get("mappings")
-                # Sanitize old mappings, as we use to do in Importer.from_dict
+                # Sanitize old mappings, as we used to do in Importer.from_dict
                 if mappings is None:
-                    mappings = list()
+                    mappings = []
                 # Convert table_types and table_row_types keys to int since json always has strings as keys.
                 for _, mapping in mappings:
                     table_types = mapping.get("table_types", {})
@@ -221,7 +237,7 @@ class ProjectUpgrader:
                 # Make item specs from sanitized mappings
                 for k, (label, mapping) in enumerate(mappings):
                     spec_name = self.make_unique_importer_specification_name(item_name, label, k)
-                    spec = dict(name=spec_name, item_type="Importer", mapping=mapping)
+                    spec = {"name": spec_name, "item_type": "Importer", "mapping": mapping}
                     spec_path = os.path.join(project_dir, spec_name + ".json")
                     # FIXME: Let's try and handle write errors here...
                     with open(spec_path, "w") as fp:
@@ -333,8 +349,8 @@ class ProjectUpgrader:
         """
 
         def fix_file_selection(item_dict):
-            old_selection = item_dict.get("file_selection", list())
-            new_selection = list()
+            old_selection = item_dict.get("file_selection", [])
+            new_selection = []
             for path, selected in old_selection:
                 deserialized = deserialize_path(path, project_dir)
                 if deserialized.startswith("{") and deserialized.endswith("}"):
@@ -344,8 +360,8 @@ class ProjectUpgrader:
             item_dict["file_selection"] = new_selection
 
         def fix_cmd_line_args(item_dict):
-            old_args = item_dict.get("cmd_line_args", list())
-            new_args = list()
+            old_args = item_dict.get("cmd_line_args", [])
+            new_args = []
             for arg in old_args:
                 deserialized = deserialize_path(arg, project_dir)
                 if deserialized.startswith("{") and deserialized.endswith("}"):
@@ -363,7 +379,7 @@ class ProjectUpgrader:
             fix_file_selection(import_dict)
         gimlet_dicts = [item_dict for item_dict in new["items"].values() if item_dict["type"] == "Gimlet"]
         for gimlet_dict in gimlet_dicts:
-            gimlet_dict["file_selection"] = gimlet_dict.pop("selections", list())
+            gimlet_dict["file_selection"] = gimlet_dict.pop("selections", [])
             fix_file_selection(gimlet_dict)
             fix_cmd_line_args(gimlet_dict)
         tool_dicts = [item_dict for item_dict in new["items"].values() if item_dict["type"] == "Tool"]
@@ -487,13 +503,13 @@ class ProjectUpgrader:
         """
         new = copy.deepcopy(old)
         new["project"]["version"] = 10
-        names_to_remove = list()  # Gimlet and GdxExporter item names
+        names_to_remove = []  # Gimlet and GdxExporter item names
         # Get Gimlet and GdxExporter names and remove connections
         for name, item_dict in new["items"].items():
             if item_dict["type"] in ["Gimlet", "GdxExporter"]:
                 names_to_remove.append(name)
         # Get list of connections to remove
-        connections_to_remove = list()
+        connections_to_remove = []
         for conn in new["project"]["connections"]:
             for name_to_remove in names_to_remove:
                 if name_to_remove in conn["from"] or name_to_remove in conn["to"]:
@@ -505,6 +521,62 @@ class ProjectUpgrader:
             new["items"].pop(name)
         return new
 
+    @staticmethod
+    def upgrade_v10_to_v11(old):
+        """Upgrades version 10 project dictionary to version 11.
+
+        Changes:
+            1. Add ["project"]["settings"] key
+
+        Args:
+            old (dict): Version 10 project dictionary
+
+        Returns:
+            dict: Version 11 project dictionary
+        """
+        new = copy.deepcopy(old)
+        new["project"]["version"] = 11
+        new["project"]["settings"] = ProjectSettings().to_dict()
+        return new
+
+    @staticmethod
+    def upgrade_v11_to_v12(old):
+        """Upgrades version 11 project dictionary to version 12.
+
+        Changes:
+            1. Julia's execution settings are now Tool Spec settings instead of global settings
+            Execution settings are local user settings so this only updates the project version
+            to make sure that these projects cannot be opened with an older Toolbox version.
+
+        Args:
+            old (dict): Version 11 project dictionary
+
+        Returns:
+            dict: Version 12 project dictionary
+        """
+        new = copy.deepcopy(old)
+        new["project"]["version"] = 12
+        return new
+
+    @staticmethod
+    def upgrade_v12_to_v13(old):
+        """Upgrades version 12 project dictionary to version 13.
+
+        Changes:
+            1. Connections now have enabled filter types field.
+            Old projects should open just fine so this only updates the project version
+            to make sure that these projects cannot be opened with an older Toolbox version.
+
+        Args:
+            old (dict): Version 12 project dictionary
+
+        Returns:
+            dict: Version 13 project dictionary
+        """
+        new = copy.deepcopy(old)
+        new["project"]["version"] = 13
+        return new
+
     @staticmethod
     def make_unique_importer_specification_name(importer_name, label, k):
         return f"{importer_name} - {os.path.basename(label['path'])} - {k}"
@@ -529,8 +601,8 @@ class ProjectUpgrader:
         # Check if the selected directory is already a project directory and ask if overwrite is ok
         if os.path.isdir(os.path.join(answer, ".spinetoolbox")):
             msg = (
-                "Directory \n\n{0}\n\nalready contains a Spine Toolbox project."
-                "\n\nWould you like to overwrite it?".format(answer)
+                f"Directory \n\n{answer}\n\nalready contains a Spine Toolbox project."
+                f"\n\nWould you like to overwrite it?"
             )
             message_box = QMessageBox(
                 QMessageBox.Icon.Question,
@@ -546,13 +618,23 @@ class ProjectUpgrader:
         return answer  # New project directory
 
     def is_valid(self, v, p):
-        """Checks given project dict if it is valid for given version."""
+        """Checks given project dict if it is valid for given version.
+
+        Args:
+            v (int): project version to validate against
+            p (dict): project dictionary
+
+        Returns:
+            bool: True if project is valid, False otherwise
+        """
         if v == 1:
             return self.is_valid_v1(p)
         if 2 <= v <= 8:
             return self.is_valid_v2_to_v8(p, v)
         if 9 <= v <= 10:
             return self.is_valid_v9_to_v10(p)
+        if 11 <= v <= 13:
+            return self.is_valid_v11_to_v12(p)
         raise NotImplementedError(f"No validity check available for version {v}")
 
     def is_valid_v1(self, p):
@@ -567,10 +649,10 @@ class ProjectUpgrader:
         Returns:
             bool: True if project is a valid version 1 project, False if it is not
         """
-        if "project" not in p.keys():
+        if "project" not in p:
             self._toolbox.msg_error.emit("Invalid project.json file. Key 'project' not found.")
             return False
-        if "objects" not in p.keys():
+        if "objects" not in p:
             self._toolbox.msg_error.emit("Invalid project.json file. Key 'objects' not found.")
             return False
         required_project_keys = ["version", "name", "description", "tool_specifications", "connections"]
@@ -584,7 +666,7 @@ class ProjectUpgrader:
             return False
         for req_key in required_project_keys:
             if req_key not in project:
-                self._toolbox.msg_error.emit("Invalid project.json file. Key {0} not found.".format(req_key))
+                self._toolbox.msg_error.emit(f"Invalid project.json file. Key {req_key} not found.")
                 return False
         # Check types in project dict
         if not project["version"] == 1:
@@ -614,10 +696,10 @@ class ProjectUpgrader:
         Returns:
             bool: True if project is a valid version 2 to version 8 project, False if it is not
         """
-        if "project" not in p.keys():
+        if "project" not in p:
             self._toolbox.msg_error.emit("Invalid project.json file. Key 'project' not found.")
             return False
-        if "items" not in p.keys():
+        if "items" not in p:
             self._toolbox.msg_error.emit("Invalid project.json file. Key 'items' not found.")
             return False
         required_project_keys = ["version", "name", "description", "specifications", "connections"]
@@ -631,11 +713,11 @@ class ProjectUpgrader:
             return False
         for req_key in required_project_keys:
             if req_key not in project:
-                self._toolbox.msg_error.emit("Invalid project.json file. Key {0} not found.".format(req_key))
+                self._toolbox.msg_error.emit(f"Invalid project.json file. Key {req_key} not found.")
                 return False
         # Check types in project dict
         if not project["version"] == v:
-            self._toolbox.msg_error.emit("Invalid project version:'{0}'".format(project["version"]))
+            self._toolbox.msg_error.emit(f"Invalid project version:'{project['version']}'")
             return False
         if not isinstance(project["name"], str) or not isinstance(project["description"], str):
             self._toolbox.msg_error.emit("Invalid project.json file. 'name' and 'description' must be strings.")
@@ -660,10 +742,10 @@ class ProjectUpgrader:
         Returns:
             bool: True if project is a valid version 9 and 10 project, False otherwise
         """
-        if "project" not in p.keys():
+        if "project" not in p:
             self._toolbox.msg_error.emit("Invalid project.json file. Key 'project' not found.")
             return False
-        if "items" not in p.keys():
+        if "items" not in p:
             self._toolbox.msg_error.emit("Invalid project.json file. Key 'items' not found.")
             return False
         required_project_keys = ["version", "description", "specifications", "connections"]
@@ -677,10 +759,33 @@ class ProjectUpgrader:
             return False
         for req_key in required_project_keys:
             if req_key not in project:
-                self._toolbox.msg_error.emit("Invalid project.json file. Key {0} not found.".format(req_key))
+                self._toolbox.msg_error.emit(f"Invalid project.json file. Key {req_key} not found.")
                 return False
         return True
 
+    def is_valid_v11_to_v12(self, p):
+        """Checks that the given project JSON dictionary contains
+        a valid version 11 or 12 Spine Toolbox project. Valid meaning, that
+        it contains all required keys and values are of the correct
+        type.
+
+        Args:
+            p (dict): Project information JSON
+
+        Returns:
+            bool: True if project is a valid version 11 or 12 project, False otherwise
+        """
+        if "project" not in p:
+            self._toolbox.msg_error.emit("Invalid project.json file. Key 'project' not found.")
+            return False
+        if "settings" not in p["project"]:
+            self._toolbox.msg_error.emit("Invalid project.json file. Key 'items' not found in 'project'.")
+            return False
+        if not isinstance(p["project"]["settings"], dict):
+            self._toolbox.msg_error.emit("Invalid project.json file. 'settings' must be a dict.")
+            return False
+        return True
+
     def backup_project_file(self, project_dir, v):
         """Makes a backup copy of project.json file."""
         src = os.path.join(project_dir, ".spinetoolbox", PROJECT_FILENAME)
