@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -9,9 +10,4 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Package for handling the client part of executing projects on Spine Engine Server.
-
-:authors: P. Pääkkönen (VTT), P. Savolainen (VTT)
-:date:    02.09.2021
-"""
+"""Package for handling the client part of executing projects on Spine Engine Server."""
