@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,21 +10,16 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Client for exchanging messages between the toolbox and the Spine Engine Server.
-:author: P. Pääkkönen (VTT), P. Savolainen (VTT)
-:date:   02.09.2021
-"""
-
+"""Client for exchanging messages between the toolbox and the Spine Engine Server."""
+from enum import Enum
+import json
 import os
+import random
+import time
 import zmq
 import zmq.auth
-import time
-import random
-import json
-from enum import Enum
-from spine_engine.server.util.server_message import ServerMessage
 from spine_engine.exception import RemoteEngineInitFailed
+from spine_engine.server.util.server_message import ServerMessage
 
 
 class ClientSecurityModel(Enum):
@@ -58,9 +54,8 @@ class EngineClient:
             # implementation below based on https://github.com/zeromq/pyzmq/blob/main/examples/security/stonehouse.py
             # prepare folders
             base_dir = sec_folder
-            secret_keys_dir = os.path.join(base_dir, 'private_keys')
-            keys_dir = os.path.join(base_dir, 'certificates')
-            public_keys_dir = os.path.join(base_dir, 'public_keys')
+            secret_keys_dir = os.path.join(base_dir, "private_keys")
+            public_keys_dir = os.path.join(base_dir, "public_keys")
             # We need two certificates, one for the client and one for
             # the server. The client must know the server's public key
             # to make a CURVE connection.
@@ -124,16 +119,13 @@ class EngineClient:
         event = self.dealer_socket.poll(timeout=timeout)
         if event == 0:
             raise RemoteEngineInitFailed("Timeout expired. Pinging the server failed.")
-        else:
-            msg = self.dealer_socket.recv_multipart()
-            response = ServerMessage.parse(msg[1])
-            response_id = int(response.getId())  # Check that request ID matches the response ID
-            if not response_id == random_id:
-                raise RemoteEngineInitFailed(
-                    f"Ping failed. Request Id '{random_id}' does not " f"match reply Id '{response_id}'"
-                )
-            stop_time_ms = round(time.time() * 1000.0)  # debugging
-        return
+        msg = self.dealer_socket.recv_multipart()
+        response = ServerMessage.parse(msg[1])
+        response_id = int(response.getId())  # Check that request ID matches the response ID
+        if not response_id == random_id:
+            raise RemoteEngineInitFailed(
+                f"Ping failed. Request Id '{random_id}' does not match reply Id '{response_id}'"
+            )
 
     def set_start_time(self):
         """Sets a start time for an operation. Call get_elapsed_time() after
@@ -192,6 +184,17 @@ class EngineClient:
         response_server_message = ServerMessage.parse(response[1])
         return response_server_message.getData()
 
+    def answer_prompt(self, job_id, prompter_id, answer):
+        """Sends a request to answer a prompt from the DAG that is managed by this client.
+
+        Args:
+            job_id (str): Job Id on server to stop
+            prompter_id (int)
+            answer
+        """
+        req = ServerMessage("answer_prompt", job_id, json.dumps((prompter_id, answer)), None)
+        self.socket.send_multipart([req.to_bytes()])
+
     def download_files(self, q):
         """Pulls files from server until b'END' is received."""
         i = 0
@@ -201,7 +204,7 @@ class EngineClient:
                 if i > 0:
                     q.put(("server_status_msg", {"msg_type": "neutral", "text": f"Downloaded {i} files"}))
                 break
-            elif rcv[0] == b"incoming_file":
+            if rcv[0] == b"incoming_file":
                 q.put(
                     ("server_status_msg", {"msg_type": "warning", "text": "Downloading file " + rcv[1].decode("utf-8")})
                 )
@@ -327,7 +330,7 @@ class EngineClient:
                 break
             yield json.loads(rcv[0].decode("utf-8"))
         pull_socket.close()
-        completed_msg = self.dealer_socket.recv_multipart()  # Get the final 'completed' msg
+        _ = self.dealer_socket.recv_multipart()  # Get the final 'completed' msg
 
     def get_elapsed_time(self):
         """Returns the elapsed time between now and when self.start_time was set.
@@ -338,12 +341,11 @@ class EngineClient:
         t = round(time.time() * 1000.0) - self.start_time  # ms
         if t <= 1000:
             return str(t) + " ms"
-        elif 1000 < t < 60000:  # 1 < t < 60 s
+        if 1000 < t < 60000:  # 1 < t < 60 s
             return str(t / 1000) + " s"
-        else:
-            m = (t / 1000) / 60
-            s = (t / 1000) % 60
-            return str(m) + " min " + str(s) + " s"
+        m = int((t / 1000) / 60)
+        s = (t / 1000) % 60
+        return str(m) + " min " + str(s) + " s"
 
     def close(self):
         """Closes client sockets, context and thread."""
