@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -8,12 +9,10 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-"""
-Color constants for models.
 
-:authors: A. Soininen (VTT)
-:date:    30.3.2022
-"""
+"""Color constants for models."""
 from PySide6.QtGui import QColor
 
+PIVOT_TABLE_HEADER_COLOR = QColor("#efefef")
 FIXED_FIELD_COLOR = QColor("lightGray")
+SELECTED_COLOR = QColor.fromString("paleturquoise")
