@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,15 +10,9 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains base class for metadata table models associated functionality.
-
-:author: A. Soininen (VTT)
-:date:   25.4.2022
-"""
+"""Contains base class for metadata table models associated functionality."""
 from enum import IntEnum, unique
 from operator import itemgetter
-
 from PySide6.QtCore import QAbstractTableModel, QModelIndex, Qt, Signal
 from spinetoolbox.helpers import rows_to_row_count_tuples
 from .colors import FIXED_FIELD_COLOR
@@ -54,7 +49,7 @@ class MetadataTableModelBase(QAbstractTableModel):
         """
         Args:
             db_mngr (SpineDBManager): database manager
-            db_maps (Iterable of DatabaseMappingBase): database maps
+            db_maps (Iterable of DatabaseMapping): database maps
             db_editor (SpineDBEditor): DB editor
         """
         super().__init__(db_editor)
@@ -129,10 +124,10 @@ class MetadataTableModelBase(QAbstractTableModel):
     def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         column = index.column()
         row = index.row()
-        if role == Qt.ItemDataRole.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole or role == Qt.ItemDataRole.EditRole:
             if column == Column.DB_MAP:
                 db_map = self._data[row][column] if row < len(self._data) else self._adder_row[column]
-                return db_map.codename if db_map is not None else ""
+                return self._db_mngr.name_registry.display_name(db_map.sa_url) if db_map is not None else ""
             return self._data[row][column] if row < len(self._data) else self._adder_row[column]
         if (
             role == Qt.ItemDataRole.BackgroundRole
@@ -213,7 +208,7 @@ class MetadataTableModelBase(QAbstractTableModel):
         columns = []
         previous_values = []
         data_length = len(self._data)
-        available_codenames = {db_map.codename: db_map for db_map in self._db_maps}
+        available_codenames = self._db_mngr.name_registry.map_display_names_to_db_maps(self._db_maps)
         reserved = self._reserved_metadata()
         for index, value in zip(indexes, values):
             if not self.flags(index) & Qt.ItemIsEditable:
@@ -258,9 +253,9 @@ class MetadataTableModelBase(QAbstractTableModel):
             else:
                 metadata_to_add.setdefault(db_map, []).append({"name": name, "value": value})
         if metadata_to_add:
-            self._db_mngr.add_metadata(metadata_to_add)
+            self._db_mngr.add_items("metadata", metadata_to_add)
         if metadata_to_update:
-            self._db_mngr.update_metadata(metadata_to_update)
+            self._db_mngr.update_items("metadata", metadata_to_update)
         if rows:
             top_left = self.index(min(rows), min(columns))
             bottom_right = self.index(max(rows), max(columns))
@@ -366,9 +361,9 @@ class MetadataTableModelBase(QAbstractTableModel):
         for db_map, items in db_map_data.items():
             unique_identifiers = {}
             for item in items:
-                unique_identifiers.setdefault(item[self._ITEM_NAME_KEY], {})[
-                    item[self._ITEM_VALUE_KEY]
-                ] = self._ids_from_added_item(item)
+                unique_identifiers.setdefault(item[self._ITEM_NAME_KEY], {})[item[self._ITEM_VALUE_KEY]] = (
+                    self._ids_from_added_item(item)
+                )
             for i, row in enumerate(self._data):
                 if row[Column.DB_MAP] != db_map:
                     continue
@@ -445,35 +440,33 @@ class MetadataTableModelBase(QAbstractTableModel):
                     self._data = self._data[:row] + self._data[row + count :]
                     self.endRemoveRows()
 
-    def sort(self, column, order=Qt.AscendingOrder):
+    def sort(self, column, order=Qt.SortOrder.AscendingOrder):
         if not self._data or column < 0:
             return
 
         def db_map_sort_key(row):
             db_map = row[Column.DB_MAP]
-            return db_map.codename if db_map is not None else ""
+            return self._db_mngr.name_registry.display_name(db_map.sa_url) if db_map is not None else ""
 
         sort_key = itemgetter(column) if column != Column.DB_MAP else db_map_sort_key
-        self._data.sort(key=sort_key, reverse=order == Qt.DescendingOrder)
+        self._data.sort(key=sort_key, reverse=order == Qt.SortOrder.DescendingOrder)
         top_left = self.index(0, 0)
         bottom_right = self.index(len(self._data) - 1, Column.DB_MAP)
         self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.BackgroundRole])
 
-    def _find_db_map(self, codename):
-        """Finds database mapping with given codename.
+    def _find_db_map(self, name):
+        """Finds database mapping with given name.
 
         Args:
-            codename (str): database mapping's code name
+            name (str): database mapping's name
 
         Returns:
-            DiffDatabaseMapping: database mapping or None if not found
+            DatabaseMapping: database mapping or None if not found
         """
-        match = None
-        for db_map in self._db_maps:
-            if codename == db_map.codename:
-                match = db_map
-                break
-        return match
+        return next(
+            iter(db_map for db_map in self._db_maps if name == self._db_mngr.name_registry.display_name(db_map.sa_url)),
+            None,
+        )
 
     def _reserved_metadata(self):
         """Collects metadata names and values that are already in database.
@@ -494,3 +487,9 @@ class MetadataTableModelBase(QAbstractTableModel):
                 continue
             reserved.setdefault(db_map, {})[name] = value
         return reserved
+
+    def begin_paste(self):
+        pass
+
+    def end_paste(self):
+        pass
