@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,14 +10,10 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Provides PivotModel.
-
-:author: P. Vennström (VTT)
-:date:   1.11.2018
-"""
-
+"""Provides PivotModel."""
+from collections.abc import Callable, Mapping, Sequence
 import operator
+from typing import Optional, Union
 from ...helpers import tuple_itemgetter
 
 
@@ -30,7 +27,8 @@ class PivotModel:
         self.pivot_columns = ()  # current selected columns indexes
         self.pivot_frozen = ()  # current filtered frozen indexes
         self.frozen_value = ()  # current selected value of index_frozen
-        self._key_getter = None  # operator.itemgetter placeholder used to translate pivot to keys in _data
+        # operator.itemgetter placeholder used to translate pivot to keys in _data
+        self._key_getter: Optional[Callable[[Union[Mapping, Sequence]], tuple]] = None
         self._row_data_header = []  # header values for row data
         self._column_data_header = []  # header values for column data
 
@@ -42,10 +40,10 @@ class PivotModel:
             columns = ()
             frozen = ()
             frozen_value = ()
-        self.pivot_rows = None
-        self.pivot_columns = None
-        self.pivot_frozen = None
-        self.frozen_value = None
+        self.pivot_rows = ()
+        self.pivot_columns = ()
+        self.pivot_frozen = ()
+        self.frozen_value = ()
         # create data dict with keys as long as index_ids
         self._data = data
         self.index_values = dict(zip(top_left_headers, zip(*data.keys())))
@@ -83,11 +81,8 @@ class PivotModel:
         self._data.update(addable_data)
         if not any(self.frozen_value):
             first = next(iter(self._data), None)
-            if first is None:
-                self.frozen_value = [None] * range(len(self.pivot_frozen))
-            else:
-                frozen_getter = self._index_key_getter(self.pivot_frozen)
-                self.frozen_value = frozen_getter(first)
+            frozen_getter = self._index_key_getter(self.pivot_frozen)
+            self.frozen_value = frozen_getter(first)
         self.index_values = dict(zip(self.index_ids, zip(*self._data.keys())))
         old_row_count = len(self._row_data_header)
         old_column_count = len(self._column_data_header)
@@ -108,6 +103,18 @@ class PivotModel:
         removed_column_count = old_column_count - len(self._column_data_header)
         return removed_row_count, removed_column_count
 
+    def frozen_values(self, data):
+        """Collects frozen values from data.
+
+        Args:
+            data (dict): pivot model data
+
+        Returns:
+            set of tuple: frozen values
+        """
+        frozen_getter = self._index_key_getter(self.pivot_frozen)
+        return {frozen_getter(item) for item in data}
+
     def _check_pivot(self, rows, columns, frozen, frozen_value):
         """Checks if given pivot is valid.
 
@@ -141,6 +148,8 @@ class PivotModel:
             Callable: an itemgetter
         """
         keys = tuple(self.index_ids.index(i) for i in indexes if i in self.index_ids)
+        if not keys:
+            return lambda _: ()
         return tuple_itemgetter(operator.itemgetter(*keys), len(keys))
 
     def _get_unique_index_values(self, indexes):
@@ -160,12 +169,19 @@ class PivotModel:
             result = {index_getter(k): None for k in self._data if frozen_getter(k) == self.frozen_value}
         else:
             result = {index_getter(k): None for k in self._data}
-        return sorted(
-            (x for x in result if None not in x),
-            key=lambda x: tuple(
-                self.top_left_headers[header_name].header_data(header_id) for header_name, header_id in zip(indexes, x)
-            ),
-        )
+        accepted = {}
+        headers = self.top_left_headers
+        for x in result:
+            sort_keys = []
+            for header_name, header_id in zip(indexes, x):
+                header = headers[header_name]
+                if not header.accepts(header_id):
+                    break
+                sort_key = header.header_data(header_id)
+                sort_keys.append(sort_key if sort_key is not None else "")
+            else:
+                accepted[x] = sort_keys
+        return [item[0] for item in sorted(accepted.items(), key=operator.itemgetter(1))]
 
     def set_pivot(self, rows, columns, frozen, frozen_value):
         """Sets pivot."""
@@ -189,13 +205,27 @@ class PivotModel:
         self._column_data_header = self._get_unique_index_values(self.pivot_columns)
 
     def set_frozen_value(self, value):
-        """Sets values for the frozen indexes."""
-        if len(value) != len(self.pivot_frozen):
-            raise ValueError("'value' must have same length as 'self.pivot_frozen'")
+        """Sets values for the frozen indexes.
+
+        Args:
+            value (tuple of str):
+        """
         if value == self.frozen_value:
             return
+        if len(value) != len(self.pivot_frozen):
+            raise ValueError("'value' must have same length as 'self.pivot_frozen'")
         self.set_pivot(self.pivot_rows, self.pivot_columns, self.pivot_frozen, value)
 
+    def set_frozen(self, frozen):
+        """Sets the frozen names without resetting the pivot.
+
+        Args:
+            frozen (Iterable of str)
+        """
+        if len(frozen) != len(self.frozen_value):
+            raise ValueError("'frozen' must have same length as 'self.frozen_value'")
+        self.pivot_frozen = tuple(frozen)
+
     def get_pivoted_data(self, row_mask, column_mask):
         """Returns data for indexes in row_mask and column_mask.
 
@@ -234,7 +264,7 @@ class PivotModel:
             return len(self.pivot_rows) * (None,)
         if row == 0:
             return ()
-        raise IndexError('index out of range for current row pivot')
+        raise IndexError("index out of range for current row pivot")
 
     def column_key(self, column):
         if self.pivot_columns:
@@ -243,7 +273,7 @@ class PivotModel:
             return len(self.pivot_columns) * (None,)
         if column == 0:
             return ()
-        raise IndexError('index out of range for current column pivot')
+        raise IndexError("index out of range for current column pivot")
 
     @property
     def rows(self):
