@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -8,13 +9,9 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-"""
-Models to represent things in a tree.
 
-:authors: M. Marin (KTH)
-:date:    1.0.2020
-"""
-from PySide6.QtCore import Qt, QModelIndex
+"""Models to represent things in a tree."""
+from PySide6.QtCore import QModelIndex, Qt
 from spinetoolbox.mvcmodels.minimal_tree_model import MinimalTreeModel
 from .tree_item_utility import StandardTreeItem
 
@@ -27,12 +24,13 @@ class TreeModelBase(MinimalTreeModel):
         Args:
             db_editor (SpineDBEditor)
             db_mngr (SpineDBManager)
-            *db_maps: DiffDatabaseMapping instances
+            *db_maps: DatabaseMapping instances
         """
         super().__init__(db_editor)
         self.db_editor = db_editor
         self.db_mngr = db_mngr
         self.db_maps = db_maps
+        self.destroyed.connect(lambda _: self._invisible_root_item.tear_down_recursively())
 
     def columnCount(self, parent=QModelIndex()):
         """Returns the number of columns under the given parent. Always 2.
@@ -50,19 +48,14 @@ class TreeModelBase(MinimalTreeModel):
     def build_tree(self):
         """Builds tree."""
         self.beginResetModel()
+        self._invisible_root_item.tear_down_recursively()
         self._invisible_root_item = StandardTreeItem(self)
         self.endResetModel()
         for db_map in self.db_maps:
             db_item = self._make_db_item(db_map)
             self._invisible_root_item.append_children([db_item])
-            db_item.append_children(self._top_children())
 
-    @staticmethod
-    def _make_db_item(db_map):
-        raise NotImplementedError()
-
-    @staticmethod
-    def _top_children():
+    def _make_db_item(self, db_map):
         raise NotImplementedError()
 
     @staticmethod
