@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,35 +10,21 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Classes for custom QTreeView.
-
-:author: M. Marin (KTH)
-:date:   25.4.2018
-"""
-import pickle
-
-from PySide6.QtWidgets import QApplication, QMenu, QAbstractItemView
-from PySide6.QtCore import Signal, Slot, Qt, QEvent, QTimer, QModelIndex, QItemSelection
-from PySide6.QtGui import QMouseEvent, QIcon
-
+"""Classes for custom QTreeViews and QTreeWidgets."""
+from PySide6.QtCore import QEvent, QItemSelection, QModelIndex, QTimer, Signal, Slot
+from PySide6.QtGui import QIcon
+from PySide6.QtWidgets import QAbstractItemView, QApplication, QHeaderView, QMenu
+from spinetoolbox.helpers import CharIconEngine, busy_effect
 from spinetoolbox.widgets.custom_qtreeview import CopyPasteTreeView
-from spinetoolbox.helpers import busy_effect, CharIconEngine
-from spinetoolbox.widgets.custom_qwidgets import ResizingViewMixin
-from .custom_delegates import ScenarioDelegate, ToolFeatureDelegate, AlternativeDelegate, ParameterValueListDelegate
-from .scenario_generator import ScenarioGenerator
 from ..mvcmodels import mime_types
 from ..mvcmodels.alternative_item import AlternativeItem
-from ..mvcmodels.scenario_item import ScenarioDBItem, ScenarioAlternativeItem, ScenarioItem
+from ..mvcmodels.scenario_item import ScenarioAlternativeItem, ScenarioDBItem, ScenarioItem
+from .custom_delegates import AddEntityButtonDelegate, AlternativeDelegate, ParameterValueListDelegate, ScenarioDelegate
+from .scenario_generator import ScenarioGenerator
 
 
-class ResizableTreeView(ResizingViewMixin, CopyPasteTreeView):
-    def _do_resize(self):
-        self.resizeColumnToContents(0)
-
-
-class EntityTreeView(ResizableTreeView):
-    """Tree view base class for object and relationship tree views."""
+class EntityTreeView(CopyPasteTreeView):
+    """Tree view for entity classes and entities."""
 
     tree_selection_changed = Signal(dict)
 
@@ -47,27 +34,37 @@ class EntityTreeView(ResizableTreeView):
             parent (QWidget): parent widget
         """
         super().__init__(parent=parent)
+        self.setItemDelegate(AddEntityButtonDelegate(self))
+        self.setRootIsDecorated(False)
         self._context_item = None
         self._selected_indexes = {}
         self._menu = QMenu(self)
         self._spine_db_editor = None
         self._fully_expand_action = None
         self._fully_collapse_action = None
-        self._add_relationship_classes_action = None
-        self._add_relationships_action = None
-        self._manage_relationships_action = None
+        self._add_entity_classes_action = None
+        self._add_entities_action = None
+        self._add_entity_group_action = None
+        self._duplicate_entity_action = None
+        self._manage_elements_action = None
+        self._manage_members_action = None
+        self._select_superclass_action = None
         self._export_action = None
         self._edit_action = None
         self._remove_action = None
         self._cube_plus_icon = QIcon(":/icons/menu_icons/cube_plus.svg")
         self._cube_minus_icon = QIcon(":/icons/menu_icons/cube_minus.svg")
         self._cube_pen_icon = QIcon(":/icons/menu_icons/cube_pen.svg")
-        self._cubes_plus_icon = QIcon(":/icons/menu_icons/cubes_plus.svg")
         self._cubes_pen_icon = QIcon(":/icons/menu_icons/cubes_pen.svg")
         self._fetch_more_timer = QTimer(self)
         self._fetch_more_timer.setSingleShot(True)
         self._fetch_more_timer.setInterval(100)
         self._fetch_more_timer.timeout.connect(self._fetch_more_visible)
+        self._find_next_action = None
+        self._hide_empty_classes_action = None
+        self._entity_index = None
+        self._header = self.header()
+        self._header.setSectionResizeMode(QHeaderView.ResizeMode.ResizeToContents)
 
     def reset(self):
         super().reset()
@@ -83,10 +80,34 @@ class EntityTreeView(ResizableTreeView):
         self._create_context_menu()
         self.connect_signals()
 
+    def set_db_column_visibility(self, visible):
+        """Sets the visibility of the db column"""
+        if visible:
+            self._header.showSection(1)
+        else:
+            self._header.hideSection(1)
+
     def _add_middle_actions(self):
-        """Adds action at the middle of the context menu.
-        Subclasses can reimplement at will.
-        """
+        self._add_entity_classes_action = self._menu.addAction(
+            self._cube_plus_icon, "Add entity classes...", self.add_entity_classes
+        )
+        self._add_entities_action = self._menu.addAction(self._cube_plus_icon, "Add entities...", self.add_entities)
+        self._add_entity_group_action = self._menu.addAction(
+            self._cube_plus_icon, "Add entity group...", self.add_entity_group
+        )
+        self._manage_elements_action = self._menu.addAction(
+            self._cubes_pen_icon, "Manage elements...", self.manage_elements
+        )
+        self._manage_members_action = self._menu.addAction(
+            self._cube_pen_icon, "Manage members...", self.manage_members
+        )
+        self._select_superclass_action = self._menu.addAction(
+            self._cube_pen_icon, "Select superclass...", self.select_superclass
+        )
+        self._menu.addSeparator()
+        self._find_next_action = self._menu.addAction(
+            QIcon(CharIconEngine("\uf141")), "Find next occurrence", self.find_next_entity
+        )
 
     def _create_context_menu(self):
         """Creates a context menu for this view."""
@@ -96,6 +117,9 @@ class EntityTreeView(ResizableTreeView):
         self._menu.addSeparator()
         self._edit_action = self._menu.addAction(self._cube_pen_icon, "Edit...", self.edit_selected)
         self._remove_action = self._menu.addAction(self._cube_minus_icon, "Remove...", self.remove_selected)
+        self._duplicate_entity_action = self._menu.addAction(
+            self._cube_plus_icon, "Duplicate entity", self.duplicate_entity
+        )
         self._menu.addSeparator()
         self._export_action = self._menu.addAction(
             QIcon(":/icons/menu_icons/database-export.svg"), "Export", self.export_selected
@@ -107,6 +131,14 @@ class EntityTreeView(ResizableTreeView):
         self._fully_collapse_action = self._menu.addAction(
             QIcon(CharIconEngine("\uf100")), "Fully collapse", self.fully_collapse
         )
+        self._menu.addSeparator()
+        self._hide_empty_classes_action = self._menu.addAction("Hide empty classes", self.toggle_hide_empty_classes)
+        self._hide_empty_classes_action.setCheckable(True)
+        self._hide_empty_classes_action.setChecked(self.model().hide_empty_classes)
+
+    def toggle_hide_empty_classes(self):
+        self.model().hide_empty_classes = self._hide_empty_classes_action.isChecked()
+        self.model().save_hide_empty_classes()
 
     @Slot(QModelIndex, int, QEvent)
     def edit(self, index, trigger, event):
@@ -119,10 +151,12 @@ class EntityTreeView(ResizableTreeView):
     def connect_signals(self):
         """Connects signals."""
         self.selectionModel().selectionChanged.connect(self._handle_selection_changed)
+        self.doubleClicked.connect(self.find_next_entity)
 
     def rowsInserted(self, parent, start, end):
         super().rowsInserted(parent, start, end)
         self._refresh_selected_indexes()
+        QTimer.singleShot(20, self._do_find_next_entity)  # Keep looking for the next entity after new rows are inserted
 
     def rowsRemoved(self, parent, start, end):
         super().rowsRemoved(parent, start, end)
@@ -151,10 +185,11 @@ class EntityTreeView(ResizableTreeView):
     @Slot(QItemSelection, QItemSelection)
     def _handle_selection_changed(self, selected, deselected):
         """Classifies selection by item type and emits signal."""
+        if self._spine_db_editor.clear_tree_selections:
+            self._spine_db_editor.clear_tree_selections = False
+            self._spine_db_editor._clear_all_other_selections(self)
         self._spine_db_editor.refresh_copy_paste_actions()
         self._refresh_selected_indexes()
-        if not self.selectionModel().hasSelection():
-            return
         self.tree_selection_changed.emit(self._selected_indexes)
 
     def _refresh_selected_indexes(self):
@@ -167,12 +202,6 @@ class EntityTreeView(ResizableTreeView):
             item = model.item_from_index(index)
             self._selected_indexes.setdefault(item.item_type, {})[index] = None
 
-    def clear_any_selections(self):
-        """Clears the selection if any."""
-        selection_model = self.selectionModel()
-        if selection_model.hasSelection():
-            selection_model.clearSelection()
-
     @busy_effect
     def fully_expand(self):
         """Expands selected indexes and all their children."""
@@ -191,6 +220,12 @@ class EntityTreeView(ResizableTreeView):
             for item in model.visit_all(index):
                 self.collapse(model.index_from_item(item))
 
+    def collapse(self, index):
+        """Overridden to prevent the collapse of the root item"""
+        if not index.parent().isValid():
+            return
+        super().collapse(index)
+
     def export_selected(self):
         """Exports data from selected indexes using the connected Spine db editor."""
         self._spine_db_editor.export_selected(self._selected_indexes)
@@ -199,9 +234,6 @@ class EntityTreeView(ResizableTreeView):
         """Removes selected indexes using the connected Spine db editor."""
         self._spine_db_editor.remove_entity_tree_items(self._selected_indexes)
 
-    def manage_relationships(self):
-        self._spine_db_editor.show_manage_relationships_form(self._context_item)
-
     def contextMenuEvent(self, event):
         """Shows context menu.
 
@@ -215,180 +247,94 @@ class EntityTreeView(ResizableTreeView):
         self.update_actions_availability()
         self._menu.exec(event.globalPos())
 
+    def mouseDoubleClickEvent(self, event):
+        """Overridden to not allow collapsing of the root item by double click."""
+        pos = self.viewport().mapFromGlobal(event.globalPos())
+        index = self.indexAt(pos)
+        model = index.model()
+        if model:
+            item = model.item_from_index(index)
+            if item.item_type == "root":
+                event.ignore()
+                return
+        super().mouseDoubleClickEvent(event)
+
     def mousePressEvent(self, event):
-        """Overrides selection behaviour if the user has selected sticky selection in Settings.
-        If sticky selection is enabled, multiple-selection is enabled when selecting items in the Object tree.
-        Pressing the Ctrl-button down, enables single selection.
-
-        Args:
-            event (QMouseEvent)
-        """
-        sticky_selection = self._spine_db_editor.qsettings.value("appSettings/stickySelection", defaultValue="false")
-        if sticky_selection == "false":
-            super().mousePressEvent(event)
-            return
-        local_pos = event.localPos()
-        window_pos = event.windowPos()
-        screen_pos = event.screenPos()
-        button = event.button()
-        buttons = event.buttons()
-        modifiers = event.modifiers()
-        if modifiers & Qt.ControlModifier:
-            modifiers &= ~Qt.ControlModifier
-        else:
-            modifiers |= Qt.ControlModifier
-        source = event.source()
-        new_event = QMouseEvent(
-            QEvent.MouseButtonPress, local_pos, window_pos, screen_pos, button, buttons, modifiers, source
-        )
+        new_event = self._spine_db_editor.handle_mousepress(self, event) if self._spine_db_editor else event
         super().mousePressEvent(new_event)
 
-    def _add_relationship_actions(self):
-        self._add_relationship_classes_action = self._menu.addAction(
-            self._cubes_plus_icon, "Add relationship classes", self.add_relationship_classes
-        )
-        self._add_relationships_action = self._menu.addAction(
-            self._cubes_plus_icon, "Add relationships", self.add_relationships
-        )
-        self._manage_relationships_action = self._menu.addAction(
-            self._cubes_pen_icon, "Manage relationships", self.manage_relationships
-        )
-
     def update_actions_availability(self):
         """Updates the visible property of actions according to whether or not they apply to given item."""
         item = self._context_item
         item_has_children = item.has_children()
         self._fully_expand_action.setEnabled(item_has_children)
         self._fully_collapse_action.setEnabled(item_has_children)
-        self._add_relationships_action.setEnabled(item.item_type == "relationship_class")
-        self._manage_relationships_action.setEnabled(item.item_type in ("root", "relationship_class"))
+        self._add_entity_classes_action.setEnabled(item.item_type in ("root", "entity_class"))
+        self._add_entities_action.setEnabled(
+            item.item_type in ("root", "entity_class")
+            or (item.item_type == "entity" and item.parent_item.item_type == "entity_class")
+        )
+        self._add_entity_group_action.setEnabled(item.item_type == "entity_class")
+        self._duplicate_entity_action.setEnabled(
+            item.item_type == "entity" and not item.is_group and not item.element_name_list
+        )
+        self._manage_members_action.setEnabled(item.item_type == "entity" and item.is_group)
+        self._select_superclass_action.setEnabled(item.item_type == "entity_class")
+        self._manage_elements_action.setEnabled(
+            item.item_type == "root" or (item.item_type == "entity_class" and item.has_dimensions)
+        )
         read_only = item.item_type in ("root", "members")
         self._export_action.setEnabled(not read_only)
         self._edit_action.setEnabled(not read_only)
         self._remove_action.setEnabled(not read_only)
+        self._find_next_action.setEnabled(
+            item.item_type == "entity" and item.parent_item.parent_item.item_type == "entity_class"
+        )
 
     def edit_selected(self):
         """Edits all selected indexes using the connected Spine db editor."""
         self._spine_db_editor.edit_entity_tree_items(self._selected_indexes)
 
+    def add_entity_classes(self):
+        self._spine_db_editor.show_add_entity_classes_form(self._context_item)
 
-class ObjectTreeView(EntityTreeView):
-    """Custom QTreeView class for the object tree in SpineDBEditor."""
+    def add_entities(self):
+        self._spine_db_editor.show_add_entities_form(self._context_item)
 
-    def __init__(self, parent):
-        """
-        Args:
-            parent (QWidget): parent widget
-        """
-        super().__init__(parent=parent)
-        self._add_objects_action = None
-        self._add_object_classes_action = None
-        self._add_object_group_action = None
-        self._manage_members_action = None
-        self._duplicate_object_action = None
-        self._find_next_action = None
-        self._relationship_index = None
-
-    def update_actions_availability(self):
-        super().update_actions_availability()
-        item = self._context_item
-        self._add_object_classes_action.setEnabled(item.item_type == "root")
-        self._add_objects_action.setEnabled(item.item_type in ("root", "object_class"))
-        self._add_object_group_action.setEnabled(item.item_type == "object_class")
-        self._add_relationship_classes_action.setEnabled(item.item_type in ("root", "object_class"))
-        self._manage_members_action.setEnabled(item.item_type == "members")
-        self._duplicate_object_action.setEnabled(item.item_type == "object" and not item.is_group)
-        self._find_next_action.setEnabled(item.item_type == "relationship")
-
-    def _add_middle_actions(self):
-        self._add_object_classes_action = self._menu.addAction(
-            self._cube_plus_icon, "Add object classes", self.add_object_classes
-        )
-        self._add_objects_action = self._menu.addAction(self._cube_plus_icon, "Add objects", self.add_objects)
-        self._add_relationship_actions()
-        self._menu.addSeparator()
-        self._find_next_action = self._menu.addAction(
-            QIcon(CharIconEngine("\uf141")), "Find next relationship", self.find_next_relationship
-        )
-        self._add_object_group_action = self._menu.addAction(
-            self._cube_plus_icon, "Add object group", self.add_object_group
-        )
-        self._manage_members_action = self._menu.addAction(self._cube_pen_icon, "Manage members", self.manage_members)
-        self._duplicate_object_action = self._menu.addAction(
-            self._cube_plus_icon, "Duplicate object", self.duplicate_object
-        )
-
-    def connect_signals(self):
-        super().connect_signals()
-        self.doubleClicked.connect(self.find_next_relationship)
-
-    def rowsInserted(self, parent, start, end):
-        super().rowsInserted(parent, start, end)
-        self._do_find_next_relationship()
-
-    def add_object_classes(self):
-        self._spine_db_editor.show_add_object_classes_form()
-
-    def add_objects(self):
-        self._spine_db_editor.show_add_objects_form(self._context_item)
-
-    def add_relationship_classes(self):
-        self._spine_db_editor.show_add_relationship_classes_form(self._context_item)
-
-    def add_relationships(self):
-        self._spine_db_editor.show_add_relationships_form(self._context_item)
-
-    def find_next_relationship(self):
+    def find_next_entity(self):
         """Finds the next occurrence of the relationship at the current index and expands it."""
-        self._relationship_index = self.currentIndex()
-        self._do_find_next_relationship()
+        self._entity_index = self.currentIndex()
+        self._do_find_next_entity()
 
-    def _do_find_next_relationship(self):
-        if self._relationship_index is None:
+    def _do_find_next_entity(self):
+        if self._entity_index is None:
             return
-        next_index = self.model().find_next_relationship_index(self._relationship_index)
+        next_index = self.model().find_next_entity_index(self._entity_index)
         if not next_index:
             return
-        self._relationship_index = None
+        self._entity_index = None
         self.setCurrentIndex(next_index)
         self.scrollTo(next_index)
         self.expand(next_index)
 
-    def duplicate_object(self):
+    def duplicate_entity(self):
         """Duplicates the object at the current index using the connected Spine db editor."""
-        object_item = self.currentIndex().internalPointer()
-        self._spine_db_editor.duplicate_object(object_item)
+        self._spine_db_editor.duplicate_entity(self._context_item)
 
-    def add_object_group(self):
-        index = self.currentIndex()
-        item = index.internalPointer()
-        self._spine_db_editor.show_add_object_group_form(item)
+    def add_entity_group(self):
+        self._spine_db_editor.show_add_entity_group_form(self._context_item)
+
+    def manage_elements(self):
+        self._spine_db_editor.show_manage_elements_form(self._context_item)
 
     def manage_members(self):
-        index = self.currentIndex()
-        item = index.internalPointer().parent_item
-        self._spine_db_editor.show_manage_members_form(item)
+        self._spine_db_editor.show_manage_members_form(self._context_item)
+
+    def select_superclass(self):
+        self._spine_db_editor.show_select_superclass_form(self._context_item)
 
 
-class RelationshipTreeView(EntityTreeView):
-    """Custom QTreeView class for the relationship tree in SpineDBEditor."""
-
-    def _add_middle_actions(self):
-        self._add_relationship_actions()
-
-    def update_actions_availability(self):
-        super().update_actions_availability()
-        item = self._context_item
-        self._add_relationship_classes_action.setEnabled(item.item_type == "root")
-
-    def add_relationship_classes(self):
-        self._spine_db_editor.show_add_relationship_classes_form(self._context_item)
-
-    def add_relationships(self):
-        self._spine_db_editor.show_add_relationships_form(self._context_item)
-
-
-class ItemTreeView(ResizableTreeView):
+class ItemTreeView(CopyPasteTreeView):
     """Base class for all non-entity tree views."""
 
     def __init__(self, parent):
@@ -399,6 +345,12 @@ class ItemTreeView(ResizableTreeView):
         super().__init__(parent=parent)
         self._spine_db_editor = None
         self._menu = QMenu(self)
+        header = self.header()
+        header.setSectionResizeMode(QHeaderView.ResizeMode.ResizeToContents)
+
+    def mousePressEvent(self, event):
+        new_event = self._spine_db_editor.handle_mousepress(self, event) if self._spine_db_editor else event
+        super().mousePressEvent(new_event)
 
     def rowsInserted(self, parent, start, end):
         super().rowsInserted(parent, start, end)
@@ -450,71 +402,18 @@ class ItemTreeView(ResizableTreeView):
         """Refreshes copy and paste actions enabled state."""
         self._spine_db_editor.refresh_copy_paste_actions()
 
-
-class ToolFeatureTreeView(ItemTreeView):
-    """Custom QTreeView class for tools and features in SpineDBEditor."""
-
-    def connect_spine_db_editor(self, spine_db_editor):
-        """see base class"""
-        super().connect_spine_db_editor(spine_db_editor)
-        delegate = ToolFeatureDelegate(self._spine_db_editor)
-        delegate.data_committed.connect(self.model().setData)
-        self.setItemDelegateForColumn(0, delegate)
-
-    def remove_selected(self):
-        """See base class."""
-        if not self.selectionModel().hasSelection():
+    def _clear_trees(self):
+        """Clears selections from all other trees if such clearing is set as pending."""
+        if not self._spine_db_editor.clear_tree_selections:
             return
-        db_map_typed_data_to_rm = {}
-        items = [self.model().item_from_index(index) for index in self.selectionModel().selectedIndexes()]
-        for db_item in self.model()._invisible_root_item.children:
-            db_map_typed_data_to_rm[db_item.db_map] = {
-                "feature": set(),
-                "tool": set(),
-                "tool_feature": set(),
-                "tool_feature_method": set(),
-            }
-            for feat_item in reversed(db_item.child(0).children[:-1]):
-                if feat_item in items:
-                    db_map_typed_data_to_rm[db_item.db_map]["feature"].add(feat_item.id)
-            for tool_item in reversed(db_item.child(1).children[:-1]):
-                if tool_item in items:
-                    db_map_typed_data_to_rm[db_item.db_map]["tool"].add(tool_item.id)
-                    continue
-                tool_feat_root_item = tool_item.child(0)
-                for tool_feat_item in reversed(tool_feat_root_item.children):
-                    if tool_feat_item in items:
-                        db_map_typed_data_to_rm[db_item.db_map]["tool_feature"].add(tool_feat_item.id)
-                        continue
-                    tool_feat_meth_root_item = tool_feat_item.child(1)
-                    if tool_feat_meth_root_item is None:
-                        continue
-                    for tool_feat_meth_item in reversed(tool_feat_meth_root_item.children):
-                        if tool_feat_meth_item in items:
-                            db_map_typed_data_to_rm[db_item.db_map]["tool_feature_method"].add(tool_feat_meth_item.id)
-        self.model().db_mngr.remove_items(db_map_typed_data_to_rm)
-        self.selectionModel().clearSelection()
-
-    def update_actions_availability(self, item):
-        """See base class."""
-
-    def dragMoveEvent(self, event):
-        super().dragMoveEvent(event)
-        index = self.indexAt(event.position().toPoint())
-        item = self.model().item_from_index(index)
-        if item and item.item_type == "tool":
-            self.expand(index)
-
-    def dragEnterEvent(self, event):
-        super().dragEnterEvent(event)
-        if event.source() is self:
-            event.accept()
+        self._spine_db_editor.clear_tree_selections = False
+        self._spine_db_editor._clear_all_other_selections(self)
 
 
 class AlternativeTreeView(ItemTreeView):
     """Custom QTreeView for the alternative tree in SpineDBEditor."""
 
-    alternative_selection_changed = Signal(dict)
+    alternative_selection_changed = Signal(object)
 
     def __init__(self, parent):
         """
@@ -522,7 +421,7 @@ class AlternativeTreeView(ItemTreeView):
             parent (QWidget): parent widget
         """
         super().__init__(parent=parent)
-        self._selected_alternative_ids = dict()
+        self._selected_alternative_ids = {}
         self._generate_scenarios_action = None
 
     @property
@@ -572,12 +471,15 @@ class AlternativeTreeView(ItemTreeView):
     @Slot(QItemSelection, QItemSelection)
     def _handle_selection_changed(self, selected, deselected):
         """Emits alternative_selection_changed with the current selection."""
+        self._clear_trees()
         selected_db_map_alt_ids = self._db_map_alt_ids_from_selection(selected)
         deselected_db_map_alt_ids = self._db_map_alt_ids_from_selection(deselected)
         for db_map, ids in deselected_db_map_alt_ids.items():
-            self._selected_alternative_ids[db_map].difference_update(ids)
+            if ids:
+                self._selected_alternative_ids[db_map].difference_update(ids)
         for db_map, ids in selected_db_map_alt_ids.items():
-            self._selected_alternative_ids.setdefault(db_map, set()).update(ids)
+            if ids:
+                self._selected_alternative_ids.setdefault(db_map, set()).update(ids)
         self.alternative_selection_changed.emit(self._selected_alternative_ids)
 
     def remove_selected(self):
@@ -606,7 +508,7 @@ class AlternativeTreeView(ItemTreeView):
         if not isinstance(item, AlternativeItem):
             return
         included_ids = set()
-        alternatives = list()
+        alternatives = []
         db_map = item.db_map
         for id_ in self._selected_alternative_ids.get(db_map, ()):
             if id_ not in included_ids:
@@ -674,7 +576,7 @@ class AlternativeTreeView(ItemTreeView):
 class ScenarioTreeView(ItemTreeView):
     """Custom QTreeView for the scenario tree in SpineDBEditor."""
 
-    alternative_selection_changed = Signal(dict)
+    scenario_selection_changed = Signal(object)
 
     def __init__(self, parent):
         """
@@ -682,16 +584,12 @@ class ScenarioTreeView(ItemTreeView):
             parent (QWidget): parent widget
         """
         super().__init__(parent=parent)
-        self._selected_alternative_ids = dict()
+        self._selected_scenario_ids = {}
         self._duplicate_scenario_action = None
 
-    @property
-    def selected_alternative_ids(self):
-        return self._selected_alternative_ids
-
     def reset(self):
         super().reset()
-        self._selected_alternative_ids.clear()
+        self._selected_scenario_ids.clear()
 
     def connect_signals(self):
         """Connects signals."""
@@ -732,15 +630,20 @@ class ScenarioTreeView(ItemTreeView):
 
     @Slot(QItemSelection, QItemSelection)
     def _handle_selection_changed(self, selected, deselected):
-        """Emits alternative_selection_changed with the current selection."""
-        self._selected_alternative_ids.clear()
-        for index in self.selectionModel().selectedRows(column=0):
+        """Emits scenario_selection_changed with the current selection."""
+        self._clear_trees()
+        self._selected_scenario_ids.clear()
+        for index in self.selectionModel().selectedIndexes():
             item = self.model().item_from_index(index)
             if isinstance(item, ScenarioItem) and item.id is not None:
-                self._selected_alternative_ids.setdefault(item.db_map, set()).update(item.alternative_id_list)
+                self._selected_scenario_ids.setdefault("scenario", {}).setdefault(item.db_map, {}).update(
+                    {item.id: item.alternative_id_list}
+                )
             elif isinstance(item, ScenarioAlternativeItem) and item.alternative_id is not None:
-                self._selected_alternative_ids.setdefault(item.db_map, set()).add(item.alternative_id)
-        self.alternative_selection_changed.emit(self._selected_alternative_ids)
+                self._selected_scenario_ids.setdefault("scenario_alternative", {}).setdefault(item.db_map, set()).add(
+                    item.alternative_id
+                )
+        self.scenario_selection_changed.emit(self._selected_scenario_ids)
 
     def remove_selected(self):
         """See base class."""
