@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,15 +10,9 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Classes for custom QDialogs to add edit and remove database items.
-
-:author: M. Marin (KTH)
-:date:   13.5.2018
-"""
-from PySide6.QtWidgets import QCheckBox, QDialogButtonBox, QWidget
+"""Classes for custom QDialogs to add edit and remove database items."""
 from PySide6.QtCore import Signal, Slot
-
+from PySide6.QtWidgets import QCheckBox, QDialogButtonBox, QWidget
 from spinetoolbox.widgets.custom_qwidgets import SelectDatabaseItemsDialog
 from spinetoolbox.widgets.select_database_items import add_check_boxes
 
@@ -27,11 +22,12 @@ class _SelectDatabases(QWidget):
 
     checked_state_changed = Signal(int)
 
-    def __init__(self, db_maps, checked_states, parent):
+    def __init__(self, db_maps, checked_states, db_name_registry, parent):
         """
         Args:
-            db_maps (tuple of DatabaseMappingBase): database maps
+            db_maps (tuple of DatabaseMapping): database maps
             checked_states (dict, optional): mapping from item name to check state boolean
+            db_name_registry (NameRegistry): database display name registry
             parent (QWidget): parent widget
         """
         super().__init__(parent)
@@ -39,7 +35,9 @@ class _SelectDatabases(QWidget):
 
         self._ui = Ui_Form()
         self._ui.setupUi(self)
-        self._check_boxes = {db_map: QCheckBox(db_map.codename, self) for db_map in db_maps}
+        self._check_boxes = {
+            db_map: QCheckBox(db_name_registry.display_name(db_map.sa_url), self) for db_map in db_maps
+        }
         add_check_boxes(
             self._check_boxes,
             checked_states,
@@ -85,7 +83,9 @@ class MassSelectItemsDialog(SelectDatabaseItemsDialog):
         database_checked_states = (
             stored_state["databases"] if stored_state is not None else {db_map: True for db_map in db_maps}
         )
-        self._database_check_boxes_widget = _SelectDatabases(tuple(db_maps), database_checked_states, self)
+        self._database_check_boxes_widget = _SelectDatabases(
+            tuple(db_maps), database_checked_states, db_mngr.name_registry, self
+        )
         self._database_check_boxes_widget.checked_state_changed.connect(self._handle_check_box_state_changed)
         self._ui.root_layout.insertWidget(0, self._database_check_boxes_widget)
 
@@ -116,7 +116,7 @@ class MassRemoveItemsDialog(MassSelectItemsDialog):
         Args:
             parent (SpineDBEditor)
             db_mngr (SpineDBManager)
-            db_maps (DiffDatabaseMapping): the dbs to select items from
+            db_maps (DatabaseMapping): the dbs to select items from
             stored_state (dict, Optional): widget's previous state
         """
         super().__init__(parent, db_mngr, *db_maps, stored_state=stored_state, ok_button_text="Purge")
@@ -138,14 +138,14 @@ class MassExportItemsDialog(MassSelectItemsDialog):
     """A dialog to let users chose items for JSON export."""
 
     _warn_checked_non_data_items = False
-    data_submitted = Signal(dict)
+    data_submitted = Signal(object)
 
     def __init__(self, parent, db_mngr, *db_maps, stored_state=None):
         """
         Args:
             parent (SpineDBEditor)
             db_mngr (SpineDBManager)
-            db_maps (DiffDatabaseMapping): the dbs to select items from
+            db_maps (DatabaseMapping): the dbs to select items from
             stored_state (dict, Optional): widget's previous state
         """
         super().__init__(parent, db_mngr, *db_maps, stored_state=stored_state, ok_button_text="Export")
