@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,39 +10,33 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains custom QHeaderView for the pivot table.
-
-:author: M. Marin (KTH)
-:date:   2.12.2019
-"""
-
-from PySide6.QtCore import Signal, Slot, Qt, QPoint
-from PySide6.QtWidgets import QHeaderView, QMenu
+"""Contains custom QHeaderView for the pivot table."""
+from PySide6.QtCore import QPoint, Qt, Signal, Slot
 from PySide6.QtGui import QAction
-from .tabular_view_header_widget import TabularViewHeaderWidget
-from ...widgets.report_plotting_failure import report_plotting_failure
-from ...widgets.plot_widget import PlotWidget, prepare_plot_in_window_menu
+from PySide6.QtWidgets import QHeaderView, QMenu, QWidget
 from ...plotting import PlottingError, plot_pivot_table_selection
-from ...config import PIVOT_TABLE_HEADER_COLOR
+from ...widgets.plot_widget import PlotWidget, prepare_plot_in_window_menu
+from ...widgets.report_plotting_failure import report_plotting_failure
+from ..mvcmodels.colors import PIVOT_TABLE_HEADER_COLOR
+from .tabular_view_header_widget import TabularViewHeaderWidget
 
 
 class PivotTableHeaderView(QHeaderView):
     """Header view for the pivot table."""
 
-    header_dropped = Signal(object, object)
+    header_dropped = Signal(QWidget, QWidget)
 
     def __init__(self, orientation, area, pivot_table_view):
         """
         Args:
-            orientation (int): Qt.Orientation.Horizontal or Qt.Orientation.Vertical
+            orientation (Qt.Orientation): Qt.Orientation.Horizontal or Qt.Orientation.Vertical
             area (str): which pivot area the header represents: "columns", "rows" or "frozen"
             pivot_table_view (PivotTableView): parent view
         """
         super().__init__(orientation, parent=pivot_table_view)
         self._area = area
         self.setAcceptDrops(True)
-        self.setStyleSheet("QHeaderView::section {background-color: " + PIVOT_TABLE_HEADER_COLOR + ";}")
+        self.setStyleSheet(f"QHeaderView::section {{background-color: {PIVOT_TABLE_HEADER_COLOR.name()};}}")
         self.setSectionsClickable(True)
         self.setVisible(True)
 
@@ -67,7 +62,7 @@ class ParameterValuePivotHeaderView(PivotTableHeaderView):
     def __init__(self, orientation, area, pivot_table_view):
         """
         Args:
-            orientation (int): Qt.Orientation.Horizontal or Qt.Orientation.Vertical
+            orientation (Qt.Orientation): Qt.Orientation.Horizontal or Qt.Orientation.Vertical
             area (str): which pivot area the header represents: "columns", "rows" or "frozen"
             pivot_table_view (PivotTableView): parent view
         """
