@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,18 +10,12 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains SpineEngineManagerBase.
-
-:authors: M. Marin (KTH), P. Pääkkönen (VTT), P. Savolainen (VTT)
-:date:   14.10.2020
-"""
+"""Contains SpineEngineManagerBase."""
+import json
 import queue
 import threading
-import json
-from spinetoolbox.server.engine_client import EngineClient, ClientSecurityModel
-from spine_engine.exception import EngineInitFailed, RemoteEngineInitFailed
 from spine_engine.server.util.event_data_converter import EventDataConverter
+from spinetoolbox.server.engine_client import ClientSecurityModel, EngineClient
 
 
 class SpineEngineManagerBase:
@@ -44,12 +39,12 @@ class SpineEngineManagerBase:
         """Stops a running engine."""
         raise NotImplementedError()
 
-    def answer_prompt(self, item_name, accepted):
+    def answer_prompt(self, prompter_id, answer):
         """Answers prompt.
 
         Args:
-            item_name (str): The item that emitted the prompt
-            accepted (bool): The user's decision.
+            prompter_id (int): The id of the prompter
+            answer: The user's decision.
         """
         raise NotImplementedError()
 
@@ -161,24 +156,26 @@ class LocalSpineEngineManager(SpineEngineManagerBase):
         if self._engine is not None:
             self._engine.stop()
 
-    def answer_prompt(self, item_name, accepted):
-        self._engine.answer_prompt(item_name, accepted)
+    def answer_prompt(self, prompter_id, answer):
+        self._engine.answer_prompt(prompter_id, answer)
 
     def restart_kernel(self, connection_file):
         # pylint: disable=import-outside-toplevel
-        from spine_engine.execution_managers.kernel_execution_manager import get_kernel_manager
+        from spine_engine.execution_managers.kernel_execution_manager import restart_kernel_manager
 
-        km = get_kernel_manager(connection_file)
-        if km is not None:
-            km.restart_kernel(now=True)
+        return restart_kernel_manager(connection_file)
 
     def shutdown_kernel(self, connection_file):
         # pylint: disable=import-outside-toplevel
-        from spine_engine.execution_managers.kernel_execution_manager import pop_kernel_manager
+        from spine_engine.execution_managers.kernel_execution_manager import shutdown_kernel_manager
 
-        km = pop_kernel_manager(connection_file)
-        if km is not None:
-            km.shutdown_kernel(now=True)
+        return shutdown_kernel_manager(connection_file)
+
+    def kernel_managers(self):
+        # pylint: disable=import-outside-toplevel
+        from spine_engine.execution_managers.kernel_execution_manager import n_kernel_managers
+
+        return n_kernel_managers()
 
     def issue_persistent_command(self, persistent_key, command):
         # pylint: disable=import-outside-toplevel
@@ -227,7 +224,10 @@ class RemoteSpineEngineManager(SpineEngineManagerBase):
     """Responsible for remote project execution."""
 
     def __init__(self, job_id=""):
-        """Initializer."""
+        """
+        Args:
+            job_id (str): Job id of ProjectExtractionService for finding the extracted project on server
+        """
         super().__init__()
         self._runner = threading.Thread(name="RemoteSpineEngineManagerRunnerThread", target=self._run)
         self._engine_data = None
@@ -238,12 +238,7 @@ class RemoteSpineEngineManager(SpineEngineManagerBase):
 
     def make_engine_client(self, host, port, security, sec_folder, ping=True):
         """Creates a client for connecting to Spine Engine Server."""
-        try:
-            self.engine_client = EngineClient(host, port, security, sec_folder, ping)
-        except RemoteEngineInitFailed:
-            raise
-        except Exception:
-            raise
+        self.engine_client = EngineClient(host, port, security, sec_folder, ping)
 
     def run_engine(self, engine_data):
         """Makes an engine client for communicating with the engine server.
@@ -319,26 +314,22 @@ class RemoteSpineEngineManager(SpineEngineManagerBase):
                 self.q.put(("server_status_msg", {"msg_type": "success", "text": f"Execution time: {t}"}))
                 self.q.put(event)
                 break
-            elif event[0] == "server_execution_error":
+            if event[0] == "server_execution_error":
                 # spine engine raised an exception during execution
                 self.q.put(("server_status_msg", {"msg_type": "fail", "text": f"{event[0]: {event[1]}}"}))
                 break
-            else:
-                self.q.put(event)
-        self.engine_client.close()
+            self.q.put(event)
 
-    def answer_prompt(self, item_name, accepted):
+    def answer_prompt(self, prompter_id, answer):
         """See base class."""
-        raise NotImplementedError()
+        self.engine_client.answer_prompt(self.exec_job_id, prompter_id, answer)
 
     def restart_kernel(self, connection_file):
         """See base class."""
         # TODO: This does not restart the kernel, only replaces the client. Do kernel_manager.restart_kernel() on server
-        pass
 
     def shutdown_kernel(self, connection_file):
         """See base class."""
-        pass
 
     def is_persistent_command_complete(self, persistent_key, command):
         return self.engine_client.send_is_complete(persistent_key, command)
@@ -356,7 +347,8 @@ class RemoteSpineEngineManager(SpineEngineManagerBase):
 
     def kill_persistent(self, persistent_key):
         """See base class."""
-        return self.engine_client.send_kill_persistent(persistent_key)
+        if self.engine_client is not None:
+            return self.engine_client.send_kill_persistent(persistent_key)
 
     def get_persistent_completions(self, persistent_key, text):
         """See base class."""
