@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,108 +10,88 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains SpineEngineWorker.
-:authors: M. Marin (KTH)
-:date:   14.10.2020
-"""
+"""Contains GUIUpdater and SpineEngineWorker classes."""
 import copy
-from PySide6.QtCore import Signal, Slot, QObject, QThread
-from PySide6.QtWidgets import QMessageBox
+from PySide6.QtCore import QObject, QThread, Signal, Slot
 from spine_engine.exception import EngineInitFailed, RemoteEngineInitFailed
 from spine_engine.spine_engine import ItemExecutionFinishState, SpineEngineState
-from .spine_engine_manager import make_engine_manager, LocalSpineEngineManager
-from .helpers import get_upgrade_db_promt_text
+from spine_engine.utils.helpers import ExecutionDirection
+from .project_item.project_item import ProjectItem
+from .spine_engine_manager import LocalSpineEngineManager, make_engine_manager
+from .widgets.options_dialog import OptionsDialog
 
 
-@Slot(list)
-def _handle_dag_execution_started(project_items):
-    for item in project_items:
-        item.get_icon().execution_icon.mark_execution_waiting()
+class GUIUpdater(QObject):
+    """Contains slots for updating UI widgets based on messages received from the engine worker."""
 
+    @Slot(list)
+    def handle_dag_execution_started(self, project_items):
+        for item in project_items:
+            item.get_icon().execution_icon.mark_execution_waiting()
 
-@Slot(object, object)
-def _handle_node_execution_started(item, direction):
-    icon = item.get_icon()
-    if direction == "FORWARD":
-        icon.execution_icon.mark_execution_started()
-        if hasattr(icon, "animation_signaller"):
-            icon.animation_signaller.animation_started.emit()
+    @Slot(list)
+    def handle_node_execution_ignored(self, project_items):
+        for item in project_items:
+            item.get_icon().execution_icon.mark_execution_ignored()
 
-
-@Slot(object, object, object, object)
-def _handle_node_execution_finished(item, direction, item_state):
-    icon = item.get_icon()
-    if direction == "FORWARD":
-        icon.execution_icon.mark_execution_finished(item_state)
-        if hasattr(icon, "animation_signaller"):
-            icon.animation_signaller.animation_stopped.emit()
-
-
-@Slot(object, str, str)
-def _handle_event_message_arrived(item, filter_id, msg_type, msg_text):
-    item.add_event_message(filter_id, msg_type, msg_text)
-
-
-@Slot(object, str, str)
-def _handle_process_message_arrived(item, filter_id, msg_type, msg_text):
-    item.add_process_message(filter_id, msg_type, msg_text)
-
-
-@Slot(dict, object)
-def _handle_prompt_arrived(prompt, engine_mngr):
-    prompt_type = prompt["type"]
-    if prompt_type == "upgrade_db":
-        url = prompt["url"]
-        current = prompt["current"]
-        expected = prompt["expected"]
-        text, info_text = get_upgrade_db_promt_text(url, current, expected)
-    else:
-        info_text = ""
-        text = prompt["text"]
-    item_name = prompt["item_name"]
-    # pylint: disable=undefined-variable
-    box = QMessageBox(
-        QMessageBox.Icon.Question,
-        item_name,
-        text,
-        buttons=QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
-        parent=qApp.activeWindow(),
-    )
-    if info_text:
-        box.setInformativeText(info_text)
-    answer = box.exec()
-    accepted = answer == QMessageBox.StandardButton.Yes
-    engine_mngr.answer_prompt(item_name, accepted)
-
-
-@Slot(object)
-def _handle_flash_arrived(connection):
-    connection.graphics_item.run_execution_animation()
-
-
-@Slot(list)
-def _mark_all_items_failed(items):
-    """Fails all project items.
-
-    Args:
-        items (list of ProjectItem): project items
-    """
-    for item in items:
+    @Slot(object, object)
+    def handle_node_execution_started(self, item, direction):
         icon = item.get_icon()
-        icon.execution_icon.mark_execution_finished(ItemExecutionFinishState.FAILURE)
-        if hasattr(icon, "animation_signaller"):
-            icon.animation_signaller.animation_stopped.emit()
+        if direction == ExecutionDirection.FORWARD:
+            icon.execution_icon.mark_execution_started()
+            if hasattr(icon, "animation_signaller"):
+                icon.animation_signaller.animation_started.emit()
+
+    @Slot(object, object, object)
+    def handle_node_execution_finished(self, item, direction, item_state):
+        icon = item.get_icon()
+        if direction == ExecutionDirection.FORWARD:
+            icon.execution_icon.mark_execution_finished(item_state)
+            if hasattr(icon, "animation_signaller"):
+                icon.animation_signaller.animation_stopped.emit()
+
+    @Slot(object, str, str, str)
+    def handle_event_message_arrived(self, item, filter_id, msg_type, msg_text):
+        item.add_event_message(filter_id, msg_type, msg_text)
+
+    @Slot(object, str, str, str)
+    def handle_process_message_arrived(self, item, filter_id, msg_type, msg_text):
+        item.add_process_message(filter_id, msg_type, msg_text)
+
+    @Slot(dict, object)
+    def handle_prompt_arrived(self, prompt, engine_mngr, logger=None):
+        prompter_id = prompt["prompter_id"]
+        title, text, option_to_answer, notes, preferred = prompt["data"]
+        answer = OptionsDialog.get_answer(logger, title, text, option_to_answer, notes=notes, preferred=preferred)
+        engine_mngr.answer_prompt(prompter_id, answer)
+
+    @Slot(object)
+    def handle_flash_arrived(self, connection):
+        connection.graphics_item.run_execution_animation()
+
+    @Slot(list)
+    def mark_all_items_failed(self, items):
+        """Fails all project items.
+
+        Args:
+            items (list of ProjectItem): project items
+        """
+        for item in items:
+            icon = item.get_icon()
+            icon.execution_icon.mark_execution_finished(ItemExecutionFinishState.FAILURE)
+            if hasattr(icon, "animation_signaller"):
+                icon.animation_signaller.animation_stopped.emit()
 
 
 class SpineEngineWorker(QObject):
-    finished = Signal()
+    finished = Signal(object)
+    _mark_items_ignored = Signal(list)
     _dag_execution_started = Signal(list)
     _node_execution_started = Signal(object, object)
     _node_execution_finished = Signal(object, object, object)
     _event_message_arrived = Signal(object, str, str, str)
     _process_message_arrived = Signal(object, str, str, str)
-    _prompt_arrived = Signal(dict, object)
+    _prompt_arrived = Signal(dict, object, object)
     _flash_arrived = Signal(object)
     _all_items_failed = Signal(list)
 
@@ -123,11 +104,12 @@ class SpineEngineWorker(QObject):
             project_items (dict): mapping from project item name to :class:`ProjectItem`
             connections (dict): mapping from jump name to :class:`LoggingConnection` or :class:`LoggingJump`
             logger (LoggerInterface): a logger
-            job_id: Job Id for remote execution
+            job_id (str): Job id for remote execution
         """
         super().__init__()
         self._engine_data = engine_data
         exec_remotely = engine_data["settings"].get("engineSettings/remoteExecutionEnabled", "false") == "true"
+        self._job_id = job_id
         self._engine_mngr = make_engine_manager(exec_remotely, job_id)
         self.dag = dag
         self.dag_identifier = dag_identifier
@@ -142,6 +124,11 @@ class SpineEngineWorker(QObject):
         self._thread = QThread()
         self.moveToThread(self._thread)
         self._thread.started.connect(self.do_work)
+        self._gui_updater = GUIUpdater()
+
+    @property
+    def job_id(self):
+        return self._job_id
 
     @property
     def engine_data(self):
@@ -165,11 +152,11 @@ class SpineEngineWorker(QObject):
         """
         self._engine_data = engine_data
 
-    @Slot(object, str, str)
+    @Slot(ProjectItem, str, str, str)
     def _handle_event_message_arrived_silent(self, item, filter_id, msg_type, msg_text):
         self.event_messages.setdefault(msg_type, []).append(msg_text)
 
-    @Slot(object, str, str)
+    @Slot(object, str, str, str)
     def _handle_process_message_arrived_silent(self, item, filter_id, msg_type, msg_text):
         self.process_messages.setdefault(msg_type, []).append(msg_text)
 
@@ -187,13 +174,14 @@ class SpineEngineWorker(QObject):
             self._event_message_arrived.connect(self._handle_event_message_arrived_silent)
             self._process_message_arrived.connect(self._handle_process_message_arrived_silent)
             return
-        self._dag_execution_started.connect(_handle_dag_execution_started)
-        self._node_execution_started.connect(_handle_node_execution_started)
-        self._node_execution_finished.connect(_handle_node_execution_finished)
-        self._event_message_arrived.connect(_handle_event_message_arrived)
-        self._process_message_arrived.connect(_handle_process_message_arrived)
-        self._prompt_arrived.connect(_handle_prompt_arrived)
-        self._flash_arrived.connect(_handle_flash_arrived)
+        self._mark_items_ignored.connect(self._gui_updater.handle_node_execution_ignored)
+        self._dag_execution_started.connect(self._gui_updater.handle_dag_execution_started)
+        self._node_execution_started.connect(self._gui_updater.handle_node_execution_started)
+        self._node_execution_finished.connect(self._gui_updater.handle_node_execution_finished)
+        self._event_message_arrived.connect(self._gui_updater.handle_event_message_arrived)
+        self._process_message_arrived.connect(self._gui_updater.handle_process_message_arrived)
+        self._prompt_arrived.connect(self._gui_updater.handle_prompt_arrived)
+        self._flash_arrived.connect(self._gui_updater.handle_flash_arrived)
 
     def start(self, silent=False):
         """Connects log signals.
@@ -203,10 +191,43 @@ class SpineEngineWorker(QObject):
                 but saved in internal dicts.
         """
         self._connect_log_signals(silent)
-        self._all_items_failed.connect(_mark_all_items_failed)
-        self._dag_execution_started.emit(list(self._project_items.values()))
+        self._all_items_failed.connect(self._gui_updater.mark_all_items_failed)
+        included_items, ignored_items = self._included_and_ignored_items()
+        self._dag_execution_started.emit(included_items)
+        self._mark_items_ignored.emit(ignored_items)
         self._thread.start()
 
+    def _included_and_ignored_items(self):
+        """Returns two lists, where the first one contains project items that are about to be executed and the second
+        one contains project items that are about to be ignored."""
+        included = self._included_items(self.engine_data["execution_permits"], self.engine_data["connections"])
+        included_items = [item_dict for name, item_dict in self._project_items.items() if name in included]
+        ignored_items = [item_dict for name, item_dict in self._project_items.items() if name not in included]
+        return included_items, ignored_items
+
+    def _included_items(self, permitted_items, connections):
+        """Collects a list of project item names that are going to be executed in this DAG
+        based on execution permits and connections in the DAG.
+
+        Args:
+            permitted_items (dict): Mapping of item names to bool. True items have been selected by user for execution.
+            connections (list): Serialized connections
+
+        Returns:
+            list: Project item names
+        """
+        selected_items = [name for name, item_dict in self._project_items.items() if permitted_items[name]]
+        i_names = set()  # Names of items that are connected to permitted items
+        for conn in connections:
+            src_item = conn["from"][0]
+            dst_item = conn["to"][0]
+            if src_item in selected_items or dst_item in selected_items:
+                i_names.add(src_item)
+                i_names.add(dst_item)
+        if not i_names:  # Single node DAG with no connections
+            return selected_items
+        return list(i_names)
+
     @Slot()
     def do_work(self):
         """Does the work and emits finished when done."""
@@ -216,7 +237,7 @@ class SpineEngineWorker(QObject):
             self._logger.msg_error.emit(f"Failed to start engine: {error}")
             self._engine_final_state = str(SpineEngineState.FAILED)
             self._all_items_failed.emit(list(self._project_items.values()))
-            self.finished.emit()
+            self.finished.emit(self)
             return
         except RemoteEngineInitFailed as e:
             self._logger.msg_error.emit(
@@ -224,7 +245,7 @@ class SpineEngineWorker(QObject):
             )
             self._engine_final_state = str(SpineEngineState.FAILED)
             self._all_items_failed.emit(list(self._project_items.values()))
-            self.finished.emit()
+            self.finished.emit(self)
             return
         while True:
             event_type, data = self._engine_mngr.get_engine_event()
@@ -232,12 +253,12 @@ class SpineEngineWorker(QObject):
             if event_type == "dag_exec_finished":
                 self._engine_final_state = data
                 break
-            elif event_type == "remote_execution_init_failed" or event_type == "server_init_failed":
+            if event_type in ("remote_execution_init_failed", "server_init_failed"):
                 self._logger.msg_error.emit(f"{data}")
                 self._engine_final_state = str(SpineEngineState.FAILED)
                 self._all_items_failed.emit(list(self._project_items.values()))
                 break
-        self.finished.emit()
+        self.finished.emit(self)
 
     def _process_event(self, event_type, data):
         handler = {
@@ -257,7 +278,7 @@ class SpineEngineWorker(QObject):
         handler(data)
 
     def _handle_prompt(self, prompt):
-        self._prompt_arrived.emit(prompt, self._engine_mngr)
+        self._prompt_arrived.emit(prompt, self._engine_mngr, self._logger)
 
     def _handle_flash(self, flash):
         connection = self._connections[flash["item_name"]]
@@ -312,7 +333,7 @@ class SpineEngineWorker(QObject):
                 msg["filter_id"],
                 msg["kernel_name"],
                 msg["connection_file"],
-                msg.get("connection_file_dict", dict()),
+                msg.get("connection_file_dict", {}),
             )
         elif msg["type"] == "kernel_spec_not_found":
             msg_text = (
@@ -340,41 +361,34 @@ class SpineEngineWorker(QObject):
                 item, msg["filter_id"], "msg", f"*** Starting execution on kernel spec <b>{msg['kernel_name']}</b> ***"
             )
             self._event_message_arrived.emit(item, msg["filter_id"], "msg_warning", "See Console for messages")
+        elif msg["type"] == "kernel_shutdown":
+            self._logger.kernel_shutdown.emit(item, msg["filter_id"])
 
     def _handle_process_msg(self, data):
-        self._do_handle_process_msg(**data)
-
-    def _do_handle_process_msg(self, item_name, filter_id, msg_type, msg_text):
+        item_name = data["item_name"]
         item = self._project_items.get(item_name) or self._connections.get(item_name)
-        self._process_message_arrived.emit(item, filter_id, msg_type, msg_text)
+        self._process_message_arrived.emit(item, data["filter_id"], data["msg_type"], data["msg_text"])
 
     def _handle_event_msg(self, data):
-        self._do_handle_event_msg(**data)
-
-    def _do_handle_event_msg(self, item_name, filter_id, msg_type, msg_text):
+        item_name = data["item_name"]
         item = self._project_items.get(item_name) or self._connections.get(item_name)
-        self._event_message_arrived.emit(item, filter_id, msg_type, msg_text)
+        self._event_message_arrived.emit(item, data["filter_id"], data["msg_type"], data["msg_text"])
 
     def _handle_node_execution_started(self, data):
-        self._do_handle_node_execution_started(**data)
-
-    def _do_handle_node_execution_started(self, item_name, direction):
         """Starts item icon animation when executing forward."""
-        item = self._project_items[item_name]
+        item = self._project_items[data["item_name"]]
         self._executing_items.add(item)
-        self._node_execution_started.emit(item, direction)
+        self._node_execution_started.emit(item, data["direction"])
 
     def _handle_node_execution_finished(self, data):
-        self._do_handle_node_execution_finished(**data)
-
-    def _do_handle_node_execution_finished(self, item_name, direction, state, item_state):
-        item = self._project_items[item_name]
-        if item_state == ItemExecutionFinishState.SUCCESS:
-            self.successful_executions.append((item, direction, state))
+        item = self._project_items[data["item_name"]]
+        direction = data["direction"]
+        if data["item_state"] == ItemExecutionFinishState.SUCCESS:
+            self.successful_executions.append((item, direction))
         self._executing_items.discard(item)
         # NOTE: A single item may seemingly finish multiple times
         # when the execution is stopped by user during filtered execution.
-        self._node_execution_finished.emit(item, direction, item_state)
+        self._node_execution_finished.emit(item, direction, data["item_state"])
 
     def _handle_server_status_msg(self, data):
         if data["msg_type"] == "success":
@@ -393,6 +407,7 @@ class SpineEngineWorker(QObject):
             self._engine_mngr.stop_engine()
         else:
             self._engine_mngr.clean_up()
+        self._gui_updater = None
         self._thread.quit()
         self._thread.wait()
         self._thread.deleteLater()
