@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,13 +10,10 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""Version info for Spine Toolbox package. Inspired by python sys.version and sys.version_info.
-
-:author: P. Savolainen (VTT)
-:date: 8.1.2020
-"""
-
+"""Version info for Spine Toolbox package. Inspired by python sys.version and sys.version_info."""
+import re
 from typing import NamedTuple
+from ._version import version_tuple
 
 
 class VersionInfo(NamedTuple):
@@ -32,18 +30,36 @@ class VersionInfo(NamedTuple):
     def __str__(self) -> str:
         """Create a version string following PEP 440"""
         version = f"{self.major}.{self.minor}.{self.micro}"
-        if self.releaselevel == 'final':  # pylint: disable=no-else-return
+        if self.releaselevel == "final":  # pylint: disable=no-else-return
             return version
-        elif self.releaselevel.startswith('dev'):
+        elif self.releaselevel.startswith("dev"):
             return version + f".dev{self.serial}"
         else:
             return version + f"-{self.releaselevel}.{self.serial}"
 
 
-major = 0
-minor = 6
-micro = 17
-releaselevel = "final"
-serial = 0
+major, minor, micro, *dev = version_tuple
+
+if dev:
+    rel, commit = dev
+    if match := re.search(r"[0-9]+", rel):
+        split = match.span()[0]
+        releaselevel, serial = rel[:split], int(rel[split:])
+
+        # name cleanup
+        del split
+    else:
+        # shouldn't happen
+        releaselevel, serial = rel, 0
+
+    # name cleanup
+    del match, rel
+else:
+    # compat: move away gradually
+    releaselevel, serial = "final", 0
+
+# name cleanup
+del dev
+
 __version_info__ = VersionInfo(major, minor, micro, releaselevel, serial)
 __version__ = str(__version_info__)
