@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,16 +10,10 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Classes for custom QDialogs to add edit and remove database items.
-
-:author: M. Marin (KTH)
-:date:   13.5.2018
-"""
-
-from PySide6.QtWidgets import QDialog, QVBoxLayout, QPlainTextEdit, QDialogButtonBox, QApplication
-from PySide6.QtCore import Slot, Qt
+"""Classes for custom QDialogs to add edit and remove database items."""
+from PySide6.QtCore import Qt, Slot
 from PySide6.QtGui import QAction
+from PySide6.QtWidgets import QApplication, QDialog, QDialogButtonBox, QPlainTextEdit, QVBoxLayout
 
 
 class CommitDialog(QDialog):
@@ -33,20 +28,22 @@ class CommitDialog(QDialog):
         super().__init__(parent)
         self.setWindowFlag(Qt.WindowType.WindowContextHelpButtonHint, False)
         self.setWindowModality(Qt.ApplicationModal)
-        self.commit_msg = None
-        self.setWindowTitle('Commit changes to {}'.format(",".join(db_names)))
+        self.setWindowTitle(f"Commit changes to {','.join(db_names)}")
         form = QVBoxLayout(self)
         form.setContentsMargins(4, 4, 4, 4)
         self.action_accept = QAction(self)
         self.action_accept.setShortcut(QApplication.translate("Dialog", "Ctrl+Return", None, -1))
         self.action_accept.triggered.connect(self.accept)
-        self.action_accept.setEnabled(False)
+        self.action_accept.setEnabled(True)
+        self.commit_msg = "Updated"
         self.commit_msg_edit = QPlainTextEdit(self)
-        self.commit_msg_edit.setPlaceholderText('Commit message \t(press Ctrl+Enter to commit)')
+        self.commit_msg_edit.setPlainText(self.commit_msg)
+        self.commit_msg_edit.selectAll()
+        self.commit_msg_edit.setFocus()
         self.commit_msg_edit.addAction(self.action_accept)
         button_box = QDialogButtonBox()
         button_box.addButton(QDialogButtonBox.StandardButton.Cancel)
-        self.commit_button = button_box.addButton('Commit', QDialogButtonBox.ButtonRole.AcceptRole)
+        self.commit_button = button_box.addButton("Commit", QDialogButtonBox.ButtonRole.AcceptRole)
         button_box.accepted.connect(self.accept)
         button_box.rejected.connect(self.reject)
         form.addWidget(self.commit_msg_edit)
