@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,13 +10,7 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Classes for custom line edits.
-
-:authors: M. Marin (KTH), P. Savolainen (VTT)
-:date:   11.10.2018
-"""
-
+"""Contains a custom line edit."""
 from PySide6.QtWidgets import QLineEdit
 from .custom_qwidgets import UndoRedoMixin
 
