@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,17 +10,10 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Class for a custom QTextBrowser for showing the logs and tool output.
-
-:author: P. Savolainen (VTT)
-:date:   6.2.2018
-"""
-
-from contextlib import contextmanager
+"""Class for a custom QTextBrowser for showing the logs and tool output."""
 from PySide6.QtCore import Slot
-from PySide6.QtGui import QTextCursor, QFontDatabase, QTextBlockFormat, QTextFrameFormat, QBrush, QAction, QPalette
-from PySide6.QtWidgets import QTextBrowser, QMenu
+from PySide6.QtGui import QAction, QBrush, QFontDatabase, QPalette, QTextBlockFormat, QTextCursor, QTextFrameFormat
+from PySide6.QtWidgets import QMenu, QTextBrowser
 from ..config import TEXTBROWSER_SS
 from ..helpers import scrolling_to_bottom
 
@@ -54,7 +48,7 @@ class CustomQTextBrowser(QTextBrowser):
         self._frame_format.setBorder(1)
         self._selected_frame_format = QTextFrameFormat(self._frame_format)
         palette = self.palette()
-        self._selected_frame_format.setBackground(QBrush(palette.color(QPalette.Highlight).darker()))
+        self._selected_frame_format.setBackground(QBrush(palette.color(QPalette.ColorRole.Highlight).darker()))
         self._executions_menu.aboutToShow.connect(self._populate_executions_menu)
         self._executions_menu.triggered.connect(self._select_execution)
 
@@ -127,9 +121,9 @@ class CustomQTextBrowser(QTextBrowser):
 
     @staticmethod
     def _make_log_entry_title(title):
-        return f'<b>{title}</b>'
+        return f"<b>{title}</b>"
 
-    def start_execution(self, timestamp):
+    def make_log_entry_point(self, timestamp):
         """Creates cursors (log entry points) for given items in event log.
 
         Args:
@@ -232,12 +226,13 @@ class CustomQTextBrowser(QTextBrowser):
             frame.setFrameFormat(frame_format)
 
 
-class MonoSpaceFontTextBrowser(CustomQTextBrowser):
+class MonoSpaceFontTextBrowser(QTextBrowser):
     def __init__(self, parent):
         """
         Args:
             parent (QWidget): Parent widget
         """
         super().__init__(parent=parent)
-        font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
+        self.setStyleSheet(TEXTBROWSER_SS)
+        font = QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
         self.setFont(font)
