@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,48 +10,43 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Custom QWidgets for Filtering and Zooming.
-
-:author: P. Vennström (VTT)
-:date:   4.12.2018
-"""
-
-from PySide6.QtWidgets import (
-    QWidget,
-    QVBoxLayout,
-    QHBoxLayout,
-    QMenu,
-    QStyle,
-    QToolBar,
-    QStyleOptionMenuItem,
-    QListView,
-    QLineEdit,
-    QDialogButtonBox,
-    QWidgetAction,
-    QLabel,
-    QFrame,
-    QWizardPage,
-    QToolButton,
-    QPushButton,
-    QSpinBox,
-    QDialog,
-)
-from PySide6.QtCore import Qt, QTimer, Signal, Slot, QSize, QEvent, QRect
+"""Custom QWidgets for Filtering and Zooming."""
+from PySide6.QtCore import QEvent, QRect, QSize, Qt, QTimer, Signal, Slot
 from PySide6.QtGui import (
-    QPainter,
-    QFontMetrics,
-    QKeyEvent,
-    QFontDatabase,
-    QFont,
-    QIntValidator,
-    QKeySequence,
     QAction,
+    QFont,
+    QFontDatabase,
+    QFontMetrics,
+    QIntValidator,
+    QKeyEvent,
+    QKeySequence,
+    QPainter,
     QUndoStack,
 )
+from PySide6.QtWidgets import (
+    QDialog,
+    QDialogButtonBox,
+    QFrame,
+    QHBoxLayout,
+    QLabel,
+    QLineEdit,
+    QListView,
+    QMenu,
+    QPushButton,
+    QSpinBox,
+    QStyle,
+    QStyleOptionMenuItem,
+    QToolBar,
+    QToolButton,
+    QVBoxLayout,
+    QWidget,
+    QWidgetAction,
+    QWizardPage,
+)
+from ..font import TOOLBOX_FONT
+from ..helpers import format_log_message
 from .custom_qtextbrowser import MonoSpaceFontTextBrowser
 from .select_database_items import SelectDatabaseItems
-from ..helpers import format_log_message
 
 
 class ElidedTextMixin:
@@ -122,13 +118,13 @@ class FilterWidget(QWidget):
         # parameters
         self._filter_state = set()
         self._filter_empty_state = None
-        self._search_text = ''
+        self._search_text = ""
         self.search_delay = 200
         # create ui elements
         self._ui_vertical_layout = QVBoxLayout(self)
         self._ui_list = QListView()
         self._ui_edit = QLineEdit()
-        self._ui_edit.setPlaceholderText('Search')
+        self._ui_edit.setPlaceholderText("Search")
         self._ui_edit.setClearButtonEnabled(True)
         self._ui_buttons = QDialogButtonBox(QDialogButtonBox.StandardButton.Cancel | QDialogButtonBox.StandardButton.Ok)
         self._ui_vertical_layout.addWidget(self._ui_edit)
@@ -137,6 +133,7 @@ class FilterWidget(QWidget):
         self._search_timer = QTimer()  # Used to limit search so it doesn't search when typing
         self._search_timer.setSingleShot(True)
         self._filter_model = make_filter_model(*args, **kwargs)
+        self._filter_model.setParent(self)
         self._filter_model.set_list(self._filter_state)
         self._ui_list.setModel(self._filter_model)
         self.connect_signals()
@@ -176,14 +173,14 @@ class FilterWidget(QWidget):
         """Apply current filter and save state."""
         self._filter_model.apply_filter()
         self.save_state()
-        self._ui_edit.setText('')
+        self._ui_edit.setText("")
         self.okPressed.emit()
 
     def _cancel_filter(self):
         """Cancel current edit of filter and set the state to the stored state."""
         self._filter_model.remove_filter()
         self.reset_state()
-        self._ui_edit.setText('')
+        self._ui_edit.setText("")
         self.cancelPressed.emit()
 
     def _filter_list(self):
@@ -243,7 +240,7 @@ class ToolBarWidgetAction(CustomWidgetAction):
         parent.installEventFilter(self)
 
     def eventFilter(self, obj, ev):
-        if ev.type() == QEvent.KeyPress:
+        if ev.type() == QEvent.Type.KeyPress:
             self._parent_key_press_event = QKeyEvent(ev.type(), ev.key(), ev.modifiers())
         return super().eventFilter(obj, ev)
 
@@ -265,7 +262,7 @@ class ToolBarWidgetBase(QWidget):
         tool_bar (QToolBar)
     """
 
-    def __init__(self, text, parent=None):
+    def __init__(self, text, parent=None, io_files=None):
         """Class constructor.
 
         Args:
@@ -276,21 +273,23 @@ class ToolBarWidgetBase(QWidget):
         self._text = text
         self._parent = parent
         layout = QHBoxLayout(self)
-        layout.setContentsMargins(0, 0, 0, 0)
+        if io_files:
+            layout.setContentsMargins(0, 0, 0, 0)
+        else:
+            layout.setContentsMargins(30, 0, 0, 0)
         layout.setSpacing(0)
         self.tool_bar = _MenuToolBar(self)
         layout.addStretch()
         layout.addWidget(self.tool_bar)
-        icon_extent = qApp.style().pixelMetric(
-            QStyle.PixelMetric.PM_SmallIconSize
-        )  # pylint: disable=undefined-variable
+        # pylint: disable=undefined-variable
+        icon_extent = qApp.style().pixelMetric(QStyle.PixelMetric.PM_SmallIconSize)
         self.tool_bar.setIconSize(QSize(icon_extent, icon_extent))
         self.tool_bar.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextUnderIcon)
 
 
 class ToolBarWidget(ToolBarWidgetBase):
-    def __init__(self, text, parent=None):
-        super().__init__(text, parent)
+    def __init__(self, text, parent=None, io_files=None):
+        super().__init__(text, parent, io_files)
         spacing = self.fontMetrics().horizontalAdvance(self._text)  # pylint: disable=undefined-variable
         self.layout().insertSpacing(0, spacing)
 
@@ -313,9 +312,8 @@ class MenuItemToolBarWidget(ToolBarWidgetBase):
         if compact:
             self.tool_bar.setFixedHeight(self.option.rect.height())
         text_width = self.option.fontMetrics.horizontalAdvance(self._text)
-        icon_width = qApp.style().pixelMetric(
-            QStyle.PixelMetric.PM_ToolBarIconSize
-        )  # pylint: disable=undefined-variable
+        # pylint: disable=undefined-variable
+        icon_width = qApp.style().pixelMetric(QStyle.PixelMetric.PM_ToolBarIconSize)
         spacing = text_width + 3 * icon_width
         self.option.rect.setWidth(spacing)
         self.layout().insertSpacing(0, spacing)
@@ -345,7 +343,7 @@ class _MenuToolBar(QToolBar):
         for i in range(layout.count()):
             item = layout.itemAt(i)
             if item.widget() in self._buttons:
-                layout.itemAt(i).setAlignment(Qt.AlignBottom)
+                item.setAlignment(Qt.AlignmentFlag.AlignBottom)
 
     def add_frame(self, left, right, title):
         """Add frame around given actions, with given title.
@@ -383,7 +381,7 @@ class _MenuToolBar(QToolBar):
         """Make room for frames if needed."""
         size = super().sizeHint()
         if self._frames:
-            size = QSize(size.width(), size.height() + self.fontMetrics().height())
+            size.setHeight(size.height() + self.fontMetrics().height())
         return size
 
     def paintEvent(self, ev):
@@ -397,16 +395,16 @@ class _MenuToolBar(QToolBar):
             top_left = left.geometry().topLeft()
             bottom_right = right.geometry().bottomRight()
             rect = QRect(top_left, bottom_right).adjusted(-1, -fm.height() / 2, 1, 1)
-            painter.setPen(Qt.gray)
+            painter.setPen(Qt.GlobalColor.gray)
             painter.drawRoundedRect(rect, 1, 1)
             title_rect = fm.boundingRect(title).adjusted(-4, 0, 4, 0)
             title_rect.moveCenter(rect.center())
             title_rect.moveTop(rect.top() - fm.height() / 2)
-            painter.setBrush(Qt.white)
-            painter.setPen(Qt.NoPen)
+            painter.setBrush(Qt.GlobalColor.white)
+            painter.setPen(Qt.PenStyle.NoPen)
             painter.drawRect(title_rect)
-            painter.setPen(Qt.black)
-            painter.drawText(title_rect, Qt.AlignHCenter | Qt.AlignTop, title)
+            painter.setPen(Qt.GlobalColor.black)
+            painter.drawText(title_rect, Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignTop, title)
         painter.end()
 
     def _setup_action_button(self, action):
@@ -441,29 +439,29 @@ class _MenuToolBar(QToolBar):
         """Installed on each action's QToolButton.
         Ignores Up and Down key press events, so they are handled by the toolbar for custom navigation.
         """
-        if ev.type() == QEvent.KeyPress:
-            if ev.key() in (Qt.Key_Left, Qt.Key_Right):
+        if ev.type() == QEvent.Type.KeyPress:
+            if ev.key() in (Qt.Key.Key_Left, Qt.Key.Key_Right):
                 ev.accept()
                 return True
-            if ev.key() in (Qt.Key_Up, Qt.Key_Down):
+            if ev.key() in (Qt.Key.Key_Up, Qt.Key.Key_Down):
                 ev.ignore()
                 return True
         return super().eventFilter(obj, ev)
 
     def keyPressEvent(self, ev):
         """Navigates over the tool bar buttons."""
-        if ev.key() in (Qt.Key_Left, Qt.Key_Right):  # FIXME
+        if ev.key() in (Qt.Key.Key_Left, Qt.Key.Key_Right):  # FIXME
             ev.ignore()
             return
-        if ev.key() in (Qt.Key_Up, Qt.Key_Down):
+        if ev.key() in (Qt.Key.Key_Up, Qt.Key.Key_Down):
             widgets = [self.widgetForAction(a) for a in self.actions() if not a.isSeparator() and a.isEnabled()]
             if self._focus_widget not in widgets:
                 self._focus_widget = None
             if self._focus_widget is None:
-                next_index = 0 if ev.key() == Qt.Key_Down else len(widgets) - 1
+                next_index = 0 if ev.key() == Qt.Key.Key_Down else len(widgets) - 1
             else:
                 index = widgets.index(self._focus_widget)
-                next_index = index + 1 if ev.key() == Qt.Key_Down else index - 1
+                next_index = index + 1 if ev.key() == Qt.Key.Key_Down else index - 1
             if 0 <= next_index < len(widgets):
                 self._focus_widget = widgets[next_index]
                 self._focus_widget.setFocus()
@@ -508,7 +506,7 @@ class TitleWidgetAction(CustomWidgetAction):
         line.setFrameShadow(QFrame.Sunken)
         layout.addWidget(line)
 
-    def isSeparator(self):  # pylint: disable=no-self-use
+    def isSeparator(self):
         return True
 
 
@@ -543,7 +541,7 @@ class QWizardProcessPage(QWizardPage):
 
         def __set_name__(self, owner, name):
             self.public_name = name
-            self.private_name = '_' + name
+            self.private_name = "_" + name
 
         def __get__(self, obj, objtype=None):
             return getattr(obj, self.private_name)
@@ -551,7 +549,7 @@ class QWizardProcessPage(QWizardPage):
         def __set__(self, obj, value):
             setattr(obj, self.private_name, value)
             try:
-                value.execution_finished.connect(obj.widget_copy.show)
+                value.execution_finished.connect(lambda _: obj.widget_copy.show())
             except AttributeError:
                 pass
 
@@ -587,7 +585,7 @@ class QWizardProcessPage(QWizardPage):
         self.msg.connect(self._add_msg)
         self.msg_warning.connect(self._add_msg_warning)
         self.msg_error.connect(self._add_msg_error)
-        self.msg_success.connect(self._add_msg_succes)
+        self.msg_success.connect(self._add_msg_success)
         self.msg_proc.connect(self._add_msg)
         self.msg_proc_error.connect(self._add_msg_error)
         self._button_copy.clicked.connect(self._handle_copy_clicked)
@@ -597,16 +595,20 @@ class QWizardProcessPage(QWizardPage):
         self._label_copy.show()
         qApp.clipboard().setText(self._log.toPlainText())  # pylint: disable=undefined-variable
 
+    @Slot(str)
     def _add_msg(self, msg):
         self._log.append(format_log_message("msg", msg, show_datetime=False))
 
+    @Slot(str)
     def _add_msg_warning(self, msg):
         self._log.append(format_log_message("msg_warning", msg, show_datetime=False))
 
+    @Slot(str)
     def _add_msg_error(self, msg):
         self._log.append(format_log_message("msg_error", msg, show_datetime=False))
 
-    def _add_msg_succes(self, msg):
+    @Slot(str)
+    def _add_msg_success(self, msg):
         self._log.append(format_log_message("msg_success", msg, show_datetime=False))
 
     def isComplete(self):
@@ -630,7 +632,7 @@ class LabelWithCopyButton(QWidget):
         font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
         line_edit.setFont(font)
         button = QToolButton()
-        font = QFont('Font Awesome 5 Free Solid')
+        font = QFont(TOOLBOX_FONT.family)
         button.setFont(font)
         button.setText("\uf0c5")
         button.setToolTip("Copy text")
@@ -672,6 +674,13 @@ class HorizontalSpinBox(QToolBar):
         except TypeError:
             pass
 
+    def setMaximum(self, maximum):
+        try:
+            self._validator.setTop(maximum)
+        except TypeError:
+            pass
+
+    @Slot(str)
     def setValue(self, value, strict=False):
         try:
             value = int(value)
@@ -752,26 +761,10 @@ class SelectDatabaseItemsDialog(QDialog):
             )
         if self._warn_checked_non_data_items:
             if self._item_check_boxes_widget.any_structural_item_checked():
-                self._ui.warning_label.setText("Warning! Structural data items selected.")
+                self._ui.warning_label.setText("Warning! You are about to delete structural items.")
             else:
                 self._ui.warning_label.clear()
 
 
 class PurgeSettingsDialog(SelectDatabaseItemsDialog):
     _ok_button_can_be_disabled = False
-
-
-class ResizingViewMixin:
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._resizing_timer = QTimer()
-        self._resizing_timer.setSingleShot(True)
-        self._resizing_timer.setInterval(20)
-        self._resizing_timer.timeout.connect(self._do_resize)
-
-    def rowsInserted(self, parent, start, end):
-        super().rowsInserted(parent, start, end)
-        self._resizing_timer.start()
-
-    def _do_resize(self):
-        raise NotImplementedError()
