@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,17 +10,11 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Context menus for parameter value editor widgets.
-
-:author: A. Soininen (VTT)
-:date:   5.7.2019
-"""
+"""Context menus for parameter value editor widgets."""
 from operator import itemgetter
-
 from PySide6.QtCore import Slot
-from PySide6.QtWidgets import QInputDialog, QMenu
 from PySide6.QtGui import QAction
+from PySide6.QtWidgets import QInputDialog, QMenu
 from spinetoolbox.plotting import PlottingError, plot_value_editor_table_selection
 from spinetoolbox.widgets.plot_widget import PlotWidget, prepare_plot_in_window_menu
 from spinetoolbox.widgets.report_plotting_failure import report_plotting_failure
