@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,38 +10,34 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Classes for custom QDialogs for julia setup.
-
-:author: M. Marin (KTH)
-:date:   13.5.2018
-"""
-
-import os
+"""Classes for custom QDialogs for julia setup."""
 from enum import IntEnum, auto
+import os
+import sys
 
 try:
     import jill.install as jill_install
 except ModuleNotFoundError:
     jill_install = None
+from PySide6.QtCore import Qt, Signal, Slot
+from PySide6.QtGui import QCursor
 from PySide6.QtWidgets import (
+    QCheckBox,
+    QFileDialog,
+    QHBoxLayout,
+    QLabel,
+    QLineEdit,
+    QPushButton,
+    QVBoxLayout,
     QWidget,
     QWizard,
     QWizardPage,
-    QLabel,
-    QPushButton,
-    QVBoxLayout,
-    QHBoxLayout,
-    QLineEdit,
-    QFileDialog,
-    QCheckBox,
+    QApplication,
 )
-from PySide6.QtCore import Signal, Slot, Qt
-from PySide6.QtGui import QCursor
-from spine_engine.utils.helpers import resolve_python_interpreter
-from ..execution_managers import QProcessExecutionManager
+from spine_engine.utils.helpers import resolve_current_python_interpreter
 from ..config import APPLICATION_PATH
-from .custom_qwidgets import HyperTextLabel, QWizardProcessPage, LabelWithCopyButton
+from ..execution_managers import QProcessExecutionManager
+from .custom_qwidgets import HyperTextLabel, LabelWithCopyButton, QWizardProcessPage
 
 
 class _PageId(IntEnum):
@@ -76,13 +73,15 @@ class InstallJuliaWizard(QWizard):
         self.setStartId(_PageId.INTRO)
 
     def set_julia_exe(self):
-        basename = next(
-            (file for file in os.listdir(self.field('symlink_dir')) if file.lower().startswith("julia")), None
-        )
-        if basename is None:
+        """Returns the path to the jill julia launcher, which always launches the latest Julia release."""
+        if not sys.platform == "win32":
+            julia_launcher_path = os.path.join(self.field("symlink_dir"), "julia")
+        else:
+            julia_launcher_path = os.path.join(self.field("symlink_dir"), "julia.cmd")
+        if not os.path.exists(julia_launcher_path):
             self.julia_exe = None
             return
-        self.julia_exe = os.path.join(self.field('symlink_dir'), basename)
+        self.julia_exe = julia_launcher_path
 
     def accept(self):
         super().accept()
@@ -94,7 +93,7 @@ class JillNotFoundPage(QWizardPage):
     def __init__(self, parent):
         super().__init__(parent)
         self.setTitle("Unable to find jill")
-        conda_env = os.environ.get('CONDA_DEFAULT_ENV', 'base')
+        conda_env = os.environ.get("CONDA_DEFAULT_ENV", "base")
         toolbox_dir = os.path.dirname(APPLICATION_PATH)
         header = (
             "<p>Spine Toolbox needs the <a href='https://pypi.org/project/jill/'>jill</a> package "
@@ -161,7 +160,7 @@ class SelectDirsPage(QWizardPage):
         install_dir_button.clicked.connect(self._select_install_dir)
         symlink_dir_button.clicked.connect(self._select_symlink_dir)
         self.setCommitPage(True)
-        self.setButtonText(QWizard.CommitButton, "Install Julia")
+        self.setButtonText(QWizard.WizardButton.CommitButton, "Install Julia")
 
     def initializePage(self):
         self._install_dir_line_edit.setText(jill_install.default_install_dir())
@@ -213,19 +212,19 @@ class InstallJuliaPage(QWizardProcessPage):
         # 1. sys.executable when not frozen
         # 2. PATH python if frozen (This fails if no jill installed)
         # 3. If no PATH python, uses embedded python <install_dir>/tools/python.exe
-        python = resolve_python_interpreter("")
+        python = resolve_current_python_interpreter()
         self._exec_mngr = QProcessExecutionManager(self, python, args, semisilent=True)
         self.completeChanged.emit()
         self._exec_mngr.execution_finished.connect(self._handle_julia_install_finished)
         self.msg_success.emit("Julia installation started")
         cmd = python + " " + " ".join(args)
         self.msg.emit(f"$ <b>{cmd}<b/>")
-        qApp.setOverrideCursor(QCursor(Qt.BusyCursor))  # pylint: disable=undefined-variable
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.BusyCursor))  # pylint: disable=undefined-variable
         self._exec_mngr.start_execution()
 
     @Slot(int)
     def _handle_julia_install_finished(self, ret):
-        qApp.restoreOverrideCursor()  # pylint: disable=undefined-variable
+        QApplication.restoreOverrideCursor()  # pylint: disable=undefined-variable
         self._exec_mngr.execution_finished.disconnect(self._handle_julia_install_finished)
         if self.wizard().currentPage() != self:
             return
