@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,16 +10,11 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-An editor dialog for map indexes and values.
-
-:author: A. Soininen (VTT)
-:date:   2.11.2020
-"""
+"""An editor dialog for map indexes and values."""
 from PySide6.QtCore import Qt
 from .array_editor import ArrayEditor
-from .duration_editor import DurationEditor
 from .datetime_editor import DatetimeEditor
+from .duration_editor import DurationEditor
 from .parameter_value_editor_base import ParameterValueEditorBase, ValueType
 from .plain_parameter_value_editor import PlainParameterValueEditor
 from .time_pattern_editor import TimePatternEditor
