@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,13 +10,7 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains the MultiTabSpecEditor class.
-
-:author: M. Marin (KTH)
-:date:   12.12.2020
-"""
-
+"""Contains the MultiTabSpecEditor class."""
 from PySide6.QtCore import Qt
 from PySide6.QtGui import QIcon
 from PySide6.QtWidgets import QMenu
@@ -87,11 +82,16 @@ class MultiTabSpecEditor(MultiTabWindow):
             other_index = other_index_by_spec.get(spec)
             if other_index is None:
                 # Spec is not open on another multi tab editor, so open it here
-                slot = lambda spec=spec: self.add_new_tab(spec)
+                def slot(spec_=spec):
+                    return self.add_new_tab(spec_)
+
             else:
                 # Spec is open on another multi tab editor, so bring it here
                 other, index = other_index
-                slot = lambda other=other, index=index: other.move_tab(index, self)
+
+                def slot(other_=other, index_=index):
+                    other_.move_tab(index_, self)
+
             menu.addAction(spec.name, slot)
         menu.popup(global_pos)
         menu.aboutToHide.connect(menu.deleteLater)
