@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,25 +10,18 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-A Qt widget showing a toolbar and a matplotlib plotting canvas.
-
-:author: A. Soininen (VTT)
-:date:   27.6.2019
-"""
-
-import itertools
-import io
+"""A Qt widget showing a toolbar and a matplotlib plotting canvas."""
 import csv
-
-import numpy
+import io
+import itertools
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolBar
+import numpy
 from PySide6.QtCore import QMetaObject, Qt
-from PySide6.QtWidgets import QVBoxLayout, QWidget, QMenu, QApplication
-from .plot_canvas import PlotCanvas, LegendPosition
-from .custom_qtableview import CopyPasteTableView
-from ..mvcmodels.minimal_table_model import MinimalTableModel
+from PySide6.QtWidgets import QApplication, QMenu, QVBoxLayout, QWidget
 from ..helpers import busy_effect
+from ..mvcmodels.minimal_table_model import MinimalTableModel
+from .custom_qtableview import CopyPasteTableView
+from .plot_canvas import LegendPosition, PlotCanvas
 
 
 class PlotWidget(QWidget):
@@ -39,7 +33,7 @@ class PlotWidget(QWidget):
         original_xy_data (list of XYData): unmodified data on which the plots are based
     """
 
-    plot_windows = dict()
+    plot_windows = {}
     """A global list of plot windows."""
 
     def __init__(self, parent=None, legend_axes_position=LegendPosition.BOTTOM):
@@ -54,7 +48,7 @@ class PlotWidget(QWidget):
         self._toolbar = NavigationToolBar(self.canvas, self)
         self._layout.addWidget(self._toolbar)
         self._layout.addWidget(self.canvas)
-        self.original_xy_data = list()
+        self.original_xy_data = []
         QMetaObject.connectSlotsByName(self)
 
     def closeEvent(self, event):
@@ -81,7 +75,7 @@ class PlotWidget(QWidget):
         indexes = []
         data_dicts = []
         for xy_data in self.original_xy_data:
-            label = " | ".join(xy_data.data_index)
+            label = " | ".join(["None" if name is None else str(name) for name in xy_data.data_index])
             header.append(label)
             indexes.append(xy_data.x)
             data_dict = dict(zip(xy_data.x, xy_data.y))
@@ -111,6 +105,7 @@ class PlotWidget(QWidget):
         widget.setWindowFlag(Qt.WindowType.Window, True)
         title = "Plot data"
         widget.setWindowTitle(title)
+        widget.set_size_according_to_parent()
         widget.show()
 
     def add_legend(self, handles):
@@ -131,7 +126,7 @@ class PlotWidget(QWidget):
         """
         self.setParent(parent_window)
         self.setWindowFlag(Qt.WindowType.Window, True)
-        self.setAttribute(Qt.WA_DeleteOnClose, True)
+        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, True)
         title = "Plot"
         if document_name:
             title += f"    -- {document_name} --"
@@ -160,17 +155,36 @@ class _PlotDataView(CopyPasteTableView):
 class _PlotDataWidget(QWidget):
     def __init__(self, rows, parent=None):
         super().__init__(parent=parent)
+        self._parent = parent
+        self._rows = rows
         self.setWindowTitle("Plot data")
         layout = QVBoxLayout(self)
         self._view = _PlotDataView(self)
         self._model = MinimalTableModel(self)
         self._view.setModel(self._model)
         self._model.reset_model(rows)
+        self._view.setHorizontalScrollMode(_PlotDataView.ScrollMode.ScrollPerPixel)
+        self._view.setVerticalScrollMode(_PlotDataView.ScrollMode.ScrollPerPixel)
         self._view.resizeColumnsToContents()
         self._view.horizontalHeader().hide()
         self._view.verticalHeader().hide()
         layout.addWidget(self._view)
-        self.setAttribute(Qt.WA_DeleteOnClose)
+        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
+        self.table_view_width = (
+            self._view.frameWidth() + self._view.verticalHeader().width() + self._view.horizontalHeader().length()
+        )
+
+    def set_size_according_to_parent(self):
+        """Sets the size of the widget according to the parent widget's dimensions and the data in the table"""
+        self.setMinimumWidth(274)
+        self.setMinimumHeight(210)
+        margins = self.layout().contentsMargins()
+        width = min(self.table_view_width + margins.left(), self._parent.size().width() * 0.8)
+        height = min(
+            self._view.verticalHeader().defaultSectionSize() * (len(self._rows) + 1) + margins.top(),
+            self._parent.size().height() * 0.8,
+        )
+        self.resize(width, height)
 
 
 def prepare_plot_in_window_menu(menu):
