@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -8,13 +9,12 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-"""
-A widget and utilities to select database items.
-"""
+
+"""A widget and utilities to select database items."""
 from PySide6.QtCore import Signal, Slot
 from PySide6.QtWidgets import QCheckBox, QWidget
-
-from spinedb_api.db_mapping_base import DatabaseMappingBase
+from spinedb_api.db_mapping import DatabaseMapping
+from spinedb_api.mapped_items import ITEM_CLASS_BY_TYPE
 
 
 def add_check_boxes(check_boxes, checked_states, select_all_button, deselect_all_button, state_changed_slot, layout):
@@ -56,14 +56,25 @@ class SelectDatabaseItems(QWidget):
     checked_state_changed = Signal(int)
     COLUMN_COUNT = 3
     _DATA_ITEMS = (
-        "object",
-        "relationship",
+        "entity",
         "entity_group",
         "parameter_value",
         "entity_metadata",
+        "entity_location",
         "parameter_value_metadata",
+        "metadata",
+    )
+    _SCENARIO_ITEMS = ("alternative", "scenario", "scenario_alternative", "entity_alternative")
+    _STRUCTURAL_ITEMS = (
+        "entity_class",
+        "display_mode",
+        "entity_class_display_mode",
+        "superclass_subclass",
+        "parameter_value_list",
+        "list_value",
+        "parameter_definition",
+        "parameter_type",
     )
-    _SCENARIO_ITEMS = ("alternative", "scenario", "scenario_alternative")
 
     def __init__(self, checked_states=None, parent=None):
         """
@@ -78,10 +89,14 @@ class SelectDatabaseItems(QWidget):
         self._ui.setupUi(self)
         self._ui.select_data_items_button.clicked.connect(self._select_data_items)
         self._ui.select_scenario_items_button.clicked.connect(self._select_scenario_items)
-        checked_states = (
-            checked_states if checked_states is not None else {item: False for item in DatabaseMappingBase.ITEM_TYPES}
+        self._ui.select_structural_items_button.clicked.connect(self._select_structural_items)
+        checkable_item_types = tuple(
+            type_ for type_ in DatabaseMapping.item_types() if not ITEM_CLASS_BY_TYPE[type_].is_protected
         )
-        self._item_check_boxes = {item_type: QCheckBox(item_type, self) for item_type in DatabaseMappingBase.ITEM_TYPES}
+        checked_states = (
+            checked_states if checked_states is not None else {item: False for item in checkable_item_types}
+        )
+        self._item_check_boxes = {item_type: QCheckBox(item_type, self) for item_type in checkable_item_types}
         add_check_boxes(
             self._item_check_boxes,
             checked_states,
@@ -128,3 +143,9 @@ class SelectDatabaseItems(QWidget):
         """Checks all scenario items."""
         for item_name in self._SCENARIO_ITEMS:
             self._item_check_boxes[item_name].setChecked(True)
+
+    @Slot(bool)
+    def _select_structural_items(self, _=False):
+        """Checks all structural items."""
+        for item_name in self._STRUCTURAL_ITEMS:
+            self._item_check_boxes[item_name].setChecked(True)
