@@ -1,5 +1,6 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,21 +10,16 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Unit tests for MapModel class.
-
-:authors: A. Soininen (VTT)
-:date:    11.2.2020
-"""
-
+"""Unit tests for MapModel class."""
 import unittest
-from PySide2.QtCore import QObject, Qt
-from PySide2.QtGui import QColor
+from PySide6.QtCore import QObject, Qt
+from PySide6.QtGui import QColor
 from spinedb_api import Array, DateTime, Duration, Map, ParameterValueFormatError
-from spinetoolbox.mvcmodels.map_model import MapModel, empty
+from spinetoolbox.mvcmodels.map_model import MapModel
+from tests.mock_helpers import TestCaseWithQApplication
 
 
-class TestMapModel(unittest.TestCase):
+class TestMapModel(TestCaseWithQApplication):
     def setUp(self):
         self._parent = QObject()
 
@@ -37,9 +33,9 @@ class TestMapModel(unittest.TestCase):
         model.append_column()
         self.assertEqual(model.columnCount(), 5)
         expected_table = [
-            ["A", -1.1, "", "", ""],
-            ["B", "a", 1.1, "", ""],
-            ["B", "b", 2.2, "", ""],
+            ["A", "-1.1", "", "", ""],
+            ["B", "a", "1.1", "", ""],
+            ["B", "b", "2.2", "", ""],
             ["", "", "", "", ""],
         ]
         for y, row in enumerate(expected_table):
@@ -58,6 +54,11 @@ class TestMapModel(unittest.TestCase):
         model = MapModel(map_value, self._parent)
         self.assertEqual(model.columnCount(), 4)
 
+    def test_columnCount_empty_map(self):
+        map_value = Map([], [], index_type=int)
+        model = MapModel(map_value, self._parent)
+        self.assertEqual(model.columnCount(), 3)
+
     def test_convert_leaf_maps(self):
         nested_map = Map([DateTime("2020-07-03 12:00:00"), DateTime("2020-07-03 12:00:00")], [22.2, 23.3])
         map_ = Map([1.0], [nested_map])
@@ -65,7 +66,7 @@ class TestMapModel(unittest.TestCase):
         model.convert_leaf_maps()
         self.assertEqual(model.columnCount(), 3)
         self.assertEqual(model.rowCount(), 2)
-        self.assertEqual(model.index(0, 0).data(), 1.0)
+        self.assertEqual(model.index(0, 0).data(), str(1.0))
         self.assertEqual(model.index(0, 1).data(), "Time series")
         self.assertEqual(model.index(0, 2).data(), "")
         self.assertEqual(model.index(1, 0).data(), "")
@@ -78,8 +79,8 @@ class TestMapModel(unittest.TestCase):
         self.assertEqual(model.index(0, 0).data(), "a")
         self.assertEqual(model.index(1, 0).data(), "b")
         self.assertEqual(model.index(2, 0).data(), "")
-        self.assertEqual(model.index(0, 1).data(), 1.1)
-        self.assertEqual(model.index(1, 1).data(), 2.2)
+        self.assertEqual(model.index(0, 1).data(), str(1.1))
+        self.assertEqual(model.index(1, 1).data(), str(2.2))
         self.assertEqual(model.index(2, 1).data(), "")
         self.assertEqual(model.index(0, 2).data(), "")
         self.assertEqual(model.index(1, 2).data(), "")
@@ -88,31 +89,31 @@ class TestMapModel(unittest.TestCase):
     def test_data_EditRole(self):
         map_value = Map(["a", "b"], [1.1, 2.2])
         model = MapModel(map_value, self._parent)
-        self.assertEqual(model.index(0, 0).data(Qt.EditRole), "a")
-        self.assertEqual(model.index(1, 0).data(Qt.EditRole), "b")
-        self.assertEqual(model.index(0, 1).data(Qt.EditRole), 1.1)
-        self.assertEqual(model.index(1, 1).data(Qt.EditRole), 2.2)
+        self.assertEqual(model.index(0, 0).data(Qt.ItemDataRole.EditRole), "a")
+        self.assertEqual(model.index(1, 0).data(Qt.ItemDataRole.EditRole), "b")
+        self.assertEqual(model.index(0, 1).data(Qt.ItemDataRole.EditRole), 1.1)
+        self.assertEqual(model.index(1, 1).data(Qt.ItemDataRole.EditRole), 2.2)
 
     def test_data_BackgroundRole(self):
         map_value = Map(["a"], [1.1])
         model = MapModel(map_value, self._parent)
         expected = QColor(245, 245, 245)
-        self.assertEqual(model.index(0, 0).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(1, 0).data(Qt.BackgroundRole), expected)
-        self.assertEqual(model.index(0, 1).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(1, 1).data(Qt.BackgroundRole), expected)
-        self.assertEqual(model.index(0, 2).data(Qt.BackgroundRole), expected)
-        self.assertEqual(model.index(1, 2).data(Qt.BackgroundRole), expected)
+        self.assertEqual(model.index(0, 0).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(1, 0).data(Qt.ItemDataRole.BackgroundRole), expected)
+        self.assertEqual(model.index(0, 1).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(1, 1).data(Qt.ItemDataRole.BackgroundRole), expected)
+        self.assertEqual(model.index(0, 2).data(Qt.ItemDataRole.BackgroundRole), expected)
+        self.assertEqual(model.index(1, 2).data(Qt.ItemDataRole.BackgroundRole), expected)
 
     def test_data_FontRole(self):
         map_value = Map(["a"], [1.1])
         model = MapModel(map_value, self._parent)
-        self.assertEqual(model.index(0, 0).data(Qt.FontRole), None)
-        self.assertEqual(model.index(1, 0).data(Qt.FontRole), None)
-        self.assertTrue(model.index(0, 1).data(Qt.FontRole).bold())
-        self.assertEqual(model.index(1, 1).data(Qt.FontRole), None)
-        self.assertEqual(model.index(0, 2).data(Qt.FontRole), None)
-        self.assertEqual(model.index(1, 2).data(Qt.FontRole), None)
+        self.assertEqual(model.index(0, 0).data(Qt.ItemDataRole.FontRole), None)
+        self.assertEqual(model.index(1, 0).data(Qt.ItemDataRole.FontRole), None)
+        self.assertTrue(model.index(0, 1).data(Qt.ItemDataRole.FontRole).bold())
+        self.assertEqual(model.index(1, 1).data(Qt.ItemDataRole.FontRole), None)
+        self.assertEqual(model.index(0, 2).data(Qt.ItemDataRole.FontRole), None)
+        self.assertEqual(model.index(1, 2).data(Qt.ItemDataRole.FontRole), None)
 
     def test_data_nested_maps_DisplayRole(self):
         nested_map = Map(["a", "b"], [1.1, 2.2])
@@ -122,13 +123,13 @@ class TestMapModel(unittest.TestCase):
         self.assertEqual(model.index(1, 0).data(), "B")
         self.assertEqual(model.index(2, 0).data(), "B")
         self.assertEqual(model.index(3, 0).data(), "")
-        self.assertEqual(model.index(0, 1).data(), -1.1)
+        self.assertEqual(model.index(0, 1).data(), str(-1.1))
         self.assertEqual(model.index(1, 1).data(), "a")
         self.assertEqual(model.index(2, 1).data(), "b")
         self.assertEqual(model.index(3, 1).data(), "")
         self.assertEqual(model.index(0, 2).data(), "")
-        self.assertEqual(model.index(1, 2).data(), 1.1)
-        self.assertEqual(model.index(2, 2).data(), 2.2)
+        self.assertEqual(model.index(1, 2).data(), str(1.1))
+        self.assertEqual(model.index(2, 2).data(), str(2.2))
         self.assertEqual(model.index(3, 2).data(), "")
         self.assertEqual(model.index(0, 3).data(), "")
         self.assertEqual(model.index(1, 3).data(), "")
@@ -139,57 +140,57 @@ class TestMapModel(unittest.TestCase):
         nested_map = Map(["a", "b"], [None, 2.2])
         map_value = Map(["A", "B"], [-1.1, nested_map])
         model = MapModel(map_value, self._parent)
-        self.assertEqual(model.index(0, 0).data(Qt.EditRole), "A")
-        self.assertEqual(model.index(1, 0).data(Qt.EditRole), "B")
-        self.assertEqual(model.index(2, 0).data(Qt.EditRole), "B")
-        self.assertEqual(model.index(0, 1).data(Qt.EditRole), -1.1)
-        self.assertEqual(model.index(1, 1).data(Qt.EditRole), "a")
-        self.assertEqual(model.index(2, 1).data(Qt.EditRole), "b")
-        self.assertEqual(model.index(0, 2).data(Qt.EditRole), "")
-        self.assertEqual(model.index(1, 2).data(Qt.EditRole), None)
-        self.assertEqual(model.index(2, 2).data(Qt.EditRole), 2.2)
+        self.assertEqual(model.index(0, 0).data(Qt.ItemDataRole.EditRole), "A")
+        self.assertEqual(model.index(1, 0).data(Qt.ItemDataRole.EditRole), "B")
+        self.assertEqual(model.index(2, 0).data(Qt.ItemDataRole.EditRole), "B")
+        self.assertEqual(model.index(0, 1).data(Qt.ItemDataRole.EditRole), -1.1)
+        self.assertEqual(model.index(1, 1).data(Qt.ItemDataRole.EditRole), "a")
+        self.assertEqual(model.index(2, 1).data(Qt.ItemDataRole.EditRole), "b")
+        self.assertEqual(model.index(0, 2).data(Qt.ItemDataRole.EditRole), "")
+        self.assertEqual(model.index(1, 2).data(Qt.ItemDataRole.EditRole), None)
+        self.assertEqual(model.index(2, 2).data(Qt.ItemDataRole.EditRole), 2.2)
 
     def test_data_nested_maps_FontRole(self):
         nested_map = Map(["a", "b"], [1.1, 2.2])
         map_value = Map(["A", "B"], [-1.1, nested_map])
         model = MapModel(map_value, self._parent)
-        self.assertEqual(model.index(0, 0).data(Qt.FontRole), None)
-        self.assertEqual(model.index(1, 0).data(Qt.FontRole), None)
-        self.assertEqual(model.index(2, 0).data(Qt.FontRole), None)
-        self.assertTrue(model.index(0, 1).data(Qt.FontRole).bold())
-        self.assertEqual(model.index(1, 1).data(Qt.FontRole), None)
-        self.assertEqual(model.index(2, 1).data(Qt.FontRole), None)
-        self.assertEqual(model.index(0, 2).data(Qt.FontRole), None)
-        self.assertTrue(model.index(1, 2).data(Qt.FontRole).bold())
-        self.assertTrue(model.index(2, 2).data(Qt.FontRole).bold())
+        self.assertEqual(model.index(0, 0).data(Qt.ItemDataRole.FontRole), None)
+        self.assertEqual(model.index(1, 0).data(Qt.ItemDataRole.FontRole), None)
+        self.assertEqual(model.index(2, 0).data(Qt.ItemDataRole.FontRole), None)
+        self.assertTrue(model.index(0, 1).data(Qt.ItemDataRole.FontRole).bold())
+        self.assertEqual(model.index(1, 1).data(Qt.ItemDataRole.FontRole), None)
+        self.assertEqual(model.index(2, 1).data(Qt.ItemDataRole.FontRole), None)
+        self.assertEqual(model.index(0, 2).data(Qt.ItemDataRole.FontRole), None)
+        self.assertTrue(model.index(1, 2).data(Qt.ItemDataRole.FontRole).bold())
+        self.assertTrue(model.index(2, 2).data(Qt.ItemDataRole.FontRole).bold())
 
     def test_data_nested_maps_BackgroundRole(self):
         nested_map = Map(["a", "b"], [1.1, 2.2])
         map_value = Map(["A", "B"], [-1.1, nested_map])
         model = MapModel(map_value, self._parent)
-        self.assertEqual(model.index(0, 0).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(1, 0).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(2, 0).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(3, 0).data(Qt.BackgroundRole), QColor(245, 245, 245))
-        self.assertEqual(model.index(0, 1).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(1, 1).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(2, 1).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(3, 1).data(Qt.BackgroundRole), QColor(245, 245, 245))
-        self.assertEqual(model.index(0, 2).data(Qt.BackgroundRole), QColor(255, 240, 240))
-        self.assertEqual(model.index(1, 2).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(2, 2).data(Qt.BackgroundRole), None)
-        self.assertEqual(model.index(3, 2).data(Qt.BackgroundRole), QColor(245, 245, 245))
-        self.assertEqual(model.index(0, 3).data(Qt.BackgroundRole), QColor(245, 245, 245))
-        self.assertEqual(model.index(1, 3).data(Qt.BackgroundRole), QColor(245, 245, 245))
-        self.assertEqual(model.index(2, 3).data(Qt.BackgroundRole), QColor(245, 245, 245))
-        self.assertEqual(model.index(3, 3).data(Qt.BackgroundRole), QColor(245, 245, 245))
+        self.assertEqual(model.index(0, 0).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(1, 0).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(2, 0).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(3, 0).data(Qt.ItemDataRole.BackgroundRole), QColor(245, 245, 245))
+        self.assertEqual(model.index(0, 1).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(1, 1).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(2, 1).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(3, 1).data(Qt.ItemDataRole.BackgroundRole), QColor(245, 245, 245))
+        self.assertEqual(model.index(0, 2).data(Qt.ItemDataRole.BackgroundRole), QColor(255, 240, 240))
+        self.assertEqual(model.index(1, 2).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(2, 2).data(Qt.ItemDataRole.BackgroundRole), None)
+        self.assertEqual(model.index(3, 2).data(Qt.ItemDataRole.BackgroundRole), QColor(245, 245, 245))
+        self.assertEqual(model.index(0, 3).data(Qt.ItemDataRole.BackgroundRole), QColor(245, 245, 245))
+        self.assertEqual(model.index(1, 3).data(Qt.ItemDataRole.BackgroundRole), QColor(245, 245, 245))
+        self.assertEqual(model.index(2, 3).data(Qt.ItemDataRole.BackgroundRole), QColor(245, 245, 245))
+        self.assertEqual(model.index(3, 3).data(Qt.ItemDataRole.BackgroundRole), QColor(245, 245, 245))
 
     def test_data_DisplayRole_repeated_indexes_do_not_show(self):
         leaf_map = Map(["a", "b"], [1.1, 2.2])
         nested_map = Map(["A"], [leaf_map])
         root_map = Map(["root"], [nested_map])
         model = MapModel(root_map, self._parent)
-        expected_data = [["root", "A", "a", 1.1], ["root", "A", "b", 2.2]]
+        expected_data = [["root", "A", "a", str(1.1)], ["root", "A", "b", str(2.2)]]
         for row in range(2):
             for column in range(4):
                 index = model.index(row, column)
@@ -205,21 +206,21 @@ class TestMapModel(unittest.TestCase):
         nested_map = Map(["a", "b"], [1.1, 2.2], index_name="nested index")
         map_value = Map(["A"], [nested_map], index_name="main index")
         model = MapModel(map_value, self._parent)
-        self.assertEqual(model.headerData(0, Qt.Horizontal), "main index")
-        self.assertEqual(model.headerData(1, Qt.Horizontal), "nested index")
-        self.assertEqual(model.headerData(2, Qt.Horizontal), "Value")
-        self.assertEqual(model.headerData(0, Qt.Vertical), 1)
+        self.assertEqual(model.headerData(0, Qt.Orientation.Horizontal), "main index")
+        self.assertEqual(model.headerData(1, Qt.Orientation.Horizontal), "nested index")
+        self.assertEqual(model.headerData(2, Qt.Orientation.Horizontal), "Value")
+        self.assertEqual(model.headerData(0, Qt.Orientation.Vertical), 1)
 
     def test_setHeaderData(self):
         nested_map = Map(["a", "b"], [1.1, 2.2], index_name="nested index")
         map_value = Map(["A"], [nested_map], index_name="main index")
         model = MapModel(map_value, self._parent)
-        self.assertEqual(model.headerData(0, Qt.Horizontal), "main index")
-        self.assertEqual(model.headerData(1, Qt.Horizontal), "nested index")
-        self.assertTrue(model.setHeaderData(0, Qt.Horizontal, "new main"))
-        self.assertTrue(model.setHeaderData(1, Qt.Horizontal, "new nested"))
-        self.assertEqual(model.headerData(0, Qt.Horizontal), "new main")
-        self.assertEqual(model.headerData(1, Qt.Horizontal), "new nested")
+        self.assertEqual(model.headerData(0, Qt.Orientation.Horizontal), "main index")
+        self.assertEqual(model.headerData(1, Qt.Orientation.Horizontal), "nested index")
+        self.assertTrue(model.setHeaderData(0, Qt.Orientation.Horizontal, "new main"))
+        self.assertTrue(model.setHeaderData(1, Qt.Orientation.Horizontal, "new nested"))
+        self.assertEqual(model.headerData(0, Qt.Orientation.Horizontal), "new main")
+        self.assertEqual(model.headerData(1, Qt.Orientation.Horizontal), "new nested")
 
     def test_insertRows_to_empty_model(self):
         map_value = Map([], [], str)
@@ -243,7 +244,7 @@ class TestMapModel(unittest.TestCase):
         self.assertEqual(model.index(0, 1).data(), "")
         self.assertEqual(model.index(0, 2).data(), "")
         self.assertEqual(model.index(1, 0).data(), "a")
-        self.assertEqual(model.index(1, 1).data(), 1.1)
+        self.assertEqual(model.index(1, 1).data(), str(1.1))
         self.assertEqual(model.index(1, 2).data(), "")
         self.assertEqual(model.index(2, 0).data(), "")
         self.assertEqual(model.index(2, 1).data(), "")
@@ -255,10 +256,10 @@ class TestMapModel(unittest.TestCase):
         self.assertTrue(model.insertRows(1, 1))
         self.assertEqual(model.rowCount(), 3)
         self.assertEqual(model.index(0, 0).data(), "a")
-        self.assertEqual(model.index(0, 1).data(), 1.1)
+        self.assertEqual(model.index(0, 1).data(), str(1.1))
         self.assertEqual(model.index(0, 2).data(), "")
         self.assertEqual(model.index(1, 0).data(), "a")
-        self.assertEqual(model.index(1, 1).data(), 1.1)
+        self.assertEqual(model.index(1, 1).data(), str(1.1))
         self.assertEqual(model.index(1, 2).data(), "")
 
     def test_insertRows_to_middle_of_nested_map(self):
@@ -267,7 +268,12 @@ class TestMapModel(unittest.TestCase):
         model = MapModel(map_value, self._parent)
         self.assertTrue(model.insertRows(1, 1))
         self.assertEqual(model.rowCount(), 4)
-        expected_table = [["A", "a", 1.1, ""], ["A", "a", 1.1, ""], ["A", "b", 2.2, ""], ["", "", "", ""]]
+        expected_table = [
+            ["A", "a", str(1.1), ""],
+            ["A", "a", str(1.1), ""],
+            ["A", "b", str(2.2), ""],
+            ["", "", "", ""],
+        ]
         for y, row in enumerate(expected_table):
             for x, expected in enumerate(row):
                 self.assertEqual(model.index(y, x).data(), expected)
@@ -283,12 +289,17 @@ class TestMapModel(unittest.TestCase):
         model = MapModel(map_value, self._parent)
         self.assertEqual(model.rowCount(), 4)
 
+    def test_row_count_empty_map(self):
+        map_value = Map([], [], index_type=str)
+        model = MapModel(map_value, self._parent)
+        self.assertEqual(model.rowCount(), 1)
+
     def test_removeRows_single_row(self):
         map_value = Map(["a"], [1.1])
         model = MapModel(map_value, self._parent)
         self.assertTrue(model.removeRows(0, 1))
         self.assertEqual(model.rowCount(), 1)
-        self.assertEqual(model.columnCount(), 1)
+        self.assertEqual(model.columnCount(), 3)
         self.assertEqual(model.index(0, 0).data(), "")
 
     def test_removeRows_first_row(self):
@@ -297,7 +308,7 @@ class TestMapModel(unittest.TestCase):
         self.assertTrue(model.removeRows(0, 1))
         self.assertEqual(model.rowCount(), 2)
         self.assertEqual(model.index(0, 0).data(), "b")
-        self.assertEqual(model.index(0, 1).data(), 2.2)
+        self.assertEqual(model.index(0, 1).data(), str(2.2))
         self.assertEqual(model.index(0, 2).data(), "")
         self.assertEqual(model.index(1, 0).data(), "")
         self.assertEqual(model.index(1, 1).data(), "")
@@ -309,7 +320,7 @@ class TestMapModel(unittest.TestCase):
         self.assertTrue(model.removeRows(0, 1))
         self.assertEqual(model.rowCount(), 2)
         self.assertEqual(model.index(0, 0).data(), "b")
-        self.assertEqual(model.index(0, 1).data(), 2.2)
+        self.assertEqual(model.index(0, 1).data(), str(2.2))
         self.assertEqual(model.index(0, 2).data(), "")
         self.assertEqual(model.index(1, 0).data(), "")
         self.assertEqual(model.index(1, 1).data(), "")
@@ -321,7 +332,7 @@ class TestMapModel(unittest.TestCase):
         model = MapModel(map_value, self._parent)
         self.assertTrue(model.removeRows(1, 1))
         self.assertEqual(model.rowCount(), 3)
-        expected_table = [["A", "a", 1.1, ""], ["A", "c", 3.3, ""], ["", "", "", ""]]
+        expected_table = [["A", "a", str(1.1), ""], ["A", "c", str(3.3), ""], ["", "", "", ""]]
         for y, row in enumerate(expected_table):
             for x, expected in enumerate(row):
                 self.assertEqual(model.index(y, x).data(), expected)
@@ -335,7 +346,7 @@ class TestMapModel(unittest.TestCase):
     def test_setData_expands_empty_table(self):
         model = MapModel(Map([], [], Duration), self._parent)
         self.assertEqual(model.rowCount(), 1)
-        self.assertEqual(model.columnCount(), 1)
+        self.assertEqual(model.columnCount(), 3)
         self.assertEqual(model.index(0, 0).data(), "")
         self.assertTrue(model.setData(model.index(0, 0), Duration("1 month")))
         self.assertEqual(model.rowCount(), 2)
@@ -355,15 +366,27 @@ class TestMapModel(unittest.TestCase):
         self.assertEqual(model.rowCount(), 3)
         self.assertEqual(model.columnCount(), 3)
         self.assertEqual(model.index(0, 0).data(), "1M")
-        self.assertEqual(model.index(0, 1).data(), 1.1)
+        self.assertEqual(model.index(0, 1).data(), str(1.1))
         self.assertEqual(model.index(0, 2).data(), "")
         self.assertEqual(model.index(1, 0).data(), "1M")
-        self.assertEqual(model.index(1, 1).data(), 2.2)
+        self.assertEqual(model.index(1, 1).data(), str(2.2))
         self.assertEqual(model.index(1, 2).data(), "")
         self.assertEqual(model.index(2, 0).data(), "")
         self.assertEqual(model.index(2, 1).data(), "")
         self.assertEqual(model.index(2, 2).data(), "")
 
+    def test_setData_does_not_expand_rows_if_value_is_empty(self):
+        model = MapModel(Map([Duration("1 month")], [1.1]), self._parent)
+        self.assertEqual(model.rowCount(), 2)
+        self.assertEqual(model.columnCount(), 3)
+        self.assertFalse(model.setData(model.index(1, 0), ""))
+
+    def test_setData_does_not_expand_columns_if_value_is_empty(self):
+        model = MapModel(Map([Duration("1 month")], [1.1]), self._parent)
+        self.assertEqual(model.rowCount(), 2)
+        self.assertEqual(model.columnCount(), 3)
+        self.assertFalse(model.setData(model.index(0, 2), ""))
+
     def test_setData_does_not_clear_value_if_it_is_zero(self):
         model = MapModel(Map([], [], str), self._parent)
         model.setData(model.index(0, 0), "idx")
@@ -371,7 +394,24 @@ class TestMapModel(unittest.TestCase):
         self.assertEqual(model.rowCount(), 1 + 1)
         self.assertEqual(model.columnCount(), 2 + 1)
         self.assertEqual(model.index(0, 0).data(), "idx")
-        self.assertEqual(model.index(0, 1).data(), 0.0)
+        self.assertEqual(model.index(0, 1).data(), str(0.0))
+
+    def test_init_converts_numpy_strings_to_real_strings(self):
+        map_value = Map(["a"], [1.1])
+        model = MapModel(map_value, self._parent)
+        model.setData(model.index(1, 0), "b", Qt.ItemDataRole.EditRole)
+        model.setData(model.index(1, 1), 2.2, Qt.ItemDataRole.EditRole)
+        value = model.value()
+        self.assertEqual(value, Map(["a", "b"], [1.1, 2.2]))
+
+    def test_reset_converts_numpy_strings_to_real_strings(self):
+        model = MapModel(Map([], [], str), self._parent)
+        map_value = Map(["a"], [1.1])
+        model.reset(map_value)
+        model.setData(model.index(1, 0), "b", Qt.ItemDataRole.EditRole)
+        model.setData(model.index(1, 1), 2.2, Qt.ItemDataRole.EditRole)
+        value = model.value()
+        self.assertEqual(value, Map(["a", "b"], [1.1, 2.2]))
 
     def test_trim_columns(self):
         map_value = Map(["a"], [1.1])
@@ -476,12 +516,57 @@ class TestMapModel(unittest.TestCase):
         with self.assertRaises(ParameterValueFormatError):
             model.value()
 
+    def test_different_index_types_in_column_raises(self):
+        map_value = Map([99.0, 101.0], [-1.1, -2.2])
+        model = MapModel(map_value, self._parent)
+        model.setData(model.index(1, 0), "a")
+        with self.assertRaises(ParameterValueFormatError):
+            model.value()
+
     def test_value_shortening_rows(self):
         original = Map(["a", "b", "c"], [0.0, Map(["bb"], [Map(["bbb"], [Array([-1.0])])]), Array([-2.0])])
         model = MapModel(original, self._parent)
         map_ = model.value()
         self.assertEqual(map_, original)
 
+    def test_insertColumns_to_empty_map(self):
+        map_value = Map([], [], index_type=str)
+        model = MapModel(map_value, self._parent)
+        self.assertEqual(model.rowCount(), 1)
+        self.assertEqual(model.columnCount(), 3)
+        self.assertTrue(model.insertColumns(0, 1))
+        self.assertEqual(model.rowCount(), 1)
+        self.assertEqual(model.columnCount(), 4)
+        self.assertEqual(model.headerData(0, Qt.Orientation.Horizontal), "x")
+        self.assertEqual(model.headerData(1, Qt.Orientation.Horizontal), "x")
+        self.assertEqual(model.headerData(2, Qt.Orientation.Horizontal), "Value")
+        self.assertEqual(model.headerData(3, Qt.Orientation.Horizontal), None)
+        self.assertEqual(model.index(0, 0).data(), "")
+        self.assertEqual(model.index(0, 1).data(), "")
+        self.assertEqual(model.index(0, 2).data(), "")
+        self.assertEqual(model.index(0, 3).data(), "")
 
-if __name__ == '__main__':
+    def test_insertColumns_to_map(self):
+        map_value = Map(["A"], [1.0], index_name="Panda")
+        model = MapModel(map_value, self._parent)
+        self.assertEqual(model.rowCount(), 2)
+        self.assertEqual(model.columnCount(), 3)
+        self.assertTrue(model.insertColumns(0, 1))
+        self.assertEqual(model.rowCount(), 2)
+        self.assertEqual(model.columnCount(), 4)
+        self.assertEqual(model.headerData(0, Qt.Orientation.Horizontal), "x")
+        self.assertEqual(model.headerData(1, Qt.Orientation.Horizontal), "Panda")
+        self.assertEqual(model.headerData(2, Qt.Orientation.Horizontal), "Value")
+        self.assertEqual(model.headerData(3, Qt.Orientation.Horizontal), None)
+        self.assertEqual(model.index(0, 0).data(), "")
+        self.assertEqual(model.index(0, 1).data(), "A")
+        self.assertEqual(model.index(0, 2).data(), str(1.0))
+        self.assertEqual(model.index(0, 3).data(), "")
+        self.assertEqual(model.index(1, 0).data(), "")
+        self.assertEqual(model.index(1, 1).data(), "")
+        self.assertEqual(model.index(1, 2).data(), "")
+        self.assertEqual(model.index(1, 3).data(), "")
+
+
+if __name__ == "__main__":
     unittest.main()
