@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,99 +10,58 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Unit tests for the plotting module.
-
-:author: A. Soininen (VTT)
-:date:   10.7.2019
-"""
-
-import unittest
+"""Unit tests for the plotting module."""
 from contextlib import contextmanager
-from unittest.mock import Mock, MagicMock, patch
-
-import numpy
-from PySide6.QtCore import QModelIndex, QItemSelectionModel, QObject
-from PySide6.QtWidgets import QApplication, QMessageBox
+import unittest
+from unittest.mock import patch
 from matplotlib.gridspec import GridSpec
-
+import numpy
+from PySide6.QtCore import QItemSelectionModel, QModelIndex, QObject
+from PySide6.QtWidgets import QApplication
 from spinedb_api import (
+    Array,
     DateTime,
     Map,
+    TimePattern,
+    TimeSeriesFixedResolution,
     TimeSeriesVariableResolution,
     to_database,
-    TimeSeriesFixedResolution,
-    TimePattern,
-    Array,
 )
-from spinetoolbox.helpers import signal_waiter
 from spinetoolbox.plotting import (
-    PlottingError,
-    convert_indexed_value_to_tree,
-    TreeNode,
-    turn_node_to_xy_data,
-    XYData,
-    reduce_indexes,
-    combine_data_with_same_indexes,
-    plot_data,
-    raise_if_incompatible_x,
-    plot_pivot_table_selection,
     LEGEND_PLACEMENT_THRESHOLD,
+    IndexName,
+    PlottingError,
+    TreeNode,
+    XYData,
     add_row_to_exception,
+    combine_data_with_same_indexes,
+    convert_indexed_value_to_tree,
+    plot_data,
+    plot_pivot_table_selection,
+    raise_if_incompatible_x,
+    reduce_indexes,
+    turn_node_to_xy_data,
 )
-from spinetoolbox.spine_db_editor.widgets.spine_db_editor import SpineDBEditor
-from tests.mock_helpers import TestSpineDBManager
+from tests.mock_helpers import TestCaseWithQApplication
+from tests.spine_db_editor.helpers import TestBase
 
 
-class TestBase(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        if not QApplication.instance():
-            QApplication()
-
-    def setUp(self):
-        with patch("spinetoolbox.spine_db_editor.widgets.spine_db_editor.SpineDBEditor.restore_ui"), patch(
-            "spinetoolbox.spine_db_editor.widgets.spine_db_editor.SpineDBEditor.show"
-        ):
-            mock_settings = Mock()
-            mock_settings.value.side_effect = lambda *args, **kwargs: 0
-            self._db_mngr = TestSpineDBManager(mock_settings, None)
-            logger = MagicMock()
-            self._db_map = self._db_mngr.get_db_map("sqlite://", logger, codename="test database", create=True)
-            self._db_editor = SpineDBEditor(self._db_mngr, {"sqlite://": "test database"})
-
-    def tearDown(self):
-        with patch("spinetoolbox.spine_db_editor.widgets.spine_db_editor.QMessageBox") as message_box:
-            message_box.exec.return_value = QMessageBox.StandardButton.Ok
-            with signal_waiter(self._db_mngr.session_rolled_back) as waiter:
-                self._db_editor.rollback_session()
-                if message_box.exec.call_count > 0:
-                    waiter.wait()
-        with patch("spinetoolbox.spine_db_editor.widgets.spine_db_editor.SpineDBEditor.save_window_state"), patch(
-            "spinetoolbox.spine_db_manager.QMessageBox"
-        ):
-            self._db_editor.close()
-        while not self._db_map.connection.closed:
-            QApplication.processEvents()
-        self._db_mngr.clean_up()
-        self._db_editor.deleteLater()
-        self._db_editor = None
-
+class TestPlotPivotTableSelection(TestBase):
     def _add_object_parameter_values(self, values):
-        self._db_mngr.add_object_classes({self._db_map: [{"name": "class"}]})
-        self._db_mngr.add_parameter_definitions(
-            {self._db_map: [{"object_class_id": 1, "name": name} for name in values]}
+        self._db_mngr.add_items("entity_class", {self._db_map: [{"name": "class", "id": 1}]})
+        self._db_mngr.add_items(
+            "parameter_definition",
+            {self._db_map: [{"entity_class_id": 1, "name": name, "id": i + 1} for i, name in enumerate(values)]},
         )
         object_count = max(len(x) for x in values.values())
-        self._db_mngr.add_objects({self._db_map: [{"class_id": 1, "name": f"o{i + 1}"} for i in range(object_count)]})
-        db_values = {
-            name: [(value, type_) for value, type_ in map(to_database, value_list)]
-            for name, value_list in values.items()
-        }
+        self._db_mngr.add_items(
+            "entity", {self._db_map: [{"class_id": 1, "name": f"o{i + 1}", "id": i + 1} for i in range(object_count)]}
+        )
+        db_values = {name: list(map(to_database, value_list)) for name, value_list in values.items()}
         value_items = [
             {
-                "object_class_id": 1,
-                "object_id": (i + 1),
+                "entity_class_id": 1,
+                "entity_id": (i + 1),
                 "parameter_definition_id": param_i + 1,
                 "alternative_id": 1,
                 "type": type_,
@@ -110,24 +70,22 @@ class TestBase(unittest.TestCase):
             for param_i, values_and_types in enumerate(db_values.values())
             for i, (db_value, type_) in enumerate(values_and_types)
         ]
-        self._db_mngr.add_parameter_values({self._db_map: value_items})
+        self._db_mngr.add_items("parameter_value", {self._db_map: value_items})
 
     def _select_object_class_in_tree_view(self):
-        object_tree_model = self._db_editor.ui.treeView_object.model()
+        object_tree_model = self._db_editor.ui.treeView_entity.model()
         root_index = object_tree_model.index(0, 0)
         if object_tree_model.canFetchMore(root_index):
             object_tree_model.fetchMore(root_index)
         self.assertEqual(object_tree_model.rowCount(root_index), 1)
         class_index = object_tree_model.index(0, 0, root_index)
-        refreshing_models = list(self._db_editor._parameter_models) + list(self._db_editor._parameter_value_models)
-        with multi_signal_waiter([model.refreshed for model in refreshing_models]) as at_filter_refresh:
-            self._db_editor.ui.treeView_object.selectionModel().setCurrentIndex(
+        refreshing_models = [self._db_editor.parameter_value_model, self._db_editor.parameter_definition_model]
+        with multi_signal_waiter([model.layoutChanged for model in refreshing_models]) as at_filter_refresh:
+            self._db_editor.ui.treeView_entity.selectionModel().setCurrentIndex(
                 class_index, QItemSelectionModel.ClearAndSelect
             )
             at_filter_refresh.wait()
 
-
-class TestPlotPivotTableSelection(TestBase):
     def _fill_pivot(self, values):
         self._add_object_parameter_values(values)
         self.assertEqual(self._db_editor.current_input_type, self._db_editor._PARAMETER_VALUE)
@@ -154,7 +112,7 @@ class TestPlotPivotTableSelection(TestBase):
         selection = self._select_column(1, model)
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | floats")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | floats")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "alternative_name")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "floats")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -177,7 +135,7 @@ class TestPlotPivotTableSelection(TestBase):
         selection = self._select_column(1, model)
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | ints")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | ints")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "alternative_name")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "ints")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -203,7 +161,7 @@ class TestPlotPivotTableSelection(TestBase):
         selection = self._select_column(1, model)
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | series | Base")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | series | Base")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "t")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "series")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -232,11 +190,13 @@ class TestPlotPivotTableSelection(TestBase):
     def test_row_filtering(self):
         self._fill_pivot({"floats": [1.1, 1.2, 1.3]})
         model = self._db_editor.pivot_table_proxy
-        model.set_filter("class", {(self._db_map, 1), (self._db_map, 3)})
+        id1 = self._db_map.get_entity_item(id=1)["id"]
+        id3 = self._db_map.get_entity_item(id=3)["id"]
+        model.set_filter("class", {(self._db_map, id1), (self._db_map, id3)})
         selection = self._select_column(1, model)
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | floats")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | floats")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "alternative_name")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "floats")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -254,11 +214,12 @@ class TestPlotPivotTableSelection(TestBase):
     def test_column_filtering(self):
         self._fill_pivot({"floats": [1.1, 1.2, 1.3], "ints": [-3, -1, 2]})
         model = self._db_editor.pivot_table_proxy
-        model.set_filter("parameter", {(self._db_map, 2)})
+        p_id = self._db_map.get_parameter_definition_item(id=2)["id"]
+        model.set_filter("parameter", {(self._db_map, p_id)})
         selection = self._select_column(1, model)
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | ints")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | ints")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "alternative_name")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "ints")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -281,7 +242,7 @@ class TestPlotPivotTableSelection(TestBase):
         selected_indexes = [model.index(row, column) for column in range(1, 3) for row in range(2, 5)]
         plot_widget = plot_pivot_table_selection(model, selected_indexes)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "alternative_name")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "floats")
             self.assertTrue(plot_widget.canvas.has_twinned_axes())
@@ -295,14 +256,14 @@ class TestPlotPivotTableSelection(TestBase):
             )
             lines = plot_widget.canvas.axes.get_lines()
             self.assertEqual(len(lines), 3)
-            for i in range(3):
+            for _ in range(3):
                 self.assertEqual(list(lines[0].get_xdata(orig=True)), ["Base"])
             self.assertEqual(list(lines[0].get_ydata(orig=True)), [1.1])
             self.assertEqual(list(lines[1].get_ydata(orig=True)), [1.2])
             self.assertEqual(list(lines[2].get_ydata(orig=True)), [1.3])
             lines = twinned[0].get_lines()
             self.assertEqual(len(lines), 3)
-            for i in range(3):
+            for _ in range(3):
                 self.assertEqual(list(lines[0].get_xdata(orig=True)), ["Base"])
             self.assertEqual(list(lines[0].get_ydata(orig=True)), [-3.0])
             self.assertEqual(list(lines[1].get_ydata(orig=True)), [-1.0])
@@ -317,7 +278,7 @@ class TestPlotPivotTableSelection(TestBase):
         selection = self._select_column(1, model)
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | a-ints | Base")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | a-ints | Base")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "b-floats")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "a-ints")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -338,11 +299,12 @@ class TestPlotPivotTableSelection(TestBase):
         self._fill_pivot({"a-ints": [-3, -1, 2], "b-floats": [1.1, 1.2, 1.3]})
         model = self._db_editor.pivot_table_proxy
         model.sourceModel().set_plot_x_column(2, True)
-        model.set_filter("parameter", {(self._db_map, 1)})
+        p_id = self._db_map.get_parameter_definition_item(id=1)["id"]
+        model.set_filter("parameter", {(self._db_map, p_id)})
         selection = self._select_column(1, model)
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | a-ints")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | a-ints")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "alternative_name")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "a-ints")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -370,7 +332,7 @@ class TestPlotPivotTableSelection(TestBase):
         selection = [model.index(first_data_row + 1, 1)]
         plot_pivot_table_selection(model, selection, plot_widget)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | series | Base")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | series | Base")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "t")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "series")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -412,7 +374,7 @@ class TestPlotPivotTableSelection(TestBase):
         selection = [model.index(2, 1)]
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | maps | o1 | Base")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | maps | o1 | Base")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "x")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "maps")
             self.assertIsNone(plot_widget.canvas.legend_axes.get_legend())
@@ -442,7 +404,7 @@ class TestPlotPivotTableSelection(TestBase):
         selection = [model.index(2, 1)]
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | maps | o1 | Base")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | maps | o1 | Base")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "x")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "maps")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -502,7 +464,7 @@ class TestPlotPivotTableSelection(TestBase):
         selection = [model.index(2, 1)]
         plot_widget = plot_pivot_table_selection(model, selection)
         try:
-            self.assertEqual(plot_widget.canvas.axes.get_title(), "test database | maps | o1 | Base")
+            self.assertEqual(plot_widget.canvas.axes.get_title(), "TestPlotPivotTableSelection_db | maps | o1 | Base")
             self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "t")
             self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "maps")
             legend = plot_widget.canvas.legend_axes.get_legend()
@@ -608,7 +570,7 @@ class TestTurnNodesToXYData(unittest.TestCase):
         node = TreeNode("my_index")
         node.content = {1: 1.1, 2: 2.2}
         xy_data = list(turn_node_to_xy_data(node, None))
-        expected = [XYData([1, 2], [1.1, 2.2], "my_index", "", [], [])]
+        expected = [XYData([1, 2], [1.1, 2.2], IndexName("my_index", 0), "", [], [])]
         self.assertEqual(xy_data, expected)
 
     def test_one_index_deep_tree(self):
@@ -620,8 +582,8 @@ class TestTurnNodesToXYData(unittest.TestCase):
         root.content = {"a": node1, "b": node2}
         xy_data = list(turn_node_to_xy_data(root, None))
         expected = [
-            XYData([1, 2], [1.1, 2.2], "index_1", "", ["a"], ["root_index"]),
-            XYData([3, 4], [3.3, 4.4], "index_2", "", ["b"], ["root_index"]),
+            XYData([1, 2], [1.1, 2.2], IndexName("index_1", 1), "", ["a"], [IndexName("root_index", 0)]),
+            XYData([3, 4], [3.3, 4.4], IndexName("index_2", 1), "", ["b"], [IndexName("root_index", 0)]),
         ]
         self.assertEqual(xy_data, expected)
 
@@ -632,8 +594,8 @@ class TestTurnNodesToXYData(unittest.TestCase):
         root.content = {"a": node1, 3: 3.3, 4: 4.4}
         xy_data = list(turn_node_to_xy_data(root, None))
         expected = [
-            XYData([1, 2], [1.1, 2.2], "index_1", "", ["a"], ["root_index"]),
-            XYData([3, 4], [3.3, 4.4], "root_index", "", [], []),
+            XYData([1, 2], [1.1, 2.2], IndexName("index_1", 1), "", ["a"], [IndexName("root_index", 0)]),
+            XYData([3, 4], [3.3, 4.4], IndexName("root_index", 0), "", [], []),
         ]
         self.assertEqual(xy_data, expected)
 
@@ -652,56 +614,98 @@ class TestTurnNodesToXYData(unittest.TestCase):
         root.content = {"a": label_node1, "b": label_node2}
         xy_data = list(turn_node_to_xy_data(root, 1))
         expected = [
-            XYData([1, 2], [1.1, 2.2], "index_1", "to the top", ["a", "to the top"], ["root_index", "Y label 1"]),
-            XYData([3, 4], [3.3, 4.4], "index_2", "upwards", ["a", "upwards"], ["root_index", "Y label 1"]),
-            XYData([5, 6], [5.5, 6.6], "index_3", "ascent", ["b", "ascent"], ["root_index", "Y label 2"]),
+            XYData(
+                [1, 2],
+                [1.1, 2.2],
+                IndexName("index_1", 2),
+                "to the top",
+                ["a", "to the top"],
+                [IndexName("root_index", 0), IndexName("Y label 1", 1)],
+            ),
+            XYData(
+                [3, 4],
+                [3.3, 4.4],
+                IndexName("index_2", 2),
+                "upwards",
+                ["a", "upwards"],
+                [IndexName("root_index", 0), IndexName("Y label 1", 1)],
+            ),
+            XYData(
+                [5, 6],
+                [5.5, 6.6],
+                IndexName("index_3", 2),
+                "ascent",
+                ["b", "ascent"],
+                [IndexName("root_index", 0), IndexName("Y label 2", 1)],
+            ),
         ]
         self.assertEqual(xy_data, expected)
 
 
 class TestReduceIndexes(unittest.TestCase):
     def test_single_shallow_xy_data(self):
-        data = [XYData([1, 2], [1.1, 2.2], "my_index", "", [], [])]
+        data = [XYData([1, 2], [1.1, 2.2], IndexName("my_index", 0), "", [], [])]
         reduced_data, common_indexes = reduce_indexes(data)
-        expected = [XYData([1, 2], [1.1, 2.2], "my_index", "", [], [])]
+        expected = [XYData([1, 2], [1.1, 2.2], IndexName("my_index", 0), "", [], [])]
         self.assertEqual(reduced_data, expected)
         self.assertEqual(common_indexes, [])
 
     def test_all_indexes_shared(self):
         data = [
-            XYData([1, 2], [1.1, 2.2], "x_index_1", "", ["my_index"], ["index name"]),
-            XYData([3, 4], [3.3, 4.4], "x_index_2", "", ["my_index"], ["index name"]),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index_1", 1), "", ["my_index"], [IndexName("index name", 0)]),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index_2", 1), "", ["my_index"], [IndexName("index name", 0)]),
         ]
         reduced_data, common_indexes = reduce_indexes(data)
         expected = [
-            XYData([1, 2], [1.1, 2.2], "x_index_1", "", [], []),
-            XYData([3, 4], [3.3, 4.4], "x_index_2", "", [], []),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index_1", 1), "", [], []),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index_2", 1), "", [], []),
         ]
         self.assertEqual(reduced_data, expected)
         self.assertEqual(common_indexes, ["my_index"])
 
     def test_uneven_depth(self):
         data = [
-            XYData([1, 2], [1.1, 2.2], "x_index_1", "", ["shared_1"], ["shared index"]),
-            XYData([3, 4], [3.3, 4.4], "x_index_2", "", ["shared_1", "extra"], ["shared index", "goes deeper"]),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index_1", 1), "", ["shared_1"], [IndexName("shared index", 0)]),
+            XYData(
+                [3, 4],
+                [3.3, 4.4],
+                IndexName("x_index_2", 2),
+                "",
+                ["shared_1", "extra"],
+                [IndexName("shared index", 0), IndexName("goes deeper", 1)],
+            ),
         ]
         reduced_data, common_indexes = reduce_indexes(data)
         expected = [
-            XYData([1, 2], [1.1, 2.2], "x_index_1", "", [], []),
-            XYData([3, 4], [3.3, 4.4], "x_index_2", "", ["extra"], ["goes deeper"]),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index_1", 1), "", [], []),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index_2", 2), "", ["extra"], [IndexName("goes deeper", 1)]),
         ]
         self.assertEqual(reduced_data, expected)
         self.assertEqual(common_indexes, ["shared_1"])
 
     def test_first_index_not_shared(self):
         data = [
-            XYData([1, 2], [1.1, 2.2], "x_index_1", "", ["different_1", "shared"], ["my_index_1", "shared index"]),
-            XYData([3, 4], [3.3, 4.4], "x_index_2", "", ["different_2", "shared"], ["my_index_2", "shared index"]),
+            XYData(
+                [1, 2],
+                [1.1, 2.2],
+                IndexName("x_index_1", 2),
+                "",
+                ["different_1", "shared"],
+                [IndexName("my_index_1", 0), IndexName("shared index", 1)],
+            ),
+            XYData(
+                [3, 4],
+                [3.3, 4.4],
+                IndexName("x_index_2", 2),
+                "",
+                ["different_2", "shared"],
+                [IndexName("my_index_2", 0), IndexName("shared index", 1)],
+            ),
         ]
         reduced_data, common_indexes = reduce_indexes(data)
         expected = [
-            XYData([1, 2], [1.1, 2.2], "x_index_1", "", ["different_1"], ["my_index_1"]),
-            XYData([3, 4], [3.3, 4.4], "x_index_2", "", ["different_2"], ["my_index_2"]),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index_1", 2), "", ["different_1"], [IndexName("my_index_1", 0)]),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index_2", 2), "", ["different_2"], [IndexName("my_index_2", 0)]),
         ]
         self.assertEqual(reduced_data, expected)
         self.assertEqual(common_indexes, ["shared"])
@@ -710,60 +714,67 @@ class TestReduceIndexes(unittest.TestCase):
 class TestCombineDataWithSameIndexes(unittest.TestCase):
     def test_not_combined_due_to_different_x_labels(self):
         data = [
-            XYData([1, 2], [1.1, 2.2], "x_index_1", "", [], []),
-            XYData([3, 4], [3.3, 4.4], "x_index_2", "", [], []),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index_1", 0), "", [], []),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index_2", 0), "", [], []),
         ]
         combined = combine_data_with_same_indexes(data)
         expected = [
-            XYData([1, 2], [1.1, 2.2], "x_index_1", "", [], []),
-            XYData([3, 4], [3.3, 4.4], "x_index_2", "", [], []),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index_1", 0), "", [], []),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index_2", 0), "", [], []),
         ]
         self.assertEqual(combined, expected)
 
     def test_not_combined_due_to_different_indexes(self):
         data = [
-            XYData([1, 2], [1.1, 2.2], "x_index", "", ["index_1"], ["index name"]),
-            XYData([3, 4], [3.3, 4.4], "x_index", "", ["index_2"], ["index name"]),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index", 1), "", ["index_1"], [IndexName("index name", 0)]),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index", 1), "", ["index_2"], [IndexName("index name", 0)]),
         ]
         combined = combine_data_with_same_indexes(data)
         expected = [
-            XYData([1, 2], [1.1, 2.2], "x_index", "", ["index_1"], ["index name"]),
-            XYData([3, 4], [3.3, 4.4], "x_index", "", ["index_2"], ["index name"]),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index", 1), "", ["index_1"], [IndexName("index name", 0)]),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index", 1), "", ["index_2"], [IndexName("index name", 0)]),
         ]
         self.assertEqual(combined, expected)
 
     def test_same_x_axes_combined(self):
-        data = [XYData([1, 2], [1.1, 2.2], "x_index", "", [], []), XYData([3, 4], [3.3, 4.4], "x_index", "", [], [])]
+        data = [
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index", 0), "", [], []),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index", 0), "", [], []),
+        ]
         combined = combine_data_with_same_indexes(data)
-        expected = [XYData([1, 2, 3, 4], [1.1, 2.2, 3.3, 4.4], "x_index", "", [], [])]
+        expected = [XYData([1, 2, 3, 4], [1.1, 2.2, 3.3, 4.4], IndexName("x_index", 0), "", [], [])]
         self.assertEqual(combined, expected)
 
     def test_all_same_indexes_combined(self):
         data = [
-            XYData([1, 2], [1.1, 2.2], "x_index", "", ["index_1"], ["index name"]),
-            XYData([3, 4], [3.3, 4.4], "x_index", "", ["index_1"], ["index name"]),
+            XYData([1, 2], [1.1, 2.2], IndexName("x_index", 1), "", ["index_1"], [IndexName("index name", 0)]),
+            XYData([3, 4], [3.3, 4.4], IndexName("x_index", 1), "", ["index_1"], [IndexName("index name", 0)]),
         ]
         combined = combine_data_with_same_indexes(data)
-        expected = [XYData([1, 2, 3, 4], [1.1, 2.2, 3.3, 4.4], "x_index", "", ["index_1"], ["index name"])]
+        expected = [
+            XYData(
+                [1, 2, 3, 4],
+                [1.1, 2.2, 3.3, 4.4],
+                IndexName("x_index", 1),
+                "",
+                ["index_1"],
+                [IndexName("index name", 0)],
+            )
+        ]
         self.assertEqual(combined, expected)
 
 
-class TestPlotData(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        if not QApplication.instance():
-            QApplication()
-
+class TestPlotData(TestCaseWithQApplication):
     def test_nothing_to_plot(self):
         plot_widget = plot_data([])
         self.assertEqual(len(plot_widget.canvas.axes.lines), 0)
 
     def test_single_plot(self):
-        data = [XYData([-11, -22], [1.1, 2.2], "x_index", "y", ["index_1"], ["index name"])]
+        data = [XYData([-11, -22], [1.1, 2.2], IndexName("x_index", 1), "y", ["index_1"], [IndexName("index name", 0)])]
         plot_widget = plot_data(data)
         lines = plot_widget.canvas.axes.lines
         self.assertEqual(plot_widget.canvas.axes.get_title(), "index_1")
-        self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "x_index")
+        self.assertEqual(plot_widget.canvas.axes.get_xlabel(), "x_index", 1)
         self.assertEqual(plot_widget.canvas.axes.get_ylabel(), "y")
         self.assertIsNone(plot_widget.canvas.legend_axes.get_legend())
         self.assertEqual(len(lines), 1)
@@ -771,7 +782,7 @@ class TestPlotData(unittest.TestCase):
         self.assertEqual(list(lines[0].get_ydata(orig=True)), [1.1, 2.2])
 
     def test_two_plots_with_shared_and_individual_indexes(self):
-        data = [XYData([-11, -22], [1.1, 2.2], "x_index", "y", ["index_1"], ["index name"])]
+        data = [XYData([-11, -22], [1.1, 2.2], IndexName("x_index", 1), "y", ["index_1"], [IndexName("index name", 0)])]
         plot_widget = plot_data(data)
         lines = plot_widget.canvas.axes.lines
         self.assertEqual(plot_widget.canvas.axes.get_title(), "index_1")
@@ -787,18 +798,18 @@ class TestPlotData(unittest.TestCase):
             XYData(
                 [numpy.datetime64("2022-11-18T16:00:00"), numpy.datetime64("2022-11-18T17:00:00")],
                 [1.1, 2.2],
-                "time",
+                IndexName("time", 1),
                 "y",
                 ["index_1"],
-                ["index name"],
+                [IndexName("index name", 0)],
             ),
             XYData(
                 [numpy.datetime64("2022-11-18T16:00:00"), numpy.datetime64("2022-11-18T17:00:00")],
                 [3.3, 4.4],
-                "time",
+                IndexName("time", 1),
                 "y",
                 ["index_2"],
-                ["index name"],
+                [IndexName("index name", 0)],
             ),
         ]
         plot_widget = plot_data(data)
@@ -819,8 +830,17 @@ class TestPlotData(unittest.TestCase):
 
     def test_we_find_unsqueezed_index_no_matter_what(self):
         data = [
-            XYData(x=["t1", "t2"], y=[13.0, 7.0], x_label="x", y_label="y", data_index=["A1"], index_names=["idx"]),
-            XYData(x=["B1", "B2"], y=[-13.0, -7.0], x_label="x", y_label="y", data_index=[], index_names=[]),
+            XYData(
+                x=["t1", "t2"],
+                y=[13.0, 7.0],
+                x_label=IndexName("x", 1),
+                y_label="y",
+                data_index=["A1"],
+                index_names=[IndexName("idx", 0)],
+            ),
+            XYData(
+                x=["B1", "B2"], y=[-13.0, -7.0], x_label=IndexName("x", 1), y_label="y", data_index=[], index_names=[]
+            ),
         ]
         plot_widget = plot_data(data)
         self.assertEqual(plot_widget.canvas.axes.get_title(), "")
@@ -838,16 +858,23 @@ class TestPlotData(unittest.TestCase):
 
     def test_y_axis_is_not_labeled_when_more_than_two_labels_are_possible(self):
         data = [
-            XYData(x=["t1", "t2"], y=[1.1, 2.2], x_label="x", y_label="y", data_index=["A1"], index_names=["idx"]),
+            XYData(
+                x=["t1", "t2"],
+                y=[1.1, 2.2],
+                x_label=IndexName("x", 1),
+                y_label="y",
+                data_index=["A1"],
+                index_names=[IndexName("idx", 0)],
+            ),
             XYData(
                 x=["t1", "t2"],
                 y=[3.3, 4.4],
-                x_label="x",
+                x_label=IndexName("x", 1),
                 y_label="z",
                 data_index=["B1", "b1"],
-                index_names=["jdx", "kdx"],
+                index_names=[IndexName("jdx", 0), IndexName("kdx", 1)],
             ),
-            XYData(x=["t1", "t2"], y=[5.5, 6.6], x_label="x", y_label="a", data_index=[], index_names=[]),
+            XYData(x=["t1", "t2"], y=[5.5, 6.6], x_label=IndexName("x", 0), y_label="a", data_index=[], index_names=[]),
         ]
         plot_widget = plot_data(data)
         self.assertFalse(plot_widget.canvas.has_twinned_axes())
@@ -868,7 +895,7 @@ class TestPlotData(unittest.TestCase):
 
     def test_legend_placement_below_threshold(self):
         data = [
-            XYData(x=["x"], y=[1.0], x_label="x", y_label="", data_index=[], index_names=[])
+            XYData(x=["x"], y=[1.0], x_label=IndexName("x", 0), y_label="", data_index=[], index_names=[])
             for _ in range(LEGEND_PLACEMENT_THRESHOLD - 1)
         ]
         plot_widget = plot_data(data)
@@ -878,7 +905,7 @@ class TestPlotData(unittest.TestCase):
 
     def test_legend_placement_above_threshold(self):
         data = [
-            XYData(x=["x"], y=[1.0], x_label="x", y_label="", data_index=[], index_names=[])
+            XYData(x=["x"], y=[1.0], x_label=IndexName("x", 0), y_label="", data_index=[], index_names=[])
             for _ in range(LEGEND_PLACEMENT_THRESHOLD)
         ]
         plot_widget = plot_data(data)
@@ -891,18 +918,18 @@ class TestRaiseIfIncompatibleX(unittest.TestCase):
             XYData(
                 x=[1.0, 2.0, 3.0],
                 y=[5.0, 2.0, -1.0],
-                x_label='x',
-                y_label='',
-                data_index=['1d_map'],
-                index_names=['parameter_name'],
+                x_label=IndexName("x", 0),
+                y_label="",
+                data_index=["1d_map"],
+                index_names=[IndexName("parameter_name", 0)],
             ),
             XYData(
-                x=['t1', 't2'],
+                x=["t1", "t2"],
                 y=[13.0, 7.0],
-                x_label='x',
-                y_label='',
-                data_index=['uneven_map', 'A1'],
-                index_names=['parameter_name', 'x'],
+                x_label=IndexName("x", 2),
+                y_label="",
+                data_index=["uneven_map", "A1"],
+                index_names=[IndexName("parameter_name", 0), IndexName("x", 1)],
             ),
         ]
         self.assertRaises(PlottingError, raise_if_incompatible_x, data_list)
@@ -955,5 +982,5 @@ def multi_signal_waiter(signals):
         waiter.deleteLater()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
