@@ -1,5 +1,6 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,36 +10,20 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Unit tests for MapTableView class.
-
-:author: A. Soininen (VTT)
-:date:   25.3.2021
-"""
-
+"""Unit tests for MapTableView class."""
 import csv
-import locale
 from io import StringIO
+import locale
 import unittest
-from PySide2.QtCore import QItemSelectionModel
-from PySide2.QtWidgets import QApplication
+from PySide6.QtCore import QItemSelectionModel
+from PySide6.QtWidgets import QApplication
 from spinedb_api import Map
 from spinetoolbox.mvcmodels.map_model import MapModel
 from spinetoolbox.widgets.custom_qtableview import MapTableView, system_lc_numeric
+from tests.mock_helpers import TestCaseWithQApplication, mock_clipboard_patch
 
 
-class TestMapTableView(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        if not QApplication.instance():
-            QApplication()
-
-    def setUp(self):
-        self._original_clip = QApplication.clipboard().text()
-
-    def tearDown(self):
-        QApplication.clipboard().setText(self._original_clip)
-
+class TestMapTableView(TestCaseWithQApplication):
     def test_copy_without_selection_returns_false(self):
         table_view = MapTableView()
         model = MapModel(Map([], [], float), table_view)
@@ -54,7 +39,21 @@ class TestMapTableView(unittest.TestCase):
             table_view.selectionModel().select(model.index(0, column), QItemSelectionModel.Select)
         self.assertTrue(table_view.copy())
         clip = StringIO(QApplication.clipboard().text())
-        table = [row for row in csv.reader(clip, delimiter="\t")]
+        table = list(csv.reader(clip, delimiter="\t"))
+        with system_lc_numeric():
+            self.assertEqual(table, [["A", locale.str(2.3)]])
+        table_view.deleteLater()
+
+    def test_copy_selection_does_not_copy_expanse_row_or_column(self):
+        table_view = MapTableView()
+        model = MapModel(Map(["A"], [2.3]), table_view)
+        table_view.setModel(model)
+        for column in range(model.columnCount()):
+            for row in range(model.rowCount()):
+                table_view.selectionModel().select(model.index(row, column), QItemSelectionModel.Select)
+        self.assertTrue(table_view.copy())
+        clip = StringIO(QApplication.clipboard().text())
+        table = list(csv.reader(clip, delimiter="\t"))
         with system_lc_numeric():
             self.assertEqual(table, [["A", locale.str(2.3)]])
         table_view.deleteLater()
@@ -63,7 +62,8 @@ class TestMapTableView(unittest.TestCase):
         table_view = MapTableView()
         model = MapModel(Map(["A"], [2.3]), table_view)
         table_view.setModel(model)
-        self.assertFalse(table_view.paste())
+        with mock_clipboard_patch("won't be pasted", "spinetoolbox.widgets.custom_qtableview.QApplication.clipboard"):
+            self.assertFalse(table_view.paste())
         table_view.deleteLater()
 
     def test_paste_to_empty_table(self):
@@ -71,8 +71,7 @@ class TestMapTableView(unittest.TestCase):
         model = MapModel(Map([], [], str), table_view)
         table_view.setModel(model)
         table_view.selectionModel().select(model.index(0, 0), QItemSelectionModel.Select)
-        self._write_to_clipboard([["A", 2.3]])
-        self.assertTrue(table_view.paste())
+        self._paste_from_clipboard([["A", 2.3]], table_view)
         self.assertEqual(model.rowCount(), 2)
         self.assertEqual(model.columnCount(), 3)
         self.assertEqual(model.value(), Map(["A"], [2.3]))
@@ -83,8 +82,7 @@ class TestMapTableView(unittest.TestCase):
         model = MapModel(Map(["A"], [2.3]), table_view)
         table_view.setModel(model)
         table_view.selectionModel().select(model.index(0, 0), QItemSelectionModel.Select)
-        self._write_to_clipboard([["V", -5.5], ["W", -6.6]])
-        self.assertTrue(table_view.paste())
+        self._paste_from_clipboard([["V", -5.5], ["W", -6.6]], table_view)
         self.assertEqual(model.rowCount(), 3)
         self.assertEqual(model.columnCount(), 3)
         self.assertEqual(model.value(), Map(["V", "W"], [-5.5, -6.6]))
@@ -96,21 +94,19 @@ class TestMapTableView(unittest.TestCase):
         table_view.setModel(model)
         for row in (0, 1):
             table_view.selectionModel().select(model.index(row, 0), QItemSelectionModel.Select)
-        self._write_to_clipboard([["Q", -4.4], ["V", -5.5], ["W", -6.6]])
-        self.assertTrue(table_view.paste())
+        self._paste_from_clipboard([["Q", -4.4], ["V", -5.5], ["W", -6.6]], table_view)
         self.assertEqual(model.rowCount(), 4)
         self.assertEqual(model.columnCount(), 3)
-        m = model.value()
         self.assertEqual(model.value(), Map(["Q", "V", "C"], [2.3, 3.2, 4.3]))
         table_view.deleteLater()
 
-    @staticmethod
-    def _write_to_clipboard(data):
+    def _paste_from_clipboard(self, data, table_view):
         with StringIO() as out_string:
             writer = csv.writer(out_string, delimiter="\t")
             writer.writerows(data)
             clip = out_string.getvalue()
-        QApplication.clipboard().setText(clip)
+        with mock_clipboard_patch(clip, "spinetoolbox.widgets.custom_qtableview.QApplication.clipboard"):
+            self.assertTrue(table_view.paste())
 
 
 if __name__ == "__main__":
