@@ -1,5 +1,6 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,39 +10,15 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Unit tests for the custom QTextBrowser.
-
-:author: A. Soininen (VTT)
-:date:   7.6.2019
-"""
-
-import logging
+"""Unit tests for the custom QTextBrowser."""
 import unittest
-import sys
-from PySide2.QtWidgets import QApplication
 from spinetoolbox.widgets.custom_qtextbrowser import CustomQTextBrowser
-
-# XXX: This modules just tests QTextBrowser.maximumBlockCount, so maybe we can remove it?
+from tests.mock_helpers import TestCaseWithQApplication
 
 
-class TestCustomQTextBrowser(unittest.TestCase):
+class TestCustomQTextBrowser(TestCaseWithQApplication):
     """Tests the CustomQTextBrowser class."""
 
-    @classmethod
-    def setUpClass(cls):
-        """Overridden method. Runs once before all tests in this class."""
-        try:
-            cls.app = QApplication().processEvents()
-        except RuntimeError:
-            pass
-        logging.basicConfig(
-            stream=sys.stderr,
-            level=logging.DEBUG,
-            format='%(asctime)s %(levelname)s: %(message)s',
-            datefmt='%Y-%m-%d %H:%M:%S',
-        )
-
     def test_default_max_blocks(self):
         browser = CustomQTextBrowser(None)
         self.assertEqual(browser.document().maximumBlockCount(), 2000)
@@ -51,17 +28,17 @@ class TestCustomQTextBrowser(unittest.TestCase):
         self.assertEqual(browser.document().blockCount(), 1)
         browser.document().setMaximumBlockCount(5)
         for _ in range(5):
-            browser.append('test text')
+            browser.append("test text")
         self.assertEqual(browser.document().blockCount(), 5)
         for _ in range(5):
-            browser.append('new text')
+            browser.append("new text")
         self.assertEqual(browser.document().blockCount(), 5)
 
     def test_extra_blocks_removed_from_start(self):
         browser = CustomQTextBrowser(None)
         self.assertEqual(browser.document().blockCount(), 1)
         browser.document().setMaximumBlockCount(3)
-        texts = ['1', '2', '3', '4', '5']
+        texts = ["1", "2", "3", "4", "5"]
         for t in texts:
             browser.append(t)
         self.assertEqual(browser.document().blockCount(), 3)
@@ -71,5 +48,5 @@ class TestCustomQTextBrowser(unittest.TestCase):
             text_block = text_block.next()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
