@@ -1,5 +1,6 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Toolbox contributors
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,16 +10,10 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Unit tests for the indexed_value_table_context_menu module.
-
-:author: A. Soininen (VTT)
-:date:   5.7.2019
-"""
-
+"""Unit tests for the indexed_value_table_context_menu module."""
 import unittest
 from unittest.mock import MagicMock
-from PySide2.QtWidgets import QApplication
+from PySide6.QtWidgets import QApplication
 from spinedb_api import Array, Map, TimePattern, TimeSeriesFixedResolution, TimeSeriesVariableResolution
 from spinetoolbox.widgets.array_editor import ArrayEditor
 from spinetoolbox.widgets.indexed_value_table_context_menu import (
@@ -30,14 +25,10 @@ from spinetoolbox.widgets.map_editor import MapEditor
 from spinetoolbox.widgets.time_pattern_editor import TimePatternEditor
 from spinetoolbox.widgets.time_series_fixed_resolution_editor import TimeSeriesFixedResolutionEditor
 from spinetoolbox.widgets.time_series_variable_resolution_editor import TimeSeriesVariableResolutionEditor
+from tests.mock_helpers import TestCaseWithQApplication
 
 
-class TestArrayTableContextMenu(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        if not QApplication.instance():
-            QApplication()
-
+class TestArrayTableContextMenu(TestCaseWithQApplication):
     def test_insert_row_after(self):
         editor = ArrayEditor()
         editor.set_value(Array([-1.0]))
@@ -50,8 +41,8 @@ class TestArrayTableContextMenu(unittest.TestCase):
         self.assertIsNotNone(insert_action)
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
-        self.assertEqual(model.index(0, 1).data(), -1.0)
-        self.assertEqual(model.index(1, 1).data(), 0.0)
+        self.assertEqual(model.index(0, 1).data(), str(-1.0))
+        self.assertEqual(model.index(1, 1).data(), str(0.0))
         editor.deleteLater()
 
     def test_insert_row_before(self):
@@ -66,8 +57,8 @@ class TestArrayTableContextMenu(unittest.TestCase):
         self.assertIsNotNone(insert_action)
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
-        self.assertEqual(model.index(0, 1).data(), 0.0)
-        self.assertEqual(model.index(1, 1).data(), -1.0)
+        self.assertEqual(model.index(0, 1).data(), str(0.0))
+        self.assertEqual(model.index(1, 1).data(), str(-1.0))
         editor.deleteLater()
 
     def test_remove_rows(self):
@@ -82,7 +73,7 @@ class TestArrayTableContextMenu(unittest.TestCase):
         self.assertIsNotNone(remove_action)
         remove_action.trigger()
         self.assertEqual(model.rowCount(), 1 + 1)
-        self.assertEqual(model.index(0, 1).data(), -2.0)
+        self.assertEqual(model.index(0, 1).data(), str(-2.0))
         editor.deleteLater()
 
     def test_show_value_editor(self):
@@ -100,12 +91,7 @@ class TestArrayTableContextMenu(unittest.TestCase):
         editor.deleteLater()
 
 
-class TestIndexedValueTableContextMenu(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        if not QApplication.instance():
-            QApplication()
-
+class TestIndexedValueTableContextMenu(TestCaseWithQApplication):
     def test_insert_row_after_with_time_pattern_editor(self):
         editor = TimePatternEditor()
         editor.set_value(TimePattern(["D1-2"], [-1.1]))
@@ -119,9 +105,9 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
         self.assertEqual(model.index(0, 0).data(), "D1-2")
-        self.assertEqual(model.index(0, 1).data(), -1.1)
+        self.assertEqual(model.index(0, 1).data(), str(-1.1))
         self.assertEqual(model.index(1, 0).data(), "")
-        self.assertEqual(model.index(1, 1).data(), 0.0)
+        self.assertEqual(model.index(1, 1).data(), str(0.0))
         editor.deleteLater()
 
     def test_insert_row_after_with_time_series_fixed_resolution_editor(self):
@@ -137,9 +123,9 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
         self.assertEqual(model.index(0, 0).data(), "2020-11-11T14:25:00")
-        self.assertEqual(model.index(0, 1).data(), -1.1)
+        self.assertEqual(model.index(0, 1).data(), str(-1.1))
         self.assertEqual(model.index(1, 0).data(), "2020-11-12T14:25:00")
-        self.assertEqual(model.index(1, 1).data(), 0.0)
+        self.assertEqual(model.index(1, 1).data(), str(0.0))
         editor.deleteLater()
 
     def test_insert_row_after_with_time_series_variable_resolution_editor(self):
@@ -155,9 +141,9 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
         self.assertEqual(model.index(0, 0).data(), "2020-11-11T14:25:00")
-        self.assertEqual(model.index(0, 1).data(), -1.1)
+        self.assertEqual(model.index(0, 1).data(), str(-1.1))
         self.assertEqual(model.index(1, 0).data(), "2020-11-11T15:25:00")
-        self.assertEqual(model.index(1, 1).data(), 0.0)
+        self.assertEqual(model.index(1, 1).data(), str(0.0))
         editor.deleteLater()
 
     def test_insert_row_before_with_time_pattern_editor(self):
@@ -173,9 +159,9 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
         self.assertEqual(model.index(0, 0).data(), "")
-        self.assertEqual(model.index(0, 1).data(), 0.0)
+        self.assertEqual(model.index(0, 1).data(), str(0.0))
         self.assertEqual(model.index(1, 0).data(), "D1-2")
-        self.assertEqual(model.index(1, 1).data(), -1.1)
+        self.assertEqual(model.index(1, 1).data(), str(-1.1))
         editor.deleteLater()
 
     def test_insert_row_before_with_time_series_fixed_resolution_editor(self):
@@ -191,9 +177,9 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
         self.assertEqual(model.index(0, 0).data(), "2020-11-11T14:25:00")
-        self.assertEqual(model.index(0, 1).data(), 0.0)
+        self.assertEqual(model.index(0, 1).data(), str(0.0))
         self.assertEqual(model.index(1, 0).data(), "2020-11-12T14:25:00")
-        self.assertEqual(model.index(1, 1).data(), -1.1)
+        self.assertEqual(model.index(1, 1).data(), str(-1.1))
         editor.deleteLater()
 
     def test_insert_row_before_with_time_series_variable_resolution_editor(self):
@@ -209,9 +195,9 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
         self.assertEqual(model.index(0, 0).data(), "2020-11-11T13:25:00")
-        self.assertEqual(model.index(0, 1).data(), 0.0)
+        self.assertEqual(model.index(0, 1).data(), str(0.0))
         self.assertEqual(model.index(1, 0).data(), "2020-11-11T14:25:00")
-        self.assertEqual(model.index(1, 1).data(), -1.1)
+        self.assertEqual(model.index(1, 1).data(), str(-1.1))
         editor.deleteLater()
 
     def test_remove_rows_with_time_pattern_editor(self):
@@ -227,7 +213,7 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         remove_action.trigger()
         self.assertEqual(model.rowCount(), 1 + 1)
         self.assertEqual(model.index(0, 0).data(), "D3-4")
-        self.assertEqual(model.index(0, 1).data(), -2.2)
+        self.assertEqual(model.index(0, 1).data(), str(-2.2))
         editor.deleteLater()
 
     def test_remove_rows_with_time_series_fixed_resolution_editor(self):
@@ -243,7 +229,7 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         remove_action.trigger()
         self.assertEqual(model.rowCount(), 1 + 1)
         self.assertEqual(model.index(0, 0).data(), "2020-11-11T14:25:00")
-        self.assertEqual(model.index(0, 1).data(), -2.2)
+        self.assertEqual(model.index(0, 1).data(), str(-2.2))
         editor.deleteLater()
 
     def test_remove_rows_with_time_series_variable_resolution_editor(self):
@@ -261,16 +247,11 @@ class TestIndexedValueTableContextMenu(unittest.TestCase):
         remove_action.trigger()
         self.assertEqual(model.rowCount(), 1 + 1)
         self.assertEqual(model.index(0, 0).data(), "2020-11-12T14:25:00")
-        self.assertEqual(model.index(0, 1).data(), -2.2)
+        self.assertEqual(model.index(0, 1).data(), str(-2.2))
         editor.deleteLater()
 
 
-class TestMapTableContextMenu(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        if not QApplication.instance():
-            QApplication()
-
+class TestMapTableContextMenu(TestCaseWithQApplication):
     def test_insert_column_after(self):
         editor = MapEditor()
         editor.set_value(Map(["a"], [-1.0]))
@@ -285,7 +266,7 @@ class TestMapTableContextMenu(unittest.TestCase):
         self.assertEqual(model.columnCount(), 3 + 1)
         self.assertEqual(model.index(0, 0).data(), "a")
         self.assertEqual(model.index(0, 1).data(), "")
-        self.assertEqual(model.index(0, 2).data(), -1.0)
+        self.assertEqual(model.index(0, 2).data(), str(-1.0))
         editor.deleteLater()
 
     def test_insert_column_before(self):
@@ -302,7 +283,7 @@ class TestMapTableContextMenu(unittest.TestCase):
         self.assertEqual(model.columnCount(), 3 + 1)
         self.assertEqual(model.index(0, 0).data(), "")
         self.assertEqual(model.index(0, 1).data(), "a")
-        self.assertEqual(model.index(0, 2).data(), -1.0)
+        self.assertEqual(model.index(0, 2).data(), str(-1.0))
         editor.deleteLater()
 
     def test_insert_row_after(self):
@@ -318,9 +299,9 @@ class TestMapTableContextMenu(unittest.TestCase):
         insert_action.trigger()
         self.assertEqual(model.rowCount(), 2 + 1)
         self.assertEqual(model.index(0, 0).data(), "a")
-        self.assertEqual(model.index(0, 1).data(), -1.0)
+        self.assertEqual(model.index(0, 1).data(), str(-1.0))
         self.assertEqual(model.index(1, 0).data(), "a")
-        self.assertEqual(model.index(1, 1).data(), -1.0)
+        self.assertEqual(model.index(1, 1).data(), str(-1.0))
         editor.deleteLater()
 
     def test_insert_row_before(self):
@@ -338,7 +319,7 @@ class TestMapTableContextMenu(unittest.TestCase):
         self.assertEqual(model.index(0, 0).data(), "")
         self.assertEqual(model.index(0, 1).data(), "")
         self.assertEqual(model.index(1, 0).data(), "a")
-        self.assertEqual(model.index(1, 1).data(), -1.0)
+        self.assertEqual(model.index(1, 1).data(), str(-1.0))
         editor.deleteLater()
 
     def test_remove_columns(self):
@@ -353,7 +334,7 @@ class TestMapTableContextMenu(unittest.TestCase):
         self.assertIsNotNone(insert_action)
         insert_action.trigger()
         self.assertEqual(model.columnCount(), 1 + 1)
-        self.assertEqual(model.index(0, 0).data(), -1.0)
+        self.assertEqual(model.index(0, 0).data(), str(-1.0))
         editor.deleteLater()
 
     def test_remove_rows(self):
@@ -393,5 +374,5 @@ def _find_action(menu, text):
     return None
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
